@@ -78,17 +78,15 @@ my_bool opt_ibx_galera_info = FALSE;
 my_bool opt_ibx_slave_info = FALSE;
 my_bool opt_ibx_no_lock = FALSE;
 my_bool opt_ibx_safe_slave_backup = FALSE;
-my_bool opt_ibx_rsync = FALSE;
 my_bool opt_ibx_force_non_empty_dirs = FALSE;
 my_bool opt_ibx_noversioncheck = FALSE;
-my_bool opt_ibx_no_backup_locks = FALSE;
 my_bool opt_ibx_decompress = FALSE;
 
 char *opt_ibx_incremental_history_name = NULL;
 char *opt_ibx_incremental_history_uuid = NULL;
 
 char *opt_ibx_user = NULL;
-char *opt_ibx_password = NULL;
+const char *opt_ibx_password = NULL;
 char *opt_ibx_host = NULL;
 char *opt_ibx_defaults_group = NULL;
 char *opt_ibx_socket = NULL;
@@ -208,14 +206,15 @@ enum innobackupex_options
 	OPT_STREAM,
 	OPT_TABLES_FILE,
 	OPT_THROTTLE,
-	OPT_USE_MEMORY
+	OPT_USE_MEMORY,
+	OPT_INNODB_FORCE_RECOVERY,
 };
 
 ibx_mode_t ibx_mode = IBX_MODE_BACKUP;
 
 static struct my_option ibx_long_options[] =
 {
-	{"version", 'v', "print xtrabackup version information",
+	{"version", 'v', "print version information",
 	 (uchar *) &opt_ibx_version, (uchar *) &opt_ibx_version, 0,
 	 GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0},
 
@@ -241,7 +240,7 @@ static struct my_option ibx_long_options[] =
 	 GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0},
 
 	{"galera-info", OPT_GALERA_INFO, "This options creates the "
-	 "xtrabackup_galera_info file which contains the local node state at "
+	 MB_GALERA_INFO " file which contains the local node state at "
 	 "the time of the backup. Option should be used when performing the "
 	 "backup of MariaDB Galera Cluster. Has no effect when backup locks "
 	 "are used to create the backup.",
@@ -251,25 +250,26 @@ static struct my_option ibx_long_options[] =
 	{"slave-info", OPT_SLAVE_INFO, "This option is useful when backing "
 	 "up a replication slave server. It prints the binary log position "
 	 "and name of the master server. It also writes this information to "
-	 "the \"xtrabackup_slave_info\" file as a \"CHANGE MASTER\" command. "
+	 "the \"" MB_SLAVE_INFO "\" file as a \"CHANGE MASTER\" command. "
 	 "A new slave for this master can be set up by starting a slave server "
 	 "on this backup and issuing a \"CHANGE MASTER\" command with the "
-	 "binary log position saved in the \"xtrabackup_slave_info\" file.",
+	 "binary log position saved in the \"" MB_SLAVE_INFO "\" file.",
 	 (uchar *) &opt_ibx_slave_info, (uchar *) &opt_ibx_slave_info, 0,
 	 GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0},
 
-	{"incremental", OPT_INCREMENTAL, "This option tells xtrabackup to "
-	 "create an incremental backup, rather than a full one. It is passed "
-	 "to the xtrabackup child process. When this option is specified, "
+	{"incremental", OPT_INCREMENTAL,
+	 "Create an incremental backup, rather than a full one. When this option is specified, "
 	 "either --incremental-lsn or --incremental-basedir can also be given. "
-	 "If neither option is given, option --incremental-basedir is passed "
-	 "to xtrabackup by default, set to the first timestamped backup "
+	 "If neither option is given, option --incremental-basedir is used "
+	 "by default, set to the first timestamped backup "
 	 "directory in the backup base directory.",
 	 (uchar *) &opt_ibx_incremental, (uchar *) &opt_ibx_incremental, 0,
 	 GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0},
 
-	{"no-lock", OPT_NO_LOCK, "Use this option to disable table lock "
-	 "with \"FLUSH TABLES WITH READ LOCK\". Use it only if ALL your "
+	{"no-lock", OPT_NO_LOCK, "This option should not be used as "
+         "mariadb-backup now is using BACKUP LOCKS, which minimizes the "
+         "lock time. ALTER TABLE can run in parallel with BACKUP LOCKS."
+         "Use the --no-lock option it only if ALL your "
 	 "tables are InnoDB and you DO NOT CARE about the binary log "
 	 "position of the backup. This option shouldn't be used if there "
 	 "are any DDL statements being executed or if any updates are "
@@ -297,19 +297,10 @@ static struct my_option ibx_long_options[] =
 	 (uchar *) &opt_ibx_safe_slave_backup,
 	 0, GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0},
 
-	{"rsync", OPT_RSYNC, "Uses the rsync utility to optimize local file "
-	 "transfers. When this option is specified, innobackupex uses rsync "
-	 "to copy all non-InnoDB files instead of spawning a separate cp for "
-	 "each file, which can be much faster for servers with a large number "
-	 "of databases or tables.  This option cannot be used together with "
-	 "--stream.",
-	 (uchar *) &opt_ibx_rsync, (uchar *) &opt_ibx_rsync,
-	 0, GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0},
-
 	{"force-non-empty-directories", OPT_FORCE_NON_EMPTY_DIRS, "This "
 	 "option, when specified, makes --copy-back or --move-back transfer "
 	 "files to non-empty directories. Note that no existing files will be "
-	 "overwritten. If --copy-back or --nove-back has to copy a file from "
+	 "overwritten. If --copy-back or --move-back has to copy a file from "
 	 "the backup directory which already exists in the destination "
 	 "directory, it will still fail with an error.",
 	 (uchar *) &opt_ibx_force_non_empty_dirs,
@@ -330,13 +321,9 @@ static struct my_option ibx_long_options[] =
 	 (uchar *) &opt_ibx_noversioncheck,
 	 0, GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0},
 
-	{"no-backup-locks", OPT_NO_BACKUP_LOCKS, "This option controls if "
-	 "backup locks should be used instead of FLUSH TABLES WITH READ LOCK "
-	 "on the backup stage. The option has no effect when backup locks are "
-	 "not supported by the server. This option is enabled by default, "
-	 "disable with --no-backup-locks.",
-	 (uchar *) &opt_ibx_no_backup_locks,
-	 (uchar *) &opt_ibx_no_backup_locks,
+	{"no-backup-locks", OPT_NO_BACKUP_LOCKS,
+         "Old disabled option which has no effect anymore.",
+	 (uchar *) 0, (uchar*) 0,
 	 0, GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0},
 
 	{"decompress", OPT_DECOMPRESS, "Decompresses all files with the .qp "
@@ -377,36 +364,35 @@ static struct my_option ibx_long_options[] =
 
 	{"incremental-history-name", OPT_INCREMENTAL_HISTORY_NAME,
 	 "This option specifies the name of the backup series stored in the "
-	 "PERCONA_SCHEMA.xtrabackup_history history record to base an "
-	 "incremental backup on. Xtrabackup will search the history table "
+	 XB_HISTORY_TABLE " history record to base an "
+	 "incremental backup on. Backup will search the history table "
 	 "looking for the most recent (highest innodb_to_lsn), successful "
 	 "backup in the series and take the to_lsn value to use as the "
 	 "starting lsn for the incremental backup. This will be mutually "
 	 "exclusive with --incremental-history-uuid, --incremental-basedir "
 	 "and --incremental-lsn. If no valid lsn can be found (no series by "
-	 "that name, no successful backups by that name) xtrabackup will "
-	 "return with an error. It is used with the --incremental option.",
+	 "that name, no successful backups by that name), "
+	 "an error will be returned. It is used with the --incremental option.",
 	 (uchar*) &opt_ibx_incremental_history_name,
 	 (uchar*) &opt_ibx_incremental_history_name, 0, GET_STR,
 	 REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
 
 	{"incremental-history-uuid", OPT_INCREMENTAL_HISTORY_UUID,
 	 "This option specifies the UUID of the specific history record "
-	 "stored in the PERCONA_SCHEMA.xtrabackup_history to base an "
+	 "stored in the " XB_HISTORY_TABLE " table to base an "
 	 "incremental backup on. --incremental-history-name, "
 	 "--incremental-basedir and --incremental-lsn. If no valid lsn can be "
-	 "found (no success record with that uuid) xtrabackup will return "
-	 "with an error. It is used with the --incremental option.",
+	 "found (no success record with that uuid), an error will be returned."
+	 " It is used with the --incremental option.",
 	 (uchar*) &opt_ibx_incremental_history_uuid,
 	 (uchar*) &opt_ibx_incremental_history_uuid, 0, GET_STR,
 	 REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
 
 	{"ftwrl-wait-query-type", OPT_LOCK_WAIT_QUERY_TYPE,
-	 "This option specifies which types of queries are allowed to complete "
-	 "before innobackupex will issue the global lock. Default is all.",
-	 (uchar*) &opt_ibx_lock_wait_query_type,
-	 (uchar*) &opt_ibx_lock_wait_query_type, &query_type_typelib,
-	 GET_ENUM, REQUIRED_ARG, QUERY_TYPE_ALL, 0, 0, 0, 0, 0},
+         "Old disabled option which has no effect anymore (not needed "
+         "with BACKUP LOCKS)",
+	 (uchar*) 0, (uchar*) 0, &query_type_typelib, GET_ENUM,
+         REQUIRED_ARG, QUERY_TYPE_ALL, 0, 0, 0, 0, 0},
 
 	{"kill-long-query-type", OPT_KILL_LONG_QUERY_TYPE,
 	 "This option specifies which types of queries should be killed to "
@@ -417,7 +403,7 @@ static struct my_option ibx_long_options[] =
 
 	{"history", OPT_HISTORY,
 	 "This option enables the tracking of backup history in the "
-	 "PERCONA_SCHEMA.xtrabackup_history table. An optional history "
+	 XB_HISTORY_TABLE " table. An optional history "
 	 "series name may be specified that will be placed with the history "
 	 "record for the current backup being taken.",
 	 NULL, NULL, 0, GET_STR, OPT_ARG, 0, 0, 0, 0, 0, 0},
@@ -425,7 +411,7 @@ static struct my_option ibx_long_options[] =
 	{"include", OPT_INCLUDE,
 	 "This option is a regular expression to be matched against table "
 	 "names in databasename.tablename format. It is passed directly to "
-	 "xtrabackup's --tables option. See the xtrabackup documentation for "
+	 "--tables option. See the documentation for "
 	 "details.",
 	 (uchar*) &opt_ibx_include,
 	 (uchar*) &opt_ibx_include, 0, GET_STR,
@@ -447,40 +433,34 @@ static struct my_option ibx_long_options[] =
 	 REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
 
 	{"kill-long-queries-timeout", OPT_KILL_LONG_QUERIES_TIMEOUT,
-	 "This option specifies the number of seconds innobackupex waits "
-	 "between starting FLUSH TABLES WITH READ LOCK and killing those "
-	 "queries that block it. Default is 0 seconds, which means "
-	 "innobackupex will not attempt to kill any queries.",
-	 (uchar*) &opt_ibx_kill_long_queries_timeout,
-	 (uchar*) &opt_ibx_kill_long_queries_timeout, 0, GET_UINT,
+         "Old disabled option which has no effect anymore (not needed "
+         "with BACKUP LOCKS)",
+	 (uchar*) 0, (uchar*) 0,  0, GET_UINT,
 	 REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
 
 	{"ftwrl-wait-timeout", OPT_LOCK_WAIT_TIMEOUT,
-	 "This option specifies time in seconds that innobackupex should wait "
-	 "for queries that would block FTWRL before running it. If there are "
-	 "still such queries when the timeout expires, innobackupex terminates "
-	 "with an error. Default is 0, in which case innobackupex does not "
-	 "wait for queries to complete and starts FTWRL immediately.",
-	 (uchar*) &opt_ibx_lock_wait_timeout,
-	 (uchar*) &opt_ibx_lock_wait_timeout, 0, GET_UINT,
+         "Alias for startup-wait-timeout",
+         (uchar*) &opt_ibx_lock_wait_timeout,
+         (uchar*) &opt_ibx_lock_wait_timeout, 0, GET_UINT,
+	 REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
+
+	{"startup-wait-timeout", OPT_LOCK_WAIT_TIMEOUT,
+         "This option specifies time in seconds that mariadb-backup should wait for "
+         "BACKUP STAGE START to complete. BACKUP STAGE START has to wait until all "
+         "currently running queries using explicite LOCK TABLES has ended. "
+         "If there are still such queries when the timeout expires, mariadb-backup "
+         "terminates with an error. Default is 0, in which case mariadb-backup waits "
+         "indefinitely for BACKUP STAGE START to finish",
+         (uchar*) &opt_ibx_lock_wait_timeout,
+         (uchar*) &opt_ibx_lock_wait_timeout, 0, GET_UINT,
 	 REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
 
 	{"ftwrl-wait-threshold", OPT_LOCK_WAIT_THRESHOLD,
-	 "This option specifies the query run time threshold which is used by "
-	 "innobackupex to detect long-running queries with a non-zero value "
-	 "of --ftwrl-wait-timeout. FTWRL is not started until such "
-	 "long-running queries exist. This option has no effect if "
-	 "--ftwrl-wait-timeout is 0. Default value is 60 seconds.",
-	 (uchar*) &opt_ibx_lock_wait_threshold,
-	 (uchar*) &opt_ibx_lock_wait_threshold, 0, GET_UINT,
+         "Old disabled option which has no effect anymore (not needed "
+         "with BACKUP LOCKS)",
+	 (uchar*) 0, (uchar*) 0,  0, GET_UINT,
 	 REQUIRED_ARG, 60, 0, 0, 0, 0, 0},
 
-	{"debug-sleep-before-unlock", OPT_DEBUG_SLEEP_BEFORE_UNLOCK,
-	 "This is a debug-only option used by the XtraBackup test suite.",
-	 (uchar*) &opt_ibx_debug_sleep_before_unlock,
-	 (uchar*) &opt_ibx_debug_sleep_before_unlock, 0, GET_UINT,
-	 REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
-
 	{"safe-slave-backup-timeout", OPT_SAFE_SLAVE_BACKUP_TIMEOUT,
 	 "How many seconds --safe-slave-backup should wait for "
 	 "Slave_open_temp_tables to become zero. (default 300)",
@@ -493,22 +473,20 @@ static struct my_option ibx_long_options[] =
 	We put them here with only purpose for them to showup in
 	innobackupex --help output */
 
-	{"close_files", OPT_CLOSE_FILES, "Do not keep files opened. This "
-	 "option is passed directly to xtrabackup. Use at your own risk.",
+	{"close_files", OPT_CLOSE_FILES, "Do not keep files opened."
+	" Use at your own risk.",
 	 (uchar*) &ibx_xb_close_files, (uchar*) &ibx_xb_close_files, 0,
 	 GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0},
 
-	{"compress", OPT_COMPRESS, "This option instructs xtrabackup to "
-	 "compress backup copies of InnoDB data files. It is passed directly "
-	 "to the xtrabackup child process. Try 'xtrabackup --help' for more "
-	 "details.", (uchar*) &ibx_xtrabackup_compress_alg,
+	{"compress", OPT_COMPRESS, "This option instructs backup to "
+	 "compress backup copies of InnoDB data files."
+	 , (uchar*) &ibx_xtrabackup_compress_alg,
 	 (uchar*) &ibx_xtrabackup_compress_alg, 0,
 	 GET_STR, OPT_ARG, 0, 0, 0, 0, 0, 0},
 
 	{"compress-threads", OPT_COMPRESS_THREADS,
 	 "This option specifies the number of worker threads that will be used "
-	 "for parallel compression. It is passed directly to the xtrabackup "
-	 "child process. Try 'xtrabackup --help' for more details.",
+	 "for parallel compression.",
 	 (uchar*) &ibx_xtrabackup_compress_threads,
 	 (uchar*) &ibx_xtrabackup_compress_threads,
 	 0, GET_UINT, REQUIRED_ARG, 1, 1, UINT_MAX, 0, 0, 0},
@@ -519,17 +497,15 @@ static struct my_option ibx_long_options[] =
 	 (uchar*) &ibx_xtrabackup_compress_chunk_size,
 	 0, GET_ULL, REQUIRED_ARG, (1 << 16), 1024, ULONGLONG_MAX, 0, 0, 0},
 
-	{"export", OPT_EXPORT, "This option is passed directly to xtrabackup's "
-	 "--export option. It enables exporting individual tables for import "
-	 "into another server. See the xtrabackup documentation for details.",
+	{"export", OPT_EXPORT, " enables exporting individual tables for import "
+	 "into another server.",
 	 (uchar*) &ibx_xtrabackup_export, (uchar*) &ibx_xtrabackup_export,
 	 0, GET_BOOL, NO_ARG, 0, 0, 0, 0, 0, 0},
 
 	{"extra-lsndir", OPT_EXTRA_LSNDIR, "This option specifies the "
 	 "directory in which to save an extra copy of the "
-	 "\"xtrabackup_checkpoints\" file. The option accepts a string "
-	 "argument. It is passed directly to xtrabackup's --extra-lsndir "
-	 "option. See the xtrabackup documentation for details.",
+	 "\"" MB_METADATA_FILENAME "\" file. The option accepts a string "
+	 "argument.",
 	 (uchar*) &ibx_xtrabackup_extra_lsndir,
 	 (uchar*) &ibx_xtrabackup_extra_lsndir,
 	 0, GET_STR, REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
@@ -551,7 +527,7 @@ static struct my_option ibx_long_options[] =
 	 0, GET_STR, REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
 
 	{"incremental-force-scan", OPT_INCREMENTAL_FORCE_SCAN,
-	 "This options tells xtrabackup to perform full scan of data files "
+	 "Perform full scan of data files "
 	 "for taking an incremental backup even if full changed page bitmap "
 	 "data is available to enable the backup without the full scan.",
 	 (uchar*)&ibx_xtrabackup_incremental_force_scan,
@@ -577,10 +553,8 @@ static struct my_option ibx_long_options[] =
 	 0, GET_STR, REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
 
 	{"parallel", OPT_PARALLEL, "On backup, this option specifies the "
-	 "number of threads the xtrabackup child process should use to back "
-	 "up files concurrently.  The option accepts an integer argument. It "
-	 "is passed directly to xtrabackup's --parallel option. See the "
-	 "xtrabackup documentation for details.",
+	 "number of threads to use to back "
+	 "up files concurrently.  The option accepts an integer argument.",
 	 (uchar*) &ibx_xtrabackup_parallel, (uchar*) &ibx_xtrabackup_parallel,
 	 0, GET_INT, REQUIRED_ARG, 1, 1, INT_MAX, 0, 0, 0},
 
@@ -588,23 +562,21 @@ static struct my_option ibx_long_options[] =
 	{"stream", OPT_STREAM, "This option specifies the format in which to "
 	 "do the streamed backup.  The option accepts a string argument. The "
 	 "backup will be done to STDOUT in the specified format. Currently, "
-	 "the only supported formats are tar and mbstream/xbstream. This "
-	 "option is passed directly to xtrabackup's --stream option.",
+	 "the only supported formats are tar and mbstream/xbstream.",
 	 (uchar*) &ibx_xtrabackup_stream_str,
 	 (uchar*) &ibx_xtrabackup_stream_str, 0, GET_STR,
 	 REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
 
 	{"tables-file", OPT_TABLES_FILE, "This option specifies the file in "
 	 "which there are a list of names of the form database.  The option "
-	 "accepts a string argument.table, one per line. The option is passed "
-	 "directly to xtrabackup's --tables-file option.",
+	 "accepts a string argument.table, one per line.",
 	 (uchar*) &ibx_xtrabackup_tables_file,
 	 (uchar*) &ibx_xtrabackup_tables_file,
 	 0, GET_STR, REQUIRED_ARG, 0, 0, 0, 0, 0, 0},
 
 	{"throttle", OPT_THROTTLE, "This option specifies a number of I/O "
 	 "operations (pairs of read+write) per second.  It accepts an integer "
-	 "argument.  It is passed directly to xtrabackup's --throttle option.",
+	 "argument.",
 	 (uchar*) &ibx_xtrabackup_throttle, (uchar*) &ibx_xtrabackup_throttle,
 	 0, GET_LONG, REQUIRED_ARG, 0, 0, LONG_MAX, 0, 1, 0},
 
@@ -616,23 +588,32 @@ static struct my_option ibx_long_options[] =
 	 0, 0, 0, 0, 0, 0},
 
 	{"use-memory", OPT_USE_MEMORY, "This option accepts a string argument "
-	 "that specifies the amount of memory in bytes for xtrabackup to use "
+	 "that specifies the amount of memory in bytes to use "
 	 "for crash recovery while preparing a backup. Multiples are supported "
 	 "providing the unit (e.g. 1MB, 1GB). It is used only with the option "
-	 "--apply-log. It is passed directly to xtrabackup's --use-memory "
-	 "option. See the xtrabackup documentation for details.",
+	 "--apply-log.",
 	 (uchar*) &ibx_xtrabackup_use_memory,
 	 (uchar*) &ibx_xtrabackup_use_memory,
 	 0, GET_LL, REQUIRED_ARG, 100*1024*1024L, 1024*1024L, LONGLONG_MAX, 0,
 	 1024*1024L, 0},
 
+	{"innodb-force-recovery", OPT_INNODB_FORCE_RECOVERY,
+	 "This option starts up the embedded InnoDB instance in crash "
+	 "recovery mode to ignore page corruption; should be used "
+	 "with the \"--apply-log\" option, in emergencies only. The "
+	 "default value is 0. Refer to \"innodb_force_recovery\" server "
+	 "system variable documentation for more details.",
+	 (uchar*)&xtrabackup_innodb_force_recovery,
+	 (uchar*)&xtrabackup_innodb_force_recovery,
+	 0, GET_ULONG, OPT_ARG, 0, 0, SRV_FORCE_IGNORE_CORRUPT, 0, 0, 0},
+
 	{ 0, 0, 0, 0, 0, 0, GET_NO_ARG, NO_ARG, 0, 0, 0, 0, 0, 0}
 };
 
 
 static void usage(void)
 {
-	puts("Open source backup tool for InnoDB and XtraDB\n\
+	puts("Open source backup tool\n\
 \n\
 Copyright (C) 2009-2015 Percona LLC and/or its affiliates.\n\
 Portions Copyright (C) 2000, 2011, MySQL AB & Innobase Oy. All Rights Reserved.\n\
@@ -671,6 +652,7 @@ innobackupex [--compress] [--compress-threads=NUMBER-OF-THREADS] [--compress-chu
 innobackupex --apply-log [--use-memory=B]\n\
              [--defaults-file=MY.CNF]\n\
              [--export] [--ibbackup=IBBACKUP-BINARY]\n\
+             [--innodb-force-recovery=1]\n\
              BACKUP-DIR\n\
 \n\
 innobackupex --copy-back [--defaults-file=MY.CNF] [--defaults-group=GROUP-NAME] BACKUP-DIR\n\
@@ -682,7 +664,7 @@ innobackupex [--decompress]\n\
 \n\
 DESCRIPTION\n\
 \n\
-The first command line above makes a hot backup of a MySQL database.\n\
+The first command line above makes a hot backup of a database.\n\
 By default it creates a backup directory (named by the current date\n\
 	and time) in the given backup root directory.  With the --no-timestamp\n\
 option it does not create a time-stamped backup directory, but it puts\n\
@@ -692,22 +674,18 @@ indexes in all databases or in all of the databases specified with the\n\
 --databases option.  The created backup contains .frm, .MRG, .MYD,\n\
 .MYI, .MAD, .MAI, .TRG, .TRN, .ARM, .ARZ, .CSM, CSV, .opt, .par, and\n\
 InnoDB data and log files.  The MY.CNF options file defines the\n\
-location of the database.  This command connects to the MySQL server\n\
-using the mysql client program, and runs xtrabackup as a child\n\
-process.\n\
+location of the database.\n\
 \n\
 The --apply-log command prepares a backup for starting a MySQL\n\
 server on the backup. This command recovers InnoDB data files as specified\n\
 in BACKUP-DIR/backup-my.cnf using BACKUP-DIR/ib_logfile0,\n\
 and creates new InnoDB log files as specified in BACKUP-DIR/backup-my.cnf.\n\
-The BACKUP-DIR should be the path to a backup directory created by\n\
-xtrabackup. This command runs xtrabackup as a child process, but it does not \n\
-connect to the database server.\n\
+The BACKUP-DIR should be the path to a backup directory\n\
 \n\
 The --copy-back command copies data, index, and log files\n\
 from the backup directory back to their original locations.\n\
 The MY.CNF options file defines the original location of the database.\n\
-The BACKUP-DIR is the path to a backup directory created by xtrabackup.\n\
+The BACKUP-DIR is the path to a backup directory.\n\
 \n\
 The --move-back command is similar to --copy-back with the only difference that\n\
 it moves files to their original locations rather than copies them. As this\n\
@@ -731,7 +709,7 @@ indicates an error.\n");
 static
 my_bool
 ibx_get_one_option(const struct my_option *opt,
-                   char *argument, const char *)
+                   const char *argument, const char *)
 {
 	switch(opt->id) {
 	case '?':
@@ -872,10 +850,8 @@ ibx_init()
 	opt_slave_info = opt_ibx_slave_info;
 	opt_no_lock = opt_ibx_no_lock;
 	opt_safe_slave_backup = opt_ibx_safe_slave_backup;
-	opt_rsync = opt_ibx_rsync;
 	opt_force_non_empty_dirs = opt_ibx_force_non_empty_dirs;
 	opt_noversioncheck = opt_ibx_noversioncheck;
-	opt_no_backup_locks = opt_ibx_no_backup_locks;
 	opt_decompress = opt_ibx_decompress;
 
 	opt_incremental_history_name = opt_ibx_incremental_history_name;
