@@ -359,7 +359,7 @@ ssize_t pvio_socket_async_read(MARIADB_PVIO *pvio, uchar *buffer, size_t length)
   r= recv(csock->socket,(void *)buffer, length, read_flags);
 #else
   /* Windows doesn't support MSG_DONTWAIT, so we need to set
-     socket to non-blocking */
+     socket to non blocking */
   pvio_socket_blocking(pvio, 0, 0);
   r= recv(csock->socket, (char *)buffer, (int)length, 0);
 #endif
@@ -435,7 +435,7 @@ ssize_t pvio_socket_async_write(MARIADB_PVIO *pvio, const uchar *buffer, size_t
   r= ma_send(csock->socket, buffer, length, write_flags);
 #else
   /* Windows doesn't support MSG_DONTWAIT, so we need to set
-     socket to non-blocking */
+     socket to non blocking */
   pvio_socket_blocking(pvio, 0, 0);
   r= send(csock->socket, (const char *)buffer, (int)length, 0);
 #endif
@@ -638,7 +638,7 @@ static int pvio_socket_internal_connect(MARIADB_PVIO *pvio,
   csock= (struct st_pvio_socket *)pvio->data;
   timeout= pvio->timeout[PVIO_CONNECT_TIMEOUT];
 
-  /* set non-blocking */
+  /* set non blocking */
   pvio_socket_blocking(pvio, 0, 0);
 
 #ifndef _WIN32
@@ -1101,10 +1101,10 @@ my_bool pvio_socket_is_alive(MARIADB_PVIO *pvio)
 
   res= poll(&poll_fd, 1, 0);
   if (res <= 0) /* timeout or error */
-    return TRUE;
+    return FALSE;
   if (!(poll_fd.revents & (POLLIN | POLLPRI)))
-    return TRUE;
-  return FALSE;
+    return FALSE;
+  return TRUE;
 #else
   /* We can't use the WSAPoll function, it's broken :-(
      (see Windows 8 Bugs 309411 - WSAPoll does not report failed connections)
@@ -1117,8 +1117,8 @@ my_bool pvio_socket_is_alive(MARIADB_PVIO *pvio)
 
   res= select((int)csock->socket + 1, &sfds, NULL, NULL, &tv);
   if (res > 0 && FD_ISSET(csock->socket, &sfds))
-    return FALSE;
-  return TRUE;
+    return TRUE;
+  return FALSE;
 #endif
 }
 /* }}} */
