@@ -65,7 +65,7 @@ static int test_conc66(MYSQL *my)
   rc= mysql_query(my, "FLUSH PRIVILEGES");
   check_mysql_rc(rc, my);
   if (!my_test_connect(mysql, hostname, NULL,
-                             NULL, schema, port, socketname, 0))
+                             NULL, schema, port, socketname, 0, 1))
   {
     diag("user: %s", mysql->options.user);
     diag("Error: %s", mysql_error(mysql));
@@ -463,7 +463,7 @@ static int test_bug31669(MYSQL *mysql)
 static int test_bug33831(MYSQL *mysql)
 {
   FAIL_IF(my_test_connect(mysql, hostname, username,
-                             password, schema, port, socketname, 0), 
+                             password, schema, port, socketname, 0, 1), 
          "Error expected");
   
   return OK;
@@ -493,7 +493,7 @@ static int test_opt_reconnect(MYSQL *mysql)
 
   if (!(my_test_connect(mysql, hostname, username,
                            password, schema, port,
-                           socketname, 0)))
+                           socketname, 0, 1)))
   {
     diag("connection failed");
     mysql_close(mysql);
@@ -513,7 +513,7 @@ static int test_opt_reconnect(MYSQL *mysql)
 
   if (!(my_test_connect(mysql, hostname, username,
                            password, schema, port,
-                           socketname, 0)))
+                           socketname, 0, 1)))
   {
     diag("connection failed");
     mysql_close(mysql);
@@ -544,7 +544,7 @@ static int test_compress(MYSQL *mysql)
 
   if (!(my_test_connect(mysql, hostname, username,
                            password, schema, port,
-                           socketname, 0)))
+                           socketname, 0, 1)))
   {
     diag("connection failed");
     return FAIL;
@@ -583,7 +583,7 @@ static int test_reconnect(MYSQL *mysql)
 
   if (!(my_test_connect(mysql1, hostname, username,
                            password, schema, port,
-                           socketname, 0)))
+                           socketname, 0, 1)))
   {
     diag("connection failed");
     return FAIL;
@@ -647,14 +647,14 @@ int test_conc26(MYSQL *unused __attribute__((unused)))
   MYSQL *mysql= mysql_init(NULL);
   mysql_options(mysql, MYSQL_SET_CHARSET_NAME, "ascii");
 
-  FAIL_IF(my_test_connect(mysql, hostname, "notexistinguser", "password", schema, port, socketname, CLIENT_REMEMBER_OPTIONS),
+  FAIL_IF(my_test_connect(mysql, hostname, "notexistinguser", "password", schema, port, socketname, CLIENT_REMEMBER_OPTIONS, 1),
           "Error expected");
   FAIL_IF(!mysql->options.charset_name || strcmp(mysql->options.charset_name, "ascii") != 0,
           "expected charsetname=ascii");
   mysql_close(mysql);
 
   mysql= mysql_init(NULL);
-  FAIL_IF(my_test_connect(mysql, hostname, "notexistinguser", "password", schema, port, socketname, 0), 
+  FAIL_IF(my_test_connect(mysql, hostname, "notexistinguser", "password", schema, port, socketname, 0, 1), 
           "Error expected");
   FAIL_IF(mysql->options.charset_name, "Error: options not freed");
   mysql_close(mysql);
@@ -669,7 +669,7 @@ int test_connection_timeout(MYSQL *unused __attribute__((unused)))
   MYSQL *mysql= mysql_init(NULL);
   mysql_options(mysql, MYSQL_OPT_CONNECT_TIMEOUT, (unsigned int *)&timeout);
   start= time(NULL);
-  if (my_test_connect(mysql, "192.168.1.101", "notexistinguser", "password", schema, port, socketname, CLIENT_REMEMBER_OPTIONS))
+  if (my_test_connect(mysql, "192.168.1.101", "notexistinguser", "password", schema, port, socketname, CLIENT_REMEMBER_OPTIONS, 1))
   {
     diag("Error expected - maybe you have to change hostname");
     return FAIL;
@@ -686,6 +686,7 @@ int test_connection_timeout2(MYSQL *unused __attribute__((unused)))
   unsigned int timeout= 5;
   time_t start, elapsed;
   MYSQL *mysql;
+  my_bool no= 0;
 
   SKIP_SKYSQL;
   SKIP_MAXSCALE;
@@ -694,8 +695,9 @@ int test_connection_timeout2(MYSQL *unused __attribute__((unused)))
   mysql= mysql_init(NULL);
   mysql_options(mysql, MYSQL_OPT_CONNECT_TIMEOUT, (unsigned int *)&timeout);
   mysql_options(mysql, MYSQL_INIT_COMMAND, "set @a:=SLEEP(7)");
+  mysql_options(mysql, MYSQL_OPT_SSL_VERIFY_SERVER_CERT, &no);
   start= time(NULL);
-  if (my_test_connect(mysql, hostname, username, password, schema, port, socketname, CLIENT_REMEMBER_OPTIONS))
+  if (my_test_connect(mysql, hostname, username, password, schema, port, socketname, CLIENT_REMEMBER_OPTIONS, 1))
   {
   elapsed= time(NULL) - start;
   diag("elapsed: %lu", (unsigned long)elapsed);
@@ -721,7 +723,7 @@ int test_connection_timeout3(MYSQL *unused __attribute__((unused)))
   mysql_options(mysql, MYSQL_OPT_WRITE_TIMEOUT, (unsigned int *)&read_write_timeout);
   mysql_options(mysql, MYSQL_INIT_COMMAND, "set @a:=SLEEP(6)");
   start= time(NULL);
-  if (my_test_connect(mysql, hostname, username, password, schema, port, socketname, CLIENT_REMEMBER_OPTIONS))
+  if (my_test_connect(mysql, hostname, username, password, schema, port, socketname, CLIENT_REMEMBER_OPTIONS, 1))
   {
     diag("timeout error expected");
     elapsed= time(NULL) - start;
@@ -738,7 +740,7 @@ int test_connection_timeout3(MYSQL *unused __attribute__((unused)))
   mysql_options(mysql, MYSQL_OPT_READ_TIMEOUT, (unsigned int *)&read_write_timeout);
   mysql_options(mysql, MYSQL_OPT_WRITE_TIMEOUT, (unsigned int *)&read_write_timeout);
 
-  if (!my_test_connect(mysql, hostname, username, password, schema, port, socketname, CLIENT_REMEMBER_OPTIONS))
+  if (!my_test_connect(mysql, hostname, username, password, schema, port, socketname, CLIENT_REMEMBER_OPTIONS, 1))
   {
     diag("Error: %s", mysql_error(mysql));
     return FAIL;
@@ -799,7 +801,7 @@ static int test_wrong_bind_address(MYSQL *unused __attribute__((unused)))
 
   mysql_options(mysql, MYSQL_OPT_BIND, bind_addr);
   if (my_test_connect(mysql, hostname, username,
-                             password, schema, port, socketname, 0))
+                             password, schema, port, socketname, 0, 1))
   {
     diag("Error expected");
     mysql_close(mysql);
@@ -846,7 +848,7 @@ static int test_bind_address(MYSQL *my)
   mysql_options(mysql, MYSQL_OPT_BIND, bind_addr);
 
   if (!my_test_connect(mysql, bind_addr, username,
-                             password, schema, port, socketname, 0))
+                             password, schema, port, socketname, 0, 1))
   {
     diag("Error: %s\n", mysql_error(mysql));
     mysql_close(mysql);
@@ -1192,7 +1194,7 @@ static int test_auth256(MYSQL *my)
   rc= mysql_query(my, query);
   check_mysql_rc(rc, my);
 
-  if (!my_test_connect(mysql, hostname, "sha256user", "foo", NULL, port, socketname, 0))
+  if (!my_test_connect(mysql, hostname, "sha256user", "foo", NULL, port, socketname, 0, 1))
   {
     diag("error: %s", mysql_error(mysql));
     mysql_close(mysql);
@@ -1202,7 +1204,7 @@ static int test_auth256(MYSQL *my)
 
   mysql= mysql_init(NULL);
   mysql_options(mysql, MYSQL_SERVER_PUBLIC_KEY, "rsa_public_key.pem");
-  if (!my_test_connect(mysql, hostname, "sha256user", "foo", NULL, port, socketname, 0))
+  if (!my_test_connect(mysql, hostname, "sha256user", "foo", NULL, port, socketname, 0, 1))
   {
     diag("error: %s", mysql_error(mysql));
     diag("host: %s", this_host);
@@ -1238,7 +1240,7 @@ static int test_mdev13100(MYSQL *my __attribute__((unused)))
   check_mysql_rc(rc, mysql);
 
   if (!my_test_connect(mysql, hostname, username,
-                             password, schema, port, socketname, 0))
+                             password, schema, port, socketname, 0, 1))
   {
     diag("Error: %s", mysql_error(mysql));
     return FAIL;
@@ -1263,7 +1265,7 @@ static int test_mdev13100(MYSQL *my __attribute__((unused)))
   check_mysql_rc(rc, mysql);
 
   if (!my_test_connect(mysql, hostname, username,
-                             password, schema, port, socketname, 0))
+                             password, schema, port, socketname, 0, 1))
   {
     diag("Error: %s", mysql_error(mysql));
     return FAIL;
@@ -1290,7 +1292,7 @@ if (!(fp= fopen("./mdev13100.cnf", "w")))
   check_mysql_rc(rc, mysql);
 
   if (!my_test_connect(mysql, hostname, username,
-                             password, schema, port, socketname, 0))
+                             password, schema, port, socketname, 0, 1))
   {
     diag("Error: %s", mysql_error(mysql));
     return FAIL;
@@ -1320,7 +1322,7 @@ if (!(fp= fopen("./mdev13100.cnf", "w")))
   check_mysql_rc(rc, mysql);
 
   if (!my_test_connect(mysql, hostname, username,
-                             password, schema, port, socketname, 0))
+                             password, schema, port, socketname, 0, 1))
   {
     diag("Error: %s", mysql_error(mysql));
     return FAIL;
@@ -1348,7 +1350,7 @@ if (!(fp= fopen("./mdev13100.cnf", "w")))
   check_mysql_rc(rc, mysql);
 
   if (!my_test_connect(mysql, hostname, username,
-                             password, schema, port, socketname, 0))
+                             password, schema, port, socketname, 0, 1))
   {
     diag("Error: %s", mysql_error(mysql));
     return FAIL;
@@ -1371,14 +1373,13 @@ static int test_conc276(MYSQL *unused __attribute__((unused)))
   mysql_options(mysql, MYSQL_OPT_SSL_ENFORCE, &val);
   mysql_options(mysql, MYSQL_OPT_RECONNECT, &val);
 
-  if (!my_test_connect(mysql, hostname, username, password, schema, port, socketname, 0))
+  if (!my_test_connect(mysql, hostname, username, password, schema, port, socketname, 0, 1))
   {
     diag("Connection failed. Error: %s", mysql_error(mysql));
     mysql_close(mysql);
     return FAIL;
   }
   diag("Cipher in use: %s", mysql_get_ssl_cipher(mysql));
-
   rc= mariadb_reconnect(mysql);
   check_mysql_rc(rc, mysql);
 
@@ -1422,7 +1423,7 @@ static int test_expired_pw(MYSQL *my)
   mysql= mysql_init(NULL);
 
   my_test_connect(mysql, hostname, "foo", "foo", schema,
-                  port, socketname, 0);
+                  port, socketname, 0, 1);
 
   FAIL_IF(!mysql_errno(mysql), "Error expected");
   mysql_close(mysql);
@@ -1431,7 +1432,7 @@ static int test_expired_pw(MYSQL *my)
   mysql_optionsv(mysql, MYSQL_OPT_CAN_HANDLE_EXPIRED_PASSWORDS, &expire);
 
   my_test_connect(mysql, hostname, "foo", "foo", schema,
-                  port, socketname, 0);
+                  port, socketname, 0, 1);
 
   /* we should be in sandbox mode now, only set commands should be allowed */
   rc= mysql_query(mysql, "DROP TABLE IF EXISTS t1");
@@ -1502,7 +1503,7 @@ static int test_conc317(MYSQL *unused __attribute__((unused)))
 
   mysql_options(mysql, MYSQL_READ_DEFAULT_GROUP, "");
   my_test_connect(mysql, hostname, username, password,
-                  schema, port, socketname, 0);
+                  schema, port, socketname, 0, 1);
 
   remove(cnf_file1);
 
@@ -1550,7 +1551,7 @@ static int test_conc327(MYSQL *unused __attribute__((unused)))
   mysql= mysql_init(NULL);
   mysql_options(mysql, MYSQL_READ_DEFAULT_GROUP, "");
   my_test_connect(mysql, hostname, username, password,
-                  schema, port, socketname, 0);
+                  schema, port, socketname, 0, 1);
 
   remove(cnf_file1);
   remove(cnf_file2);
@@ -1574,7 +1575,7 @@ static int test_conc327(MYSQL *unused __attribute__((unused)))
   mysql= mysql_init(NULL);
   mysql_options(mysql, MYSQL_READ_DEFAULT_FILE, cnf_file2);
   my_test_connect(mysql, hostname, username, password,
-                  schema, port, socketname, 0);
+                  schema, port, socketname, 0, 1);
 
   remove(cnf_file1);
   remove(cnf_file2);
@@ -1600,7 +1601,7 @@ static int test_conc332(MYSQL *unused __attribute__((unused)))
   mysql_options(mysql, MYSQL_SET_CHARSET_NAME, "utf8mb4");
 
   my_test_connect(mysql, hostname, username, password, schema,
-                  port, socketname, 0);
+                  port, socketname, 0, 1);
 
   FAIL_IF(mysql_errno(mysql), "Error during connect");
 
@@ -1639,7 +1640,7 @@ static int test_conc351(MYSQL *unused __attribute__((unused)))
   ulong capabilities= 0;
 
   my_test_connect(mysql, hostname, username, password, schema,
-                  port, socketname, 0);
+                  port, socketname, 0, 1);
 
   FAIL_IF(mysql_errno(mysql), "Error during connect");
 
@@ -1689,7 +1690,7 @@ static int test_conc312(MYSQL *my)
   check_mysql_rc(rc, my);
 
   mysql= mysql_init(NULL);
-  if (!my_test_connect(mysql, hostname, "foo", "foo", schema, port, socketname, 0))
+  if (!my_test_connect(mysql, hostname, "foo", "foo", schema, port, socketname, 0, 1))
   {
     diag("Error: %s", mysql_error(mysql));
     return FAIL;
@@ -1748,7 +1749,7 @@ static int test_conc366(MYSQL *mysql)
   my= mysql_init(NULL);
   if (plugindir)
     mysql_options(my, MYSQL_PLUGIN_DIR, plugindir);
-  if (!my_test_connect(my, hostname, "ede", "MySup8%rPassw@ord", schema, port, socketname, 0))
+  if (!my_test_connect(my, hostname, "ede", "MySup8%rPassw@ord", schema, port, socketname, 0, 1))
   {
     diag("Error: %s", mysql_error(my));
     return FAIL;
@@ -1810,7 +1811,7 @@ static int test_conc443(MYSQL *my __attribute__((unused)))
   mysql_options(mysql, MYSQL_INIT_COMMAND, "set @a:=3");
   mysql_options(mysql, MYSQL_OPT_RECONNECT, &x);
 
-  if (!my_test_connect(mysql, hostname, username, password, schema, port, socketname, CLIENT_REMEMBER_OPTIONS))
+  if (!my_test_connect(mysql, hostname, username, password, schema, port, socketname, CLIENT_REMEMBER_OPTIONS, 1))
   {
     diag("Connection failed. Error: %s", mysql_error(mysql));
     mysql_close(mysql);
@@ -1856,7 +1857,7 @@ static int test_default_auth(MYSQL *my __attribute__((unused)))
   mysql= mysql_init(NULL);
   mysql_options(mysql, MYSQL_DEFAULT_AUTH, "mysql_clear_password");
 
-  if (!my_test_connect(mysql, hostname, username, password, schema, port, socketname, CLIENT_REMEMBER_OPTIONS))
+  if (!my_test_connect(mysql, hostname, username, password, schema, port, socketname, CLIENT_REMEMBER_OPTIONS, 1))
   {
     diag("Connection failed. Error: %s", mysql_error(mysql));
     mysql_close(mysql);
@@ -1867,7 +1868,7 @@ static int test_default_auth(MYSQL *my __attribute__((unused)))
   mysql= mysql_init(NULL);
   mysql_options(mysql, MYSQL_DEFAULT_AUTH, "caching_sha2_password");
 
-  if (!my_test_connect(mysql, hostname, username, password, schema, port, socketname, CLIENT_REMEMBER_OPTIONS))
+  if (!my_test_connect(mysql, hostname, username, password, schema, port, socketname, CLIENT_REMEMBER_OPTIONS, 1))
   {
     diag("Connection failed. Error: %s", mysql_error(mysql));
     mysql_close(mysql);
@@ -1925,7 +1926,7 @@ static int test_conc490(MYSQL *my __attribute__((unused)))
   MYSQL *mysql= mysql_init(NULL);
 
   if (!my_test_connect(mysql, hostname, username,
-                             password, NULL, port, socketname, CLIENT_CONNECT_WITH_DB))
+                             password, NULL, port, socketname, CLIENT_CONNECT_WITH_DB, 1))
   {
     diag("error: %s\n", mysql_error(mysql));
     return FAIL;
@@ -1960,7 +1961,7 @@ static int test_conc544(MYSQL *mysql)
   check_mysql_rc(rc, mysql);
 
   if (my_test_connect(my, hostname, username,
-                             password, schema, port, socketname, 0))
+                             password, schema, port, socketname, 0, 1))
   {
     diag("error expected (restricted auth)");
     return FAIL;
@@ -1983,7 +1984,7 @@ static int test_conc544(MYSQL *mysql)
   if (plugindir)
     mysql_optionsv(my, MYSQL_PLUGIN_DIR, plugindir);
   mysql_optionsv(my, MARIADB_OPT_RESTRICTED_AUTH, "client_ed25519, mysql_native_password");
-  if (!my_test_connect(my, hostname, "ede", "MySup8%rPassw@ord", schema, port, socketname, 0))
+  if (!my_test_connect(my, hostname, "ede", "MySup8%rPassw@ord", schema, port, socketname, 0, 1))
   {
     diag("Error: %s", mysql_error(my));
     return FAIL;
@@ -2007,10 +2008,11 @@ static int test_conn_str(MYSQL *my __attribute__((unused)))
   char conn_str[1024];
   int rc=OK;
 
-  snprintf(conn_str, sizeof(conn_str)-1, "host=%s;user=%s;password={%s};port=%d;socket=%s",
+  snprintf(conn_str, sizeof(conn_str)-1, "host=%s;user=%s;password={%s};port=%d;socket=%s;tls_fp=%s",
                 hostname ? hostname : "localhost", username ? username : "", 
                 password ? password : "", 
-                port, socketname ? socketname : "");
+                port, socketname ? socketname : "",
+                fingerprint[0] ? fingerprint : "");
 
   /* SkySQL requires secure connection */
   if (IS_SKYSQL(hostname))
@@ -2057,7 +2059,7 @@ static int test_conn_str_1(MYSQL *my __attribute__((unused)))
   rc= mysql_options(mysql, MYSQL_READ_DEFAULT_GROUP, "");
   check_mysql_rc(rc, mysql);
 
-  if (!my_test_connect(mysql, NULL, NULL, NULL, NULL, 0, NULL, 0))
+  if (!my_test_connect(mysql, NULL, NULL, NULL, NULL, 0, NULL, 0, 1))
   {
     diag("Error: %s", mysql_error(mysql));
     remove("./conc274.cnf");
@@ -2082,9 +2084,9 @@ static int test_conc365(MYSQL *my __attribute__((unused)))
   char tmp[1024];
 
   snprintf(tmp, sizeof(tmp) - 1,
-   "host=127.0.0.1:3300,%s;user=%s;password=%s;port=%d;socket=%s",
+   "host=127.0.0.1:3300,%s;user=%s;password=%s;port=%d;socket=%s;tls_fp=%s",
    hostname ? hostname : "localhost", username ? username : "", password ? password : "",
-   port, socketname ? socketname : "");
+   port, socketname ? socketname : "", fingerprint[0] ? fingerprint : "");
 
  if (IS_SKYSQL(hostname))
    strcat(tmp, ";ssl_enforce=1");
@@ -2100,7 +2102,7 @@ static int test_conc365(MYSQL *my __attribute__((unused)))
   mysql= mysql_init(NULL);
   snprintf(tmp, sizeof(tmp) -1, "127.0.0.1:3300,%s:%d", hostname ? hostname : "localhost", port);
   if (!my_test_connect(mysql, tmp, username,
-                             password, schema, port, socketname, 0))
+                             password, schema, port, socketname, 0, 1))
   {
     diag("Error: %s", mysql_error(mysql));
     rc= FAIL;
@@ -2114,7 +2116,7 @@ static int test_conc365(MYSQL *my __attribute__((unused)))
   mysql= mysql_init(NULL);
   mysql_options(mysql, MARIADB_OPT_HOST, tmp);
   if (!my_test_connect(mysql, NULL, username,
-                             password, schema, port, socketname, 0))
+                             password, schema, port, socketname, 0, 1))
   {
     diag("Error: %s", mysql_error(mysql));
     rc= FAIL;
@@ -2130,6 +2132,7 @@ static int test_conc365_reconnect(MYSQL *my)
   MYSQL *mysql= mysql_init(NULL);
   char tmp[1024];
   my_bool reconnect= 1;
+
   SKIP_MAXSCALE;
 
   mysql_options(mysql, MYSQL_OPT_RECONNECT, &reconnect);
@@ -2137,24 +2140,25 @@ static int test_conc365_reconnect(MYSQL *my)
   if (IS_SKYSQL(hostname))
   {
     snprintf(tmp, sizeof(tmp) - 1,
-      "host=127.0.0.1:3300,%s;user=%s;password=%s;port=%d;socket=%s;ssl_enforce=1",
+      "host=127.0.0.1:3300,%s;user=%s;password=%s;port=%d;socket=%s;ssl_enforce=1;tls_fp=%s",
       hostname ? hostname : "localhost", username ? username : "", password ? password : "",
-      ssl_port, socketname ? socketname : "");
+      ssl_port, socketname ? socketname : "", fingerprint[0] ? fingerprint : "");
   } else {
     snprintf(tmp, sizeof(tmp) - 1,
-      "host=127.0.0.1:3300,%s;user=%s;password=%s;port=%d;socket=%s",
+      "host=127.0.0.1:3300,%s;user=%s;password=%s;port=%d;socket=%s;tls_fp=%s",
       hostname ? hostname : "localhost", username ? username : "", password ? password : "",
-      port, socketname ? socketname : "");
+      port, socketname ? socketname : "", fingerprint[0] ? fingerprint :"");
   }
 
   if (!my_test_connect(mysql, tmp, username,
-                             password, schema, port, socketname, CLIENT_REMEMBER_OPTIONS))
+                             password, schema, port, socketname, CLIENT_REMEMBER_OPTIONS, 1))
   {
     diag("Error: %s", mysql_error(mysql));
     rc= FAIL;
   }
 
   sprintf(tmp, "KILL %ld", mysql_thread_id(mysql));
+  diag("KILL %ld", mysql_thread_id(mysql));
 
   rc= mysql_query(my, tmp);
   check_mysql_rc(rc, my);
@@ -2229,7 +2233,7 @@ static int test_status_callback(MYSQL *my __attribute__((unused)))
   rc= mysql_optionsv(mysql, MARIADB_OPT_STATUS_CALLBACK, my_status_callback, &data);
 
   if (!my_test_connect(mysql, hostname, username,
-                      password, NULL, port, socketname, 0))
+                      password, NULL, port, socketname, 0, 1))
   {
     diag("error1: %s", mysql_error(mysql));
     return FAIL;
@@ -2275,7 +2279,7 @@ static int test_conc632(MYSQL *my __attribute__((unused)))
   MYSQL *mysql= mysql_init(NULL);
   int rc;
 
-  if (!my_test_connect(mysql, hostname, username, password, schema, port, socketname, CLIENT_REMEMBER_OPTIONS))
+  if (!my_test_connect(mysql, hostname, username, password, schema, port, socketname, CLIENT_REMEMBER_OPTIONS, 1))
   {
     diag("Connection failed. Error: %s", mysql_error(mysql));
     mysql_close(mysql);
@@ -2313,7 +2317,7 @@ static int test_conc505(MYSQL *my __attribute__((unused)))
 
 #define CLIENT_DEPRECATE_EOF (1ULL << 24)
 
-  if (my_test_connect(mysql, hostname, username, password, schema, port, socketname, CLIENT_DEPRECATE_EOF))
+  if (my_test_connect(mysql, hostname, username, password, schema, port, socketname, CLIENT_DEPRECATE_EOF, 1))
   {
     diag("Error expected: Invalid client flag");
     mysql_close(mysql);
@@ -2321,7 +2325,7 @@ static int test_conc505(MYSQL *my __attribute__((unused)))
   }
   diag("Error (expected): %s", mysql_error(mysql));
   FAIL_IF(mysql_errno(mysql) != CR_INVALID_CLIENT_FLAG, "Wrong error number");
-  if (!my_test_connect(mysql, hostname, username, password, schema, port, socketname, CLIENT_MULTI_STATEMENTS | CLIENT_MULTI_RESULTS))
+  if (!my_test_connect(mysql, hostname, username, password, schema, port, socketname, CLIENT_MULTI_STATEMENTS | CLIENT_MULTI_RESULTS, 1))
   {
     diag("Error: %s", mysql_error(mysql));
     mysql_close(mysql);
@@ -2332,6 +2336,86 @@ static int test_conc505(MYSQL *my __attribute__((unused)))
   return OK;
 }
 
+static int test_parsec(MYSQL *my)
+{
+  int rc;
+  int verify= 0;
+  MYSQL *mysql;
+
+  if (!is_mariadb)
+  {
+    diag("feature not supported by MySQL server");
+    return SKIP;
+  }
+
+  if (!mysql_client_find_plugin(my, "parsec", MYSQL_CLIENT_AUTHENTICATION_PLUGIN))
+  {
+    diag("parsec plugin not available");
+    diag("error: %s", mysql_error(my));
+    return SKIP;
+  }
+
+  rc= mysql_query(my, "INSTALL soname 'auth_parsec'");
+  if (rc)
+  {
+    diag("server doesn't support parsec plugin");
+    return SKIP;
+  }
+  rc= mysql_query(my, "CREATE OR REPLACE USER test1@'%' IDENTIFIED VIA parsec using PASSWORD('123')");
+  check_mysql_rc(rc, my);
+
+  mysql= mysql_init(NULL);
+  mysql_options(mysql, MYSQL_OPT_SSL_VERIFY_SERVER_CERT, &verify);
+  if (!my_test_connect(mysql, hostname, "test1", "123", NULL, port, socketname, 0, 0))
+  {
+    diag("Connection failed. Error: %s", mysql_error(mysql));
+    mysql_close(mysql);
+    return FAIL;
+  }
+
+  rc= mysql_query(my, "DROP USER test1@'%'");
+  check_mysql_rc(rc, my);
+
+  rc= mysql_query(my, "UNINSTALL soname IF EXISTS 'auth_parsec'");
+  check_mysql_rc(rc, my);
+
+  mysql_close(mysql);
+  return OK;
+}
+
+/* Test for PR250 */
+int test_tls_timeout(MYSQL *unused __attribute__((unused)))
+{
+  unsigned int connect_timeout= 5;
+  unsigned int read_write_timeout= 10;
+  int rc;
+  time_t start, elapsed;
+
+  MYSQL *mysql= mysql_init(NULL);
+  mysql_ssl_set(mysql, NULL, NULL, NULL, NULL, NULL);
+  mysql_options(mysql, MYSQL_OPT_CONNECT_TIMEOUT, (unsigned int *)&connect_timeout);
+  mysql_options(mysql, MYSQL_OPT_READ_TIMEOUT, (unsigned int *)&read_write_timeout);
+  mysql_options(mysql, MYSQL_OPT_WRITE_TIMEOUT, (unsigned int *)&read_write_timeout);
+  if (!my_test_connect(mysql, hostname, username, password, schema, port, socketname, CLIENT_REMEMBER_OPTIONS, 1))
+  {
+    mysql_close(mysql);
+    return FAIL;
+  }
+
+  start= time(NULL);
+  rc= mysql_query(mysql, "SET @a:=SLEEP(12)");
+  elapsed= time(NULL) - start;
+  diag("elapsed: %lu", (unsigned long)elapsed);
+  FAIL_IF((unsigned int)elapsed > read_write_timeout + 1, "timeout ignored");
+
+  FAIL_IF(!rc, "expected timeout error");
+  diag("Error: %s", mysql_error(mysql));
+
+  mysql_close(mysql);
+  return OK;
+}
+
+
 #if defined(HAVE_GNUTLS) && GNUTLS_VERSION_NUMBER >= 0x030700 || defined(HAVE_OPENSSL)
 #define HAVE_test_conc748
 static int test_conc748(MYSQL *my __attribute__((unused)))
@@ -2339,7 +2423,6 @@ static int test_conc748(MYSQL *my __attribute__((unused)))
   MYSQL *mysql;
   int i;
   const char *ciphers[3]= {"TLS_AES_128_GCM_SHA256", "TLS_AES_256_GCM_SHA384", "TLS_CHACHA20_POLY1305_SHA256"};
-  my_bool verify= 0;
 
   SKIP_MAXSCALE;
 
@@ -2349,11 +2432,10 @@ static int test_conc748(MYSQL *my __attribute__((unused)))
     mysql= mysql_init(NULL);
 
     mysql_ssl_set(mysql, NULL, NULL, NULL, NULL, NULL);
-    mysql_optionsv(mysql, MYSQL_OPT_SSL_VERIFY_SERVER_CERT, &verify);
     mysql_optionsv(mysql, MYSQL_OPT_SSL_CIPHER, ciphers[i]);
 
     if (!my_test_connect(mysql, hostname, username,
-                         password, schema, port, socketname, 0))
+                         password, schema, port, socketname, 0, 0))
     {
       diag("error: %s", mysql_error(mysql));
       return FAIL;
@@ -2370,88 +2452,20 @@ static int test_conc748(MYSQL *my __attribute__((unused)))
 #endif
 
 static int test_conc589(MYSQL *my)
-{
-  MYSQL *mysql= mysql_init(NULL);
-  int rc;
-  my_bool reconnect= 1, verify= 0;
-  unsigned long last_thread_id= 0;
-
-  SKIP_MAXSCALE;
-
-  mysql_options(mysql, MYSQL_OPT_RECONNECT, &reconnect);
-  mysql_options(mysql, MYSQL_OPT_SSL_VERIFY_SERVER_CERT, &verify);
-
-  if (!my_test_connect(mysql, hostname, username,
-                       password, schema, port, socketname, CLIENT_REMEMBER_OPTIONS))
-  {
-    diag("error: %s", mysql_error(mysql));
-    return FAIL;
-  }
-
-  rc= mysql_query(mysql, "SET SESSION wait_timeout=5");
-  check_mysql_rc(rc, mysql);
-
-  last_thread_id= mysql_thread_id(mysql);
-  rc= mysql_query(mysql, "SET @a:=1");
-  check_mysql_rc(rc, mysql);
-
-  sleep(10);
-
-  rc= mysql_query(mysql, "SET @a:=2");
-  check_mysql_rc(rc, mysql);
-  FAIL_IF(mysql_thread_id(mysql) == last_thread_id, "Expected new connection id");
-  last_thread_id= mysql_thread_id(mysql);
-
-  mysql_kill(my, last_thread_id);
-
-  sleep(10);
-
-  rc= mysql_query(mysql, "SET @a:=3");
-  check_mysql_rc(rc, mysql);
-  FAIL_IF(mysql_thread_id(mysql) == last_thread_id, "Expected new connection id");
-  mysql_close(mysql);
-  return OK;
-}
-
-#ifdef WIN32
-static int test_conc760(MYSQL *my)
 {
   MYSQL *mysql= mysql_init(NULL);
   MYSQL_RES *result;
-  MYSQL_ROW row;
   int rc;
-  char named_pipe_name[128];
   my_bool reconnect= 1, verify= 0;
   unsigned long last_thread_id= 0;
-  unsigned int protocol= MYSQL_PROTOCOL_PIPE;
-  my_bool have_named_pipe= 0;
 
   SKIP_MAXSCALE;
 
-  rc= mysql_query(my, "select @@named_pipe, @@socket");
-  check_mysql_rc(rc, mysql);
-
-  if ((result= mysql_store_result(my)))
-  {
-    if((row= mysql_fetch_row(result)))
-      have_named_pipe= atoi(row[0]);
-    strncpy(named_pipe_name, row[1], sizeof(named_pipe_name)-1);
-    named_pipe_name[sizeof(named_pipe_name)-1]= '\0';
-    mysql_free_result(result);
-  }
-
-  if (!have_named_pipe)
-  {
-    diag("Server doesn't support named pipes");
-    return SKIP;
-  }
-
   mysql_options(mysql, MYSQL_OPT_RECONNECT, &reconnect);
   mysql_options(mysql, MYSQL_OPT_SSL_VERIFY_SERVER_CERT, &verify);
-  mysql_options(mysql, MYSQL_OPT_PROTOCOL, &protocol);
 
   if (!my_test_connect(mysql, hostname, username,
-                       password, schema, port, named_pipe_name, CLIENT_REMEMBER_OPTIONS))
+                       password, schema, port, socketname, CLIENT_REMEMBER_OPTIONS, 0))
   {
     diag("error: %s", mysql_error(mysql));
     return FAIL;
@@ -2467,10 +2481,9 @@ static int test_conc760(MYSQL *my)
   mysql_free_result(result);
   sleep(10);
 
-  rc= mysql_query(mysql, "SELECT 2");
-  check_mysql_rc(rc, mysql);
-  if (result= mysql_store_result(mysql))
-    mysql_free_result(result);
+  if ((rc= mysql_query(mysql, "SELECT 2")) || (result= mysql_store_result(mysql)) == NULL)
+    check_mysql_rc(rc, mysql);
+  mysql_free_result(result);
   FAIL_IF(mysql_thread_id(mysql) == last_thread_id, "Expected new connection id");
   last_thread_id= mysql_thread_id(mysql);
 
@@ -2485,13 +2498,11 @@ static int test_conc760(MYSQL *my)
   mysql_close(mysql);
   return OK;
 }
-#endif
 
 struct my_tests_st my_tests[] = {
-#ifdef WIN32
-  {"test_conc760", test_conc760, TEST_CONNECTION_DEFAULT, 0, NULL, NULL},
-#endif
   {"test_conc589", test_conc589, TEST_CONNECTION_DEFAULT, 0, NULL, NULL},
+  {"test_tls_timeout", test_tls_timeout, TEST_CONNECTION_NONE, 0, NULL, NULL},
+  {"test_parsec", test_parsec, TEST_CONNECTION_DEFAULT, 0, NULL, NULL},
 #ifdef HAVE_test_conc748
   {"test_conc748", test_conc748, TEST_CONNECTION_NONE, 0, NULL, NULL},
 #endif
@@ -2542,6 +2553,7 @@ struct my_tests_st my_tests[] = {
   {"test_connection_timeout", test_connection_timeout, TEST_CONNECTION_NONE, 0, NULL, NULL},
   {"test_connection_timeout2", test_connection_timeout2, TEST_CONNECTION_NONE, 0, NULL, NULL}, 
   {"test_connection_timeout3", test_connection_timeout3, TEST_CONNECTION_NONE, 0, NULL, NULL},
+  {"test_tls_timeout", test_tls_timeout, TEST_CONNECTION_NONE, 0, NULL, NULL},
   {NULL, NULL, 0, 0, NULL, NULL}
 };
 
