@@ -18,7 +18,6 @@
 #include <mysys_priv.h>
 #include <my_default.h>
 #include <m_string.h>
-#include <ctype.h>
 #include <stdlib.h>
 #include <mysys_err.h>
 #include <my_getopt.h>
@@ -908,7 +907,9 @@ static int setval(const struct my_option *opts, void *value, char *argument,
       goto ret;
     };
   }
+
   validate_value(opts->name, argument, option_file);
+
   DBUG_RETURN(0);
 
 ret:
@@ -1003,7 +1004,7 @@ my_bool getopt_compare_strings(register const char *s, register const char *t,
 
   for (;s != end ; s++, t++)
   {
-    if ((*s != '-' ? tolower(*s) : '_') != (*t != '-' ? tolower(*t) : '_'))
+    if ((*s != '-' ? *s : '_') != (*t != '-' ? *t : '_'))
       DBUG_RETURN(1);
   }
   DBUG_RETURN(0);
