@@ -68,20 +68,15 @@ static ulonglong timespec_to_usec(const struct timespec *ts)
   return (ulonglong) ts->tv_sec * TIME_MILLION + ts->tv_nsec / TIME_THOUSAND;
 }
 
-static int
-signal_waiting_transaction(THD *waiting_thd, bool thd_valid,
-                           const char *binlog_file, my_off_t binlog_pos)
+int signal_waiting_transaction(THD *waiting_thd, const char *binlog_file,
+                                my_off_t binlog_pos)
 {
   /*
     It is possible that the connection thd waiting for an ACK was killed. In
     such circumstance, the connection thread will nullify the thd member of its
     Active_tranx node. So before we try to signal, ensure the THD exists.
-
-    The thd_valid is only set while the THD is waiting in commit_trx(); this
-    is defensive coding to not signal an invalid THD if we somewhere
-    accidentally did not remove the transaction from the list.
   */
-  if (waiting_thd && thd_valid)
+  if (waiting_thd)
     mysql_cond_signal(&waiting_thd->COND_wakeup_ready);
   return 0;
 }
@@ -187,7 +182,6 @@ int Active_tranx::insert_tranx_node(THD *thd_to_wait,
   ins_node->log_name[FN_REFLEN-1] = 0; /* make sure it ends properly */
   ins_node->log_pos = log_file_pos;
   ins_node->thd= thd_to_wait;
-  ins_node->thd_valid= false;
 
   if (!m_trx_front)
   {
@@ -269,8 +263,7 @@ void Active_tranx::clear_active_tranx_nodes(
     if ((log_file_name != NULL) &&
         compare(new_front, log_file_name, log_file_pos) > 0)
       break;
-    pre_delete_hook(new_front->thd, new_front->thd_valid,
-                    new_front->log_name, new_front->log_pos);
+    pre_delete_hook(new_front->thd, new_front->log_name, new_front->log_pos);
     new_front = new_front->next;
   }
 
@@ -362,17 +355,13 @@ void Active_tranx::unlink_thd_as_waiter(const char *log_file_name,
   }
 
   if (entry)
-  {
     entry->thd= NULL;
-    entry->thd_valid= false;
-  }
 
   DBUG_VOID_RETURN;
 }
 
-Tranx_node *
-Active_tranx::is_thd_waiter(THD *thd_to_check, const char *log_file_name,
-                            my_off_t log_file_pos)
+bool Active_tranx::is_thd_waiter(THD *thd_to_check, const char *log_file_name,
+                                 my_off_t log_file_pos)
 {
   DBUG_ENTER("Active_tranx::assert_thd_is_waiter");
   mysql_mutex_assert_owner(m_lock);
@@ -388,7 +377,7 @@ Active_tranx::is_thd_waiter(THD *thd_to_check, const char *log_file_name,
     entry = entry->hash_next;
   }
 
-  DBUG_RETURN(entry);
+  DBUG_RETURN(static_cast<bool>(entry));
 }
 
 /*******************************************************************************
@@ -576,14 +565,12 @@ void Repl_semi_sync_master::remove_slave()
 {
   lock();
   DBUG_ASSERT(rpl_semi_sync_master_clients > 0);
-  if (!(--rpl_semi_sync_master_clients) && !rpl_semi_sync_master_wait_no_slave
-      && get_master_enabled())
+  if (!(--rpl_semi_sync_master_clients) && !rpl_semi_sync_master_wait_no_slave)
   {
     /*
       Signal transactions waiting in commit_trx() that they do not have to
       wait anymore.
     */
-    DBUG_ASSERT(m_active_tranxs);
     m_active_tranxs->clear_active_tranx_nodes(NULL, 0,
                                               signal_waiting_transaction);
   }
@@ -659,7 +646,7 @@ int Repl_semi_sync_master::report_reply_packet(uint32 server_id,
 l_end:
   {
     char buf[256];
-    octet2hex(buf, (const char*) packet,
+    octet2hex(buf, (const unsigned char*) packet,
               MY_MIN(sizeof(buf)-1, (size_t) packet_len));
     sql_print_information("First bytes of the packet from semisync slave "
                           "server-id %d: %s", server_id, buf);
@@ -874,10 +861,6 @@ int Repl_semi_sync_master::commit_trx(const char *trx_wait_binlog_name,
 
   if (!rpl_semi_sync_master_clients && !rpl_semi_sync_master_wait_no_slave)
   {
-    lock();
-    m_active_tranxs->unlink_thd_as_waiter(trx_wait_binlog_name,
-                                          trx_wait_binlog_pos);
-    unlock();
     rpl_semi_sync_master_no_transactions++;
     DBUG_RETURN(0);
   }
@@ -937,9 +920,6 @@ int Repl_semi_sync_master::commit_trx(const char *trx_wait_binlog_name,
         }
       }
 
-      Tranx_node *tranx_entry=
-        m_active_tranxs->is_thd_waiter(thd, trx_wait_binlog_name,
-                                       trx_wait_binlog_pos);
       /* In between the binlogging of this transaction and this wait, it is
        * possible that our entry in Active_tranx was removed (i.e. if
        * semi-sync was switched off and on). It is also possible that the
@@ -950,7 +930,8 @@ int Repl_semi_sync_master::commit_trx(const char *trx_wait_binlog_name,
        * rpl_semi_sync_master_yes/no_tx consistent with it, we check for a
        * semi-sync restart _after_ checking the reply state.
        */
-      if (unlikely(!tranx_entry))
+      if (unlikely(!m_active_tranxs->is_thd_waiter(thd, trx_wait_binlog_name,
+                                                   trx_wait_binlog_pos)))
       {
         DBUG_EXECUTE_IF(
             "semisync_log_skip_trx_wait",
@@ -969,16 +950,6 @@ int Repl_semi_sync_master::commit_trx(const char *trx_wait_binlog_name,
         break;
       }
 
-      /*
-        Mark that our THD is now valid for signalling to by the ack thread.
-        It is important to ensure that we can never leave a no longer valid
-        THD in the transaction list and signal it, eg. MDEV-36934. This way,
-        we ensure the THD will only be signalled while this function is
-        running, even in case of some incorrect error handling or similar
-        that might leave a dangling THD in the list.
-      */
-      tranx_entry->thd_valid= true;
-
       /* Let us update the info about the minimum binlog position of waiting
        * threads.
        */
@@ -1311,8 +1282,6 @@ int Repl_semi_sync_master::write_tranx_in_binlog(THD *thd,
 
   DBUG_ENTER("Repl_semi_sync_master::write_tranx_in_binlog");
 
-  DEBUG_SYNC(current_thd, "semisync_at_write_tranx_in_binlog");
-
   lock();
 
   /* This is the real check inside the mutex. */
@@ -1346,8 +1315,7 @@ int Repl_semi_sync_master::write_tranx_in_binlog(THD *thd,
     m_commit_file_name_inited = true;
   }
 
-  if (is_on() &&
-      (rpl_semi_sync_master_clients || rpl_semi_sync_master_wait_no_slave))
+  if (is_on())
   {
     DBUG_ASSERT(m_active_tranxs != NULL);
     if(m_active_tranxs->insert_tranx_node(thd, log_file_name, log_file_pos))
