@@ -305,17 +305,12 @@ end:
   started with corresponding variable that is greater then 0.
 */
 
-extern "C" uchar *get_key_conn(user_conn *buff, size_t *length,
-			      my_bool not_used __attribute__((unused)))
+extern "C" const uchar *get_key_conn(const void *buff_, size_t *length,
+                                     my_bool)
 {
+  auto buff= static_cast<const user_conn *>(buff_);
   *length= buff->len;
-  return (uchar*) buff->user;
-}
-
-
-extern "C" void free_user(struct user_conn *uc)
-{
-  my_free(uc);
+  return reinterpret_cast<const uchar *>(buff->user);
 }
 
 
@@ -323,8 +318,8 @@ void init_max_user_conn(void)
 {
 #ifndef NO_EMBEDDED_ACCESS_CHECKS
   my_hash_init(key_memory_user_conn, &hash_user_connections,
-               system_charset_info, max_connections, 0, 0, (my_hash_get_key)
-               get_key_conn, (my_hash_free_key) free_user, 0);
+               system_charset_info, max_connections, 0, 0, get_key_conn,
+               my_free, 0);
 #endif
 }
 
@@ -403,16 +398,12 @@ static const char *get_client_host(THD *client)
     client->security_ctx->host ? client->security_ctx->host : "";
 }
 
-extern "C" uchar *get_key_user_stats(USER_STATS *user_stats, size_t *length,
-                                     my_bool not_used __attribute__((unused)))
+extern "C" const uchar *get_key_user_stats(const void *user_stats_,
+                                           size_t *length, my_bool)
 {
+  auto user_stats= static_cast<const USER_STATS *>(user_stats_);
   *length= user_stats->user_name_length;
-  return (uchar*) user_stats->user;
-}
-
-void free_user_stats(USER_STATS* user_stats)
-{
-  my_free(user_stats);
+  return reinterpret_cast<const uchar *>(user_stats->user);
 }
 
 void init_user_stats(USER_STATS *user_stats,
@@ -483,54 +474,44 @@ void init_user_stats(USER_STATS *user_stats,
 
 void init_global_user_stats(void)
 {
-  my_hash_init(PSI_INSTRUMENT_ME, &global_user_stats, system_charset_info, max_connections,
-               0, 0, (my_hash_get_key) get_key_user_stats,
-               (my_hash_free_key) free_user_stats, 0);
+  my_hash_init(PSI_INSTRUMENT_ME, &global_user_stats, system_charset_info,
+               max_connections, 0, 0, get_key_user_stats, my_free, 0);
 }
 
 void init_global_client_stats(void)
 {
-  my_hash_init(PSI_INSTRUMENT_ME, &global_client_stats, system_charset_info, max_connections,
-               0, 0, (my_hash_get_key) get_key_user_stats,
-               (my_hash_free_key) free_user_stats, 0);
+  my_hash_init(PSI_INSTRUMENT_ME, &global_client_stats, system_charset_info,
+               max_connections, 0, 0, get_key_user_stats, my_free, 0);
 }
 
-extern "C" uchar *get_key_table_stats(TABLE_STATS *table_stats, size_t *length,
-                                      my_bool not_used __attribute__((unused)))
+extern "C" const uchar *get_key_table_stats(const void *table_stats_,
+                                            size_t *length, my_bool)
 {
+  auto table_stats= static_cast<const TABLE_STATS *>(table_stats_);
   *length= table_stats->table_name_length;
-  return (uchar*) table_stats->table;
-}
-
-extern "C" void free_table_stats(TABLE_STATS* table_stats)
-{
-  my_free(table_stats);
+  return reinterpret_cast<const uchar *>(table_stats->table);
 }
 
 void init_global_table_stats(void)
 {
-  my_hash_init(PSI_INSTRUMENT_ME, &global_table_stats, system_charset_info,
-               max_connections, 0, 0, (my_hash_get_key) get_key_table_stats,
-               (my_hash_free_key) free_table_stats, 0);
+  my_hash_init(PSI_INSTRUMENT_ME, &global_table_stats,
+               Lex_ident_fs::charset_info(), max_connections, 0, 0,
+               get_key_table_stats, my_free, 0);
 }
 
-extern "C" uchar *get_key_index_stats(INDEX_STATS *index_stats, size_t *length,
-                                     my_bool not_used __attribute__((unused)))
+extern "C" const uchar *get_key_index_stats(const void *index_stats_,
+                                            size_t *length, my_bool)
 {
+  auto index_stats= static_cast<const INDEX_STATS *>(index_stats_);
   *length= index_stats->index_name_length;
-  return (uchar*) index_stats->index;
-}
-
-extern "C" void free_index_stats(INDEX_STATS* index_stats)
-{
-  my_free(index_stats);
+  return reinterpret_cast<const uchar *>(index_stats->index);
 }
 
 void init_global_index_stats(void)
 {
-  my_hash_init(PSI_INSTRUMENT_ME, &global_index_stats, system_charset_info,
-               max_connections, 0, 0, (my_hash_get_key) get_key_index_stats,
-               (my_hash_free_key) free_index_stats, 0);
+  my_hash_init(PSI_INSTRUMENT_ME, &global_index_stats,
+               Lex_ident_fs::charset_info(), max_connections, 0, 0,
+               get_key_index_stats, my_free, 0);
 }
 
 
@@ -779,6 +760,10 @@ void update_global_user_stats(THD *thd, bool create_user, time_t now)
 bool thd_init_client_charset(THD *thd, uint cs_number)
 {
   CHARSET_INFO *cs;
+
+  // Test a non-default collation ID. See also comments in this function below.
+  DBUG_EXECUTE_IF("thd_init_client_charset_utf8mb3_bin", cs_number= 83;);
+
   /*
    Use server character set and collation if
    - opt_character_set_client_handshake is not set
@@ -801,9 +786,25 @@ bool thd_init_client_charset(THD *thd, uint cs_number)
                cs->cs_name.str);
       return true;
     }
-    Sql_used used;
-    cs= global_system_variables.character_set_collations.
-          get_collation_for_charset(&used, cs);
+    /*
+      Some connectors (e.g. JDBC, Node.js) can send non-default collation IDs
+      in the handshake packet, to set @@collation_connection right during
+      handshake. Although this is a non-documenting feature,
+      for better backward compatibility with such connectors let's:
+      a. resolve only default collations according to @@character_set_collations
+      b. preserve non-default collations as is
+
+      Perhaps eventually we should change (b) also to resolve non-default
+      collations accoding to @@character_set_collations. Clients that used to
+      send a non-default collation ID in the handshake packet will have to set
+      @@character_set_collations instead.
+    */
+    if (cs->state & MY_CS_PRIMARY)
+    {
+      Sql_used used;
+      cs= global_system_variables.character_set_collations.
+            get_collation_for_charset(&used, cs);
+    }
     thd->org_charset= cs;
     thd->update_charset(cs,cs,cs);
   }
@@ -829,21 +830,6 @@ bool init_new_connection_handler_thread()
   return 0;
 }
 
-static bool check_require_secured_transport(THD *thd)
-{
-  Vio *vio= thd->net.vio;
-  if (opt_require_secure_transport)
-  {
-    enum enum_vio_type type= vio_type(vio);
-
-    return
-      (type != VIO_TYPE_SSL) &&
-      (type != VIO_TYPE_NAMEDPIPE) &&
-      (type != VIO_TYPE_SOCKET);
-  }
-  return 0;
-}
-
 /**
   Set client address during authentication.
 
@@ -1004,7 +990,7 @@ static int check_connection(THD *thd)
                       /* See RFC 5737, 192.0.2.0/24 is reserved. */
                       const char* fake= "192.0.2.4";
                       inet_pton(AF_INET,fake, ip4);
-                      strcpy(ip, fake);
+                      safe_strcpy(ip, sizeof(ip), fake);
                       peer_rc= 0;
                     }
                     );
@@ -1034,7 +1020,7 @@ static int check_connection(THD *thd)
                       ip6->s6_addr[13] = 0x06;
                       ip6->s6_addr[14] = 0x00;
                       ip6->s6_addr[15] = 0x06;
-                      strcpy(ip, fake);
+                      safe_strcpy(ip, sizeof(ip), fake);
                       peer_rc= 0;
                     }
                     );
@@ -1095,18 +1081,6 @@ static int check_connection(THD *thd)
     statistic_increment(aborted_connects_preauth, &LOCK_status);
     return 1; /* The error is set by alloc(). */
   }
-
-  if(check_require_secured_transport(thd))
-  {
-    Host_errors errors;
-    errors.m_ssl= 1;
-    inc_host_errors(thd->main_security_ctx.ip, &errors);
-    status_var_increment(thd->status_var.access_denied_errors);
-    my_error(ER_SECURE_TRANSPORT_REQUIRED, MYF(0));
-
-    return 1;
-  }
-
   auth_rc= acl_authenticate(thd, 0);
   if (auth_rc == 0 && connect_errors != 0)
   {
@@ -1269,8 +1243,7 @@ void prepare_new_connection_state(THD* thd)
     embedded server library.
     TODO: refactor this to avoid code duplication there
   */
-  thd->proc_info= 0;
-  thd->set_command(COM_SLEEP);
+  thd->mark_connection_idle();
   thd->init_for_queries();
 
   if (opt_init_connect.length &&
@@ -1304,7 +1277,7 @@ void prepare_new_connection_state(THD* thd)
                  thd->thread_id,
                  thd->db.str ? thd->db.str : "unconnected",
                  sctx->user ? sctx->user : "unauthenticated",
-                 sctx->host_or_ip, "init_connect command failed");
+                 sctx->host_or_ip, "", "init_connect command failed");
       thd->server_status&= ~SERVER_STATUS_CLEAR_SET;
       thd->protocol->end_statement();
       thd->killed = KILL_CONNECTION;
@@ -1390,7 +1363,7 @@ void do_handle_one_connection(CONNECT *connect, bool put_in_cache)
   THD *thd;
   if (!(thd= connect->create_thd(NULL)))
   {
-    connect->close_and_delete();
+    connect->close_and_delete(0);
     return;
   }
 
@@ -1415,16 +1388,6 @@ void do_handle_one_connection(CONNECT *connect, bool put_in_cache)
   thd->thr_create_utime= thr_create_utime;
   /* We need to set this because of time_out_user_resource_limits */
   thd->start_utime= thr_create_utime;
-
-  /*
-    handle_one_connection() is normally the only way a thread would
-    start and would always be on the very high end of the stack ,
-    therefore, the thread stack always starts at the address of the
-    first local variable of handle_one_connection, which is thd. We
-    need to know the start of the stack so that we could check for
-    stack overruns.
-  */
-  thd->thread_stack= (char*) &thd;
   setup_connection_thread_globals(thd);
 
   for (;;)
@@ -1465,7 +1428,7 @@ end_thread:
     if (!(connect->create_thd(thd)))
     {
       /* Out of resources. Free thread to get more resources */
-      connect->close_and_delete();
+      connect->close_and_delete(0);
       break;
     }
     delete connect;
@@ -1494,9 +1457,11 @@ end_thread:
   Close connection without error and delete the connect object
   This and close_with_error are only called if we didn't manage to
   create a new thd object.
+
+  Note: err can be 0 if unknown/not inportant
 */
 
-void CONNECT::close_and_delete()
+void CONNECT::close_and_delete(uint err)
 {
   DBUG_ENTER("close_and_delete");
 
@@ -1510,7 +1475,11 @@ void CONNECT::close_and_delete()
   vio_type= VIO_CLOSED;
 
   --*scheduler->connection_count;
-  statistic_increment(connection_errors_internal, &LOCK_status);
+
+  if (err == ER_CON_COUNT_ERROR)
+    statistic_increment(connection_errors_max_connection, &LOCK_status);
+  else
+    statistic_increment(connection_errors_internal, &LOCK_status);
   statistic_increment(aborted_connects,&LOCK_status);
 
   delete this;
@@ -1534,7 +1503,7 @@ void CONNECT::close_with_error(uint sql_errno,
     delete thd;
     set_current_thd(0);
   }
-  close_and_delete();
+  close_and_delete(close_error);
 }
 
 
