@@ -24,6 +24,7 @@
 #include "my_json_writer.h"
 #include "opt_range.h"
 #include "sql_expression_cache.h"
+#include "item_subselect.h"
 
 #include <stack>
 
@@ -44,8 +45,8 @@ const char *pushed_unit_operation_text[4]=
 const char *pushed_derived_text= "PUSHED DERIVED";
 const char *pushed_select_text= "PUSHED SELECT";
 
-static void write_item(Json_writer *writer, Item *item, bool no_tmp_tbl);
-static void append_item_to_str(String *out, Item *item, bool no_tmp_tbl);
+static void write_item(Json_writer *writer, Item *item);
+static void append_item_to_str(String *out, Item *item);
 
 Explain_query::Explain_query(THD *thd_arg, MEM_ROOT *root) : 
   mem_root(root), upd_del_plan(nullptr),  insert_plan(nullptr),
@@ -204,7 +205,7 @@ int Explain_query::send_explain(THD *thd, bool extended)
 
   int res= 0;
   if (thd->lex->explain_json)
-    print_explain_json(result, thd->lex->analyze_stmt, false /*is_show_cmd*/);
+    print_explain_json(result, thd->lex->analyze_stmt);
   else
   {
     res= print_explain(result, lex->describe, thd->lex->analyze_stmt);
@@ -239,6 +240,9 @@ int Explain_query::send_explain(THD *thd, bool extended)
 int Explain_query::print_explain(select_result_sink *output, 
                                  uint8 explain_flags, bool is_analyze)
 {
+  /* A sanity check for ANALYZE: */
+  DBUG_ASSERT(timer_tracker_frequency() != 0.0);
+
   if (upd_del_plan)
   {
     upd_del_plan->print_explain(this, output, explain_flags, is_analyze);
@@ -260,16 +264,8 @@ int Explain_query::print_explain(select_result_sink *output,
 }
 
 
-/*
-   @param  is_show_cmd  TRUE<=> This is a SHOW EXPLAIN|ANALYZE command.
-                        (These commands may be called at late stage in
-                         the query processing, we need to pass no_tmp_tbl=true
-                         to other print functions)
-*/
-
 int Explain_query::print_explain_json(select_result_sink *output,
                                       bool is_analyze,
-                                      bool is_show_cmd,
                                       ulonglong query_time_in_progress_ms)
 {
   Json_writer writer;
@@ -279,12 +275,7 @@ int Explain_query::print_explain_json(select_result_sink *output,
 #endif
 
   writer.start_object();
-  
-  /*
-    If we are printing ANALYZE FORMAT=JSON output, take into account that
-    query's temporary tables have already been freed. See sql_explain.h,
-    sql_explain.h:ExplainDataStructureLifetime for details.
-  */
+
   if (is_analyze)
   {
     if (query_time_in_progress_ms > 0){
@@ -293,11 +284,9 @@ int Explain_query::print_explain_json(select_result_sink *output,
     }
 
     print_query_optimization_json(&writer);
-    is_show_cmd = true;
   }
 
-  bool plan_found = print_query_blocks_json(&writer, is_analyze, is_show_cmd);
-
+  bool plan_found = print_query_blocks_json(&writer, is_analyze);
   writer.end_object();
 
   if( plan_found )
@@ -321,21 +310,19 @@ void Explain_query::print_query_optimization_json(Json_writer *writer)
   }
 }
 
-bool Explain_query::print_query_blocks_json(Json_writer *writer, 
-                                              const bool is_analyze, 
-                                              const bool is_show_cmd)
+bool Explain_query::print_query_blocks_json(Json_writer *writer, const bool is_analyze)
 {
   if (upd_del_plan)
-    upd_del_plan->print_explain_json(this, writer, is_analyze, is_show_cmd);
+    upd_del_plan->print_explain_json(this, writer, is_analyze);
   else if (insert_plan)
-    insert_plan->print_explain_json(this, writer, is_analyze, is_show_cmd);
+    insert_plan->print_explain_json(this, writer, is_analyze);
   else
   {
     /* Start printing from root node with id=1 */
     Explain_node *node= get_node(1);
     if (!node)
       return false; /* No query plan */
-    node->print_explain_json(this, writer, is_analyze, is_show_cmd);
+    node->print_explain_json(this, writer, is_analyze);
   }
 
   return true;
@@ -788,13 +775,12 @@ int Explain_union::print_explain_pushed_down(select_result_sink *output,
 
 
 void Explain_union::print_explain_json(Explain_query *query,
-                                       Json_writer *writer, bool is_analyze,
-                                       bool no_tmp_tbl)
+                                       Json_writer *writer, bool is_analyze)
 {
   if (is_pushed_down_to_engine)
-    print_explain_json_pushed_down(query, writer, is_analyze, no_tmp_tbl);
+    print_explain_json_pushed_down(query, writer, is_analyze);
   else
-    print_explain_json_regular(query, writer, is_analyze, no_tmp_tbl);
+    print_explain_json_regular(query, writer, is_analyze);
 }
 
 /*
@@ -803,7 +789,7 @@ void Explain_union::print_explain_json(Explain_query *query,
 */
 
 void Explain_union::print_explain_json_regular(
-  Explain_query *query, Json_writer *writer, bool is_analyze, bool no_tmp_tbl)
+  Explain_query *query, Json_writer *writer, bool is_analyze)
 {
   Json_writer_nesting_guard guard(writer);
   char table_name_buffer[SAFE_NAME_LEN];
@@ -848,12 +834,12 @@ void Explain_union::print_explain_json_regular(
     //writer->add_member("dependent").add_str("TODO");
     //writer->add_member("cacheable").add_str("TODO");
     Explain_select *sel= query->get_select(union_members.at(i));
-    sel->print_explain_json(query, writer, is_analyze, no_tmp_tbl);
+    sel->print_explain_json(query, writer, is_analyze);
     writer->end_object();
   }
   writer->end_array();
 
-  print_explain_json_for_children(query, writer, is_analyze, no_tmp_tbl);
+  print_explain_json_for_children(query, writer, is_analyze);
 
   writer->end_object(); // union_result
   writer->end_object(); // query_block
@@ -869,8 +855,7 @@ void Explain_union::print_explain_json_regular(
 
 void Explain_union::print_explain_json_pushed_down(Explain_query *query,
                                                    Json_writer *writer,
-                                                   bool is_analyze,
-                                                   bool no_tmp_tbl)
+                                                   bool is_analyze)
 {
   Json_writer_nesting_guard guard(writer);
 
@@ -940,8 +925,7 @@ bool is_connection_printable_in_json(enum Explain_node::explain_connection_type
 
 void Explain_node::print_explain_json_for_children(Explain_query *query, 
                                                   Json_writer *writer,
-                                                  bool is_analyze,
-                                                  bool no_tmp_tbl)
+                                                  bool is_analyze)
 {
   Json_writer_nesting_guard guard(writer);
   
@@ -968,7 +952,7 @@ void Explain_node::print_explain_json_for_children(Explain_query *query,
     }
 
     writer->start_object();
-    node->print_explain_json(query, writer, is_analyze, no_tmp_tbl);
+    node->print_explain_json(query, writer, is_analyze);
     writer->end_object();
   }
 
@@ -1006,6 +990,18 @@ bool Explain_node::print_explain_json_cache(Json_writer *writer,
 }
 
 
+bool Explain_node::print_explain_json_subq_materialization(Json_writer *writer,
+                                                           bool is_analyze)
+{
+  if (subq_materialization)
+  {
+    subq_materialization->print_explain_json(writer, is_analyze);
+    return true;
+  }
+  return false;
+}
+
+
 Explain_basic_join::~Explain_basic_join()
 {
   if (join_tabs)
@@ -1148,12 +1144,13 @@ void Explain_select::add_linkage(Json_writer *writer)
 }
 
 void Explain_select::print_explain_json(Explain_query *query, 
-                                        Json_writer *writer, bool is_analyze,
-                                        bool no_tmp_tbl)
+                                        Json_writer *writer, bool is_analyze)
 {
   Json_writer_nesting_guard guard(writer);
   
   bool started_cache= print_explain_json_cache(writer, is_analyze);
+  bool started_subq_mat= print_explain_json_subq_materialization(writer,
+                                                                 is_analyze);
 
   if (message ||
       select_type == pushed_derived_text ||
@@ -1171,7 +1168,7 @@ void Explain_select::print_explain_json(Explain_query *query,
                                           message);
     writer->end_object();
 
-    print_explain_json_for_children(query, writer, is_analyze, no_tmp_tbl);
+    print_explain_json_for_children(query, writer, is_analyze);
     writer->end_object();
   }
   else
@@ -1196,17 +1193,17 @@ void Explain_select::print_explain_json(Explain_query *query,
     if (exec_const_cond)
     {
       writer->add_member("const_condition");
-      write_item(writer, exec_const_cond, no_tmp_tbl);
+      write_item(writer, exec_const_cond);
     }
     if (outer_ref_cond)
     {
       writer->add_member("outer_ref_condition");
-      write_item(writer, outer_ref_cond, no_tmp_tbl);
+      write_item(writer, outer_ref_cond);
     }
     if (pseudo_bits_cond)
     {
       writer->add_member("pseudo_bits_condition");
-      write_item(writer, pseudo_bits_cond, no_tmp_tbl);
+      write_item(writer, pseudo_bits_cond);
     }
 
     /* we do not print HAVING which always evaluates to TRUE */
@@ -1214,7 +1211,7 @@ void Explain_select::print_explain_json(Explain_query *query,
     {
       writer->add_member("having_condition");
       if (likely(having))
-        write_item(writer, having, no_tmp_tbl);
+        write_item(writer, having);
       else
       {
         /* Normally we should not go this branch, left just for safety */
@@ -1238,7 +1235,7 @@ void Explain_select::print_explain_json(Explain_query *query,
         {
           writer->add_member("filesort").start_object();
           auto aggr_node= (Explain_aggr_filesort*)node;
-          aggr_node->print_json_members(writer, is_analyze, no_tmp_tbl);
+          aggr_node->print_json_members(writer, is_analyze);
           break;
         }
         case AGGR_OP_REMOVE_DUPLICATES:
@@ -1249,7 +1246,7 @@ void Explain_select::print_explain_json(Explain_query *query,
           //TODO: make print_json_members virtual?
           writer->add_member("window_functions_computation").start_object();
           auto aggr_node= (Explain_aggr_window_funcs*)node;
-          aggr_node->print_json_members(writer, is_analyze, no_tmp_tbl);
+          aggr_node->print_json_members(writer, is_analyze);
           break;
         }
         default:
@@ -1258,8 +1255,7 @@ void Explain_select::print_explain_json(Explain_query *query,
       started_objects++;
     }
     
-    Explain_basic_join::print_explain_json_interns(query, writer, is_analyze,
-                                                   no_tmp_tbl);
+    Explain_basic_join::print_explain_json_interns(query, writer, is_analyze);
 
     for (;started_objects; started_objects--)
       writer->end_object();
@@ -1267,6 +1263,8 @@ void Explain_select::print_explain_json(Explain_query *query,
     writer->end_object();
   }
 
+  if (started_subq_mat)
+    writer->end_object();
   if (started_cache)
     writer->end_object();
 }
@@ -1288,8 +1286,7 @@ Explain_aggr_filesort::Explain_aggr_filesort(MEM_ROOT *mem_root,
 
 
 void Explain_aggr_filesort::print_json_members(Json_writer *writer, 
-                                               bool is_analyze,
-                                               bool no_tmp_tbl)
+                                               bool is_analyze)
 {
   char item_buf[256];
   String str(item_buf, sizeof(item_buf), &my_charset_bin);
@@ -1309,7 +1306,7 @@ void Explain_aggr_filesort::print_json_members(Json_writer *writer,
     {
       str.append(STRING_WITH_LEN(", "));
     }
-    append_item_to_str(&str, item, no_tmp_tbl);
+    append_item_to_str(&str, item);
     if (*direction == ORDER::ORDER_DESC)
       str.append(STRING_WITH_LEN(" desc"));
   }
@@ -1322,8 +1319,7 @@ void Explain_aggr_filesort::print_json_members(Json_writer *writer,
 
 
 void Explain_aggr_window_funcs::print_json_members(Json_writer *writer, 
-                                                   bool is_analyze, 
-                                                   bool no_tmp_tbl)
+                                                   bool is_analyze)
 {
   Explain_aggr_filesort *srt;
   List_iterator<Explain_aggr_filesort> it(sorts);
@@ -1332,19 +1328,19 @@ void Explain_aggr_window_funcs::print_json_members(Json_writer *writer,
   {
     Json_writer_object sort(writer);
     Json_writer_object filesort(writer, "filesort");
-    srt->print_json_members(writer, is_analyze, no_tmp_tbl);
+    srt->print_json_members(writer, is_analyze);
   }
 }
 
 
 void Explain_basic_join::print_explain_json(Explain_query *query, 
                                             Json_writer *writer, 
-                                            bool is_analyze, bool no_tmp_tbl)
+                                            bool is_analyze)
 {
   writer->add_member("query_block").start_object();
   writer->add_member("select_id").add_ll(select_id);
   
-  print_explain_json_interns(query, writer, is_analyze, no_tmp_tbl);
+  print_explain_json_interns(query, writer, is_analyze);
 
   writer->end_object();
 }
@@ -1353,7 +1349,7 @@ void Explain_basic_join::print_explain_json(Explain_query *query,
 void Explain_basic_join::
 print_explain_json_interns(Explain_query *query, 
                            Json_writer *writer, 
-                           bool is_analyze, bool no_tmp_tbl)
+                           bool is_analyze)
 {
   {
     Json_writer_array loop(writer, "nested_loop");
@@ -1366,7 +1362,7 @@ print_explain_json_interns(Explain_query *query,
         writer->start_array();
       }
 
-      join_tabs[i]->print_explain_json(query, writer, is_analyze, no_tmp_tbl);
+      join_tabs[i]->print_explain_json(query, writer, is_analyze);
 
       if (join_tabs[i]->end_dups_weedout)
       {
@@ -1375,7 +1371,7 @@ print_explain_json_interns(Explain_query *query,
       }
     }
   } // "nested_loop"
-  print_explain_json_for_children(query, writer, is_analyze, no_tmp_tbl);
+  print_explain_json_for_children(query, writer, is_analyze);
 }
 
 
@@ -1777,7 +1773,7 @@ const char *String_list::append_str(MEM_ROOT *mem_root, const char *str)
 }
 
 
-static void write_item(Json_writer *writer, Item *item, bool no_tmp_tbl)
+static void write_item(Json_writer *writer, Item *item)
 {
   THD *thd= current_thd;
   char item_buf[256];
@@ -1787,27 +1783,25 @@ static void write_item(Json_writer *writer, Item *item, bool no_tmp_tbl)
   ulonglong save_option_bits= thd->variables.option_bits;
   thd->variables.option_bits &= ~OPTION_QUOTE_SHOW_CREATE;
 
-  auto qtype= QT_EXPLAIN | (no_tmp_tbl? QT_DONT_ACCESS_TMP_TABLES : 0);
-  item->print(&str, (enum_query_type)qtype);
+  item->print(&str, QT_EXPLAIN);
 
   thd->variables.option_bits= save_option_bits;
   writer->add_str(str.c_ptr_safe());
 }
 
-static void append_item_to_str(String *out, Item *item, bool no_tmp_tbl)
+static void append_item_to_str(String *out, Item *item)
 {
   THD *thd= current_thd;
   ulonglong save_option_bits= thd->variables.option_bits;
   thd->variables.option_bits &= ~OPTION_QUOTE_SHOW_CREATE;
 
-  auto qtype= QT_EXPLAIN | (no_tmp_tbl? QT_DONT_ACCESS_TMP_TABLES : 0);
-  item->print(out, (enum_query_type)qtype);
+  item->print(out, QT_EXPLAIN);
+
   thd->variables.option_bits= save_option_bits;
 }
 
 void Explain_table_access::tag_to_json(Json_writer *writer,
-                                       enum explain_extra_tag tag,
-                                       bool no_tmp_tbl)
+                                       enum explain_extra_tag tag)
 {
   switch (tag)
   {
@@ -1831,11 +1825,11 @@ void Explain_table_access::tag_to_json(Json_writer *writer,
       break;
     case ET_USING_INDEX_CONDITION:
       writer->add_member("index_condition");
-      write_item(writer, pushed_index_cond, no_tmp_tbl);
+      write_item(writer, pushed_index_cond);
       break;
     case ET_USING_INDEX_CONDITION_BKA:
       writer->add_member("index_condition_bka");
-      write_item(writer, pushed_index_cond, no_tmp_tbl);
+      write_item(writer, pushed_index_cond);
       break;
     case ET_USING_WHERE:
       {
@@ -1849,7 +1843,7 @@ void Explain_table_access::tag_to_json(Json_writer *writer,
         if (item)
         {
           writer->add_member("attached_condition");
-          write_item(writer, item, no_tmp_tbl);
+          write_item(writer, item);
         }
       }
       break;
@@ -1974,7 +1968,9 @@ static void trace_engine_stats(handler *file, Json_writer *writer)
       writer->add_member("pages_read_count").add_ull(hs->pages_read_count);
     if (hs->pages_read_time)
       writer->add_member("pages_read_time_ms").
-        add_double(hs->pages_read_time / 1000.0);
+        add_double(hs->pages_read_time * 1000. / timer_tracker_frequency());
+    if (hs->pages_prefetched)
+      writer->add_member("pages_prefetch_read_count").add_ull(hs->pages_prefetched);
     if (hs->undo_records_read)
       writer->add_member("old_rows_read").add_ull(hs->undo_records_read);
     writer->end_object();
@@ -1983,7 +1979,7 @@ static void trace_engine_stats(handler *file, Json_writer *writer)
 
 void Explain_table_access::print_explain_json(Explain_query *query,
                                               Json_writer *writer,
-                                              bool is_analyze, bool no_tmp_tbl)
+                                              bool is_analyze)
 {
   Json_writer_object jsobj(writer);
   
@@ -2014,7 +2010,7 @@ void Explain_table_access::print_explain_json(Explain_query *query,
       }
     }
     writer->add_member("filesort").start_object();
-    pre_join_sort->print_json_members(writer, is_analyze, no_tmp_tbl);
+    pre_join_sort->print_json_members(writer, is_analyze);
   }
 
   if (bka_type.is_using_jbuf())
@@ -2169,7 +2165,7 @@ void Explain_table_access::print_explain_json(Explain_query *query,
 
   for (int i=0; i < (int)extra_tags.elements(); i++)
   {
-    tag_to_json(writer, extra_tags.at(i), no_tmp_tbl);
+    tag_to_json(writer, extra_tags.at(i));
   }
   
   if (full_scan_on_null_key)
@@ -2190,7 +2186,7 @@ void Explain_table_access::print_explain_json(Explain_query *query,
     if (where_cond)
     {
       writer->add_member("attached_condition");
-      write_item(writer, where_cond, no_tmp_tbl);
+      write_item(writer, where_cond);
     }
 
     if (is_analyze)
@@ -2205,6 +2201,11 @@ void Explain_table_access::print_explain_json(Explain_query *query,
 
       writer->add_member("r_unpack_time_ms");
       writer->add_double(jbuf_unpack_tracker.get_time_ms());
+      DBUG_EXECUTE_IF("analyze_print_r_unpack_ops",
+                      {
+                        writer->add_member("r_unpack_ops");
+                        writer->add_ull(jbuf_unpack_tracker.get_loops());
+                      });
 
       writer->add_member("r_other_time_ms").
         add_double(jbuf_extra_time_tracker.get_time_ms());
@@ -2238,7 +2239,7 @@ void Explain_table_access::print_explain_json(Explain_query *query,
     {
       writer->add_member("lateral").add_ll(1);
     }
-    node->print_explain_json(query, writer, is_analyze, no_tmp_tbl);
+    node->print_explain_json(query, writer, is_analyze);
     writer->end_object();
   }
   if (non_merged_sjm_number)
@@ -2248,7 +2249,7 @@ void Explain_table_access::print_explain_json(Explain_query *query,
     writer->add_member("unique").add_ll(1);
     Explain_node *node= query->get_node(non_merged_sjm_number);
     node->connection_type= Explain_node::EXPLAIN_NODE_NON_MERGED_SJ;
-    node->print_explain_json(query, writer, is_analyze, no_tmp_tbl);
+    node->print_explain_json(query, writer, is_analyze);
     writer->end_object();
   }
   if (sjm_nest)
@@ -2256,7 +2257,7 @@ void Explain_table_access::print_explain_json(Explain_query *query,
     /* This is a non-merged semi-join table. Print its contents here */
     writer->add_member("materialized").start_object();
     writer->add_member("unique").add_ll(1);
-    sjm_nest->print_explain_json(query, writer, is_analyze, no_tmp_tbl);
+    sjm_nest->print_explain_json(query, writer, is_analyze);
     writer->end_object();
   }
 
@@ -2562,8 +2563,7 @@ int Explain_delete::print_explain(Explain_query *query,
 
 void Explain_delete::print_explain_json(Explain_query *query, 
                                         Json_writer *writer,
-                                        bool is_analyze,
-                                        bool no_tmp_tbl)
+                                        bool is_analyze)
 {
   Json_writer_nesting_guard guard(writer);
 
@@ -2578,7 +2578,7 @@ void Explain_delete::print_explain_json(Explain_query *query,
     writer->end_object(); // query_block
     return;
   }
-  Explain_update::print_explain_json(query, writer, is_analyze, no_tmp_tbl);
+  Explain_update::print_explain_json(query, writer, is_analyze);
 }
 
 
@@ -2681,8 +2681,7 @@ int Explain_update::print_explain(Explain_query *query,
 
 void Explain_update::print_explain_json(Explain_query *query,
                                         Json_writer *writer,
-                                        bool is_analyze,
-                                        bool no_tmp_tbl)
+                                        bool is_analyze)
 {
   Json_writer_nesting_guard guard(writer);
 
@@ -2849,7 +2848,7 @@ void Explain_update::print_explain_json(Explain_query *query,
   if (where_cond)
   {
     writer->add_member("attached_condition");
-    write_item(writer, where_cond, no_tmp_tbl);
+    write_item(writer, where_cond);
   }
 
   /*** The part of plan that is before the buffering/sorting ends here ***/
@@ -2861,7 +2860,7 @@ void Explain_update::print_explain_json(Explain_query *query,
 
   writer->end_object(); // table
 
-  print_explain_json_for_children(query, writer, is_analyze, no_tmp_tbl);
+  print_explain_json_for_children(query, writer, is_analyze);
   writer->end_object(); // query_block
 }
 
@@ -2891,8 +2890,7 @@ int Explain_insert::print_explain(Explain_query *query,
 }
 
 void Explain_insert::print_explain_json(Explain_query *query, 
-                                        Json_writer *writer, bool is_analyze,
-                                        bool no_tmp_tbl)
+                                        Json_writer *writer, bool is_analyze)
 {
   Json_writer_nesting_guard guard(writer);
 
@@ -2901,7 +2899,7 @@ void Explain_insert::print_explain_json(Explain_query *query,
   writer->add_member("table").start_object();
   writer->add_member("table_name").add_str(table_name.c_ptr());
   writer->end_object(); // table
-  print_explain_json_for_children(query, writer, is_analyze, no_tmp_tbl);
+  print_explain_json_for_children(query, writer, is_analyze);
   writer->end_object(); // query_block
 }
 
@@ -3021,3 +3019,41 @@ void Explain_range_checked_fer::print_json(Json_writer *writer,
     writer->end_object();
   }
 }
+
+
+void Explain_subq_materialization::print_explain_json(Json_writer *writer,
+                                                      bool is_analyze)
+{
+  writer->add_member("materialization").start_object();
+  if (is_analyze)
+    tracker.print_json_members(writer);
+}
+
+
+void Subq_materialization_tracker::print_json_members(Json_writer *writer) const
+{
+  writer->add_member("r_strategy").add_str(get_exec_strategy());
+  if (loops_count)
+    writer->add_member("r_loops").add_ull(loops_count);
+
+  if (index_lookups_count)
+    writer->add_member("r_index_lookups").add_ull(index_lookups_count);
+
+  if (partial_matches_count)
+    writer->add_member("r_partial_matches").add_ull(partial_matches_count);
+
+  if (partial_match_buffer_size)
+  {
+    writer->add_member("r_partial_match_buffer_size").
+            add_size(partial_match_buffer_size);
+  }
+
+  if (partial_match_array_sizes.elements())
+  {
+    writer->add_member("r_partial_match_array_sizes").start_array();
+    for(size_t i= 0; i < partial_match_array_sizes.elements(); i++)
+      writer->add_ull(partial_match_array_sizes[i]);
+    writer->end_array();
+  }
+}
+
