@@ -89,7 +89,11 @@ bool reload_acl_and_cache(THD *thd, unsigned long long options,
       allocate temporary THD for execution of acl_reload()/grant_reload().
     */
     if (unlikely(!thd) && (thd= (tmp_thd= new THD(0))))
+    {
       thd->store_globals();
+      thd->set_query_inner((char*) STRING_WITH_LEN("intern:reload_acl"),
+                           default_charset_info);
+    }
 
     if (likely(thd))
     {
@@ -618,7 +622,7 @@ bool flush_tables_with_read_lock(THD *thd, TABLE_LIST *all_tables)
       if (table_list->belong_to_view &&
           check_single_table_access(thd, PRIV_LOCK_TABLES, table_list, FALSE))
       {
-        table_list->replace_view_error_with_generic(thd);
+        table_list->hide_view_error(thd);
         goto error_reset_bits;
       }
       if (table_list->is_view_or_derived())
