@@ -1004,10 +1004,12 @@ class Time_zone_system : public Time_zone
 {
 public:
   Time_zone_system() = default;                       /* Remove gcc warning */
-  virtual my_time_t TIME_to_gmt_sec(const MYSQL_TIME *t, uint *error_code) const;
-  virtual void gmt_sec_to_TIME(MYSQL_TIME *tmp, my_time_t t) const;
-  virtual const String * get_name() const;
-  virtual void get_timezone_information(struct tz* curr_tz, const MYSQL_TIME *local_TIME) const;
+  my_time_t TIME_to_gmt_sec(const MYSQL_TIME *t,
+                            uint *error_code) const override;
+  void gmt_sec_to_TIME(MYSQL_TIME *tmp, my_time_t t) const override;
+  const String * get_name() const override;
+  void get_timezone_information(struct my_tz* curr_tz,
+                                const MYSQL_TIME *local_TIME) const override;
 };
 
 
@@ -1074,48 +1076,13 @@ Time_zone_system::gmt_sec_to_TIME(MYSQL_TIME *tmp, my_time_t t) const
   adjust_leap_second(tmp);
 }
 
+
 void
-Time_zone_system::get_timezone_information(struct tz* curr_tz, const MYSQL_TIME *local_TIME) const
+Time_zone_system::get_timezone_information(struct my_tz* curr_tz, const MYSQL_TIME *local_TIME) const
 {
-#ifndef _WIN32
   uint error;
-  struct tm tm_local_time;
-  time_t time_sec= TIME_to_gmt_sec(local_TIME,  &error);
-  localtime_r(&time_sec, &tm_local_time);
-  strmake_buf(curr_tz->abbrevation, tm_local_time.tm_zone);
-  curr_tz->seconds_offset= tm_local_time.tm_gmtoff;
-#else
-#define mdHMS(mon,day,h,m,s) (((((mon)*100+(day))*100+(h))*100+(m))*100+(s))
-  TIME_ZONE_INFORMATION tzi;
-  bool use_dst= false;
-  GetTimeZoneInformationForYear(local_TIME->year, NULL, &tzi);
-  if (tzi.StandardDate.wMonth)
-  {
-    ulonglong std= mdHMS(tzi.StandardDate.wMonth, tzi.StandardDate.wDay,
-                         tzi.StandardDate.wHour, tzi.StandardDate.wMinute,
-                         tzi.StandardDate.wSecond);
-    ulonglong dst= mdHMS(tzi.DaylightDate.wMonth, tzi.DaylightDate.wDay,
-                         tzi.DaylightDate.wHour, tzi.DaylightDate.wMinute,
-                         tzi.DaylightDate.wSecond);
-    ulonglong cur= mdHMS(local_TIME->month, local_TIME->day, local_TIME->hour,
-                        local_TIME->minute, local_TIME->second);
-    use_dst= std < dst ? cur < std || cur >= dst : cur >= dst && cur < std;
-  }
-  if (use_dst)
-  {
-    size_t len;
-    curr_tz->seconds_offset= -60 * (tzi.Bias + tzi.DaylightBias);
-    wcstombs_s(&len, curr_tz->abbrevation, sizeof(curr_tz->abbrevation),
-               tzi.DaylightName, _TRUNCATE);
-  }
-  else
-  {
-    size_t len;
-    curr_tz->seconds_offset= -60 * (tzi.Bias + tzi.StandardBias);
-    wcstombs_s(&len, curr_tz->abbrevation, sizeof(curr_tz->abbrevation),
-               tzi.StandardName, _TRUNCATE);
-  }
-#endif
+  time_t time_sec= TIME_to_gmt_sec(local_TIME, &error);
+  my_tzinfo(time_sec, curr_tz);
 }
 
 
@@ -1145,11 +1112,12 @@ class Time_zone_utc : public Time_zone
 {
 public:
   Time_zone_utc() = default;                          /* Remove gcc warning */
-  virtual my_time_t TIME_to_gmt_sec(const MYSQL_TIME *t,
-                                    uint *error_code) const;
-  virtual void gmt_sec_to_TIME(MYSQL_TIME *tmp, my_time_t t) const;
-  virtual const String * get_name() const;
-  virtual void get_timezone_information(struct tz* curr_tz, const MYSQL_TIME *local_TIME) const;
+  my_time_t TIME_to_gmt_sec(const MYSQL_TIME *t,
+                            uint *error_code) const override;
+  void gmt_sec_to_TIME(MYSQL_TIME *tmp, my_time_t t) const override;
+  const String * get_name() const override;
+  void get_timezone_information(struct my_tz* curr_tz,
+                                const MYSQL_TIME *local_TIME) const override;
 };
 
 
@@ -1198,9 +1166,9 @@ Time_zone_utc::gmt_sec_to_TIME(MYSQL_TIME *tmp, my_time_t t) const
 }
 
 void
-Time_zone_utc::get_timezone_information(struct tz* curr_tz, const MYSQL_TIME *local_TIME) const
+Time_zone_utc::get_timezone_information(struct my_tz* curr_tz, const MYSQL_TIME *local_TIME) const
 {
-  strmake_buf(curr_tz->abbrevation, "UTC");
+  strmake_buf(curr_tz->abbreviation, "UTC");
   curr_tz->seconds_offset= 0;
 }
 
@@ -1236,10 +1204,12 @@ class Time_zone_db : public Time_zone
 {
 public:
   Time_zone_db(TIME_ZONE_INFO *tz_info_arg, const String * tz_name_arg);
-  virtual my_time_t TIME_to_gmt_sec(const MYSQL_TIME *t, uint *error_code) const;
-  virtual void gmt_sec_to_TIME(MYSQL_TIME *tmp, my_time_t t) const;
-  virtual const String * get_name() const;
-  virtual void get_timezone_information(struct tz* curr_tz, const MYSQL_TIME *local_TIME) const;
+  my_time_t TIME_to_gmt_sec(const MYSQL_TIME *t,
+                            uint *error_code) const override;
+  void gmt_sec_to_TIME(MYSQL_TIME *tmp, my_time_t t) const override;
+  const String * get_name() const override;
+  void get_timezone_information(struct my_tz* curr_tz,
+                                const MYSQL_TIME *local_TIME) const override;
 private:
   TIME_ZONE_INFO *tz_info;
   const String *tz_name;
@@ -1311,7 +1281,7 @@ Time_zone_db::gmt_sec_to_TIME(MYSQL_TIME *tmp, my_time_t t) const
 }
 
 void
-Time_zone_db::get_timezone_information(struct tz* curr_tz, const MYSQL_TIME *local_TIME) const
+Time_zone_db::get_timezone_information(struct my_tz* curr_tz, const MYSQL_TIME *local_TIME) const
 {
   uint error;
   my_time_t sec_in_utc;
@@ -1323,7 +1293,7 @@ Time_zone_db::get_timezone_information(struct tz* curr_tz, const MYSQL_TIME *loc
   ttisp= find_transition_type(sec_in_utc, tz_info);
 
   curr_tz->seconds_offset= ttisp->tt_gmtoff;
-  strmake_buf(curr_tz->abbrevation, &(tz_info->chars[ttisp->tt_abbrind]));
+  strmake_buf(curr_tz->abbreviation, &(tz_info->chars[ttisp->tt_abbrind]));
 }
 
 
@@ -1351,11 +1321,12 @@ class Time_zone_offset : public Time_zone
 {
 public:
   Time_zone_offset(long tz_offset_arg);
-  virtual my_time_t TIME_to_gmt_sec(const MYSQL_TIME *t,
-                                    uint *error_code) const;
-  virtual void   gmt_sec_to_TIME(MYSQL_TIME *tmp, my_time_t t) const;
-  virtual const String * get_name() const;
-  virtual void get_timezone_information(struct tz* curr_tz, const MYSQL_TIME *local_TIME) const;
+  my_time_t TIME_to_gmt_sec(const MYSQL_TIME *t,
+                            uint *error_code) const override;
+  void gmt_sec_to_TIME(MYSQL_TIME *tmp, my_time_t t) const override;
+  const String * get_name() const override;
+  void get_timezone_information(struct my_tz* curr_tz,
+                                const MYSQL_TIME *local_TIME) const override;
   /*
     This have to be public because we want to be able to access it from
     my_offset_tzs_get_key() function
@@ -1482,11 +1453,11 @@ Time_zone_offset::get_name() const
 }
 
 void
-Time_zone_offset::get_timezone_information(struct tz* curr_tz, const MYSQL_TIME *local_TIME) const
+Time_zone_offset::get_timezone_information(struct my_tz* curr_tz, const MYSQL_TIME *local_TIME) const
 {
   curr_tz->seconds_offset= offset;
   const char *name= get_name()->ptr();
-  strmake_buf(curr_tz->abbrevation, name);
+  strmake_buf(curr_tz->abbreviation, name);
 }
 
 
@@ -1553,21 +1524,20 @@ public:
   they should obey C calling conventions.
 */
 
-extern "C" uchar *
-my_tz_names_get_key(Tz_names_entry *entry, size_t *length,
-                    my_bool not_used __attribute__((unused)))
+static const uchar *my_tz_names_get_key(const void *entry_, size_t *length,
+                                            my_bool)
 {
+  auto entry= static_cast<const Tz_names_entry *>(entry_);
   *length= entry->name.length();
-  return (uchar*) entry->name.ptr();
+  return reinterpret_cast<const uchar *>(entry->name.ptr());
 }
 
-extern "C" uchar *
-my_offset_tzs_get_key(Time_zone_offset *entry,
-                      size_t *length,
-                      my_bool not_used __attribute__((unused)))
+static const uchar *my_offset_tzs_get_key(const void *entry_,
+                                             size_t *length, my_bool)
 {
+  auto entry= static_cast<const Time_zone_offset *>(entry_);
   *length= sizeof(long);
-  return (uchar*) &entry->offset;
+  return reinterpret_cast<const uchar *>(&entry->offset);
 }
 
 
@@ -1676,20 +1646,20 @@ my_tz_init(THD *org_thd, const char *default_tzname, my_bool bootstrap)
   */
   if (!(thd= new THD(0)))
     DBUG_RETURN(1);
-  thd->thread_stack= (char*) &thd;
+  thd->thread_stack= (void*) &thd;              // Big stack
   thd->store_globals();
   thd->set_query_inner((char*) STRING_WITH_LEN("intern:my_tz_init"),
                        default_charset_info);
 
   /* Init all memory structures that require explicit destruction */
   if (my_hash_init(key_memory_tz_storage, &tz_names, &my_charset_latin1, 20, 0,
-                   0, (my_hash_get_key) my_tz_names_get_key, 0, 0))
+                   0, my_tz_names_get_key, 0, 0))
   {
     sql_print_error("Fatal error: OOM while initializing time zones");
     goto end;
   }
   if (my_hash_init(key_memory_tz_storage, &offset_tzs, &my_charset_latin1, 26,
-                   0, 0, (my_hash_get_key)my_offset_tzs_get_key, 0, 0))
+                   0, 0, my_offset_tzs_get_key, 0, 0))
   {
     sql_print_error("Fatal error: OOM while initializing time zones");
     my_hash_free(&tz_names);
@@ -1934,6 +1904,7 @@ tz_load_from_open_tables(const String *tz_name, TABLE_LIST *tz_tables)
   */
   TIME_ZONE_INFO tmp_tz_info;
   memset(&tmp_tz_info, 0, sizeof(TIME_ZONE_INFO));
+  memset(ttis, 0, sizeof(ttis));
 
   DBUG_ENTER("tz_load_from_open_tables");
 
@@ -2425,6 +2396,59 @@ void Time_zone::adjust_leap_second(MYSQL_TIME *t)
     t->second= 59;
 }
 
+
+/*
+  @brief
+    Check if timestamp<->datetime conversion is guaranteed to be reversible
+    around the given time value.
+*/
+
+bool Time_zone::is_monotone_continuous_around(my_time_t sec) const
+{
+  const my_time_t width= 24 * 60 * 60;
+  /*
+    Let's check that the TIMESTAMP range [sec - width, sec + width]
+    converts back to a DATETIME range [dtmin, dtmax], and the difference
+    (dtmax-dtmin), calculated using DATETIME arithmetics,
+    is also equal to exactly 2*width.
+
+    This should almost guarantee that the TIMESTAMP range around sec
+    is monotone and continuous and thus has no DST changes or leap seconds.
+
+    It's still possible that there are two or more timezone offset changes
+    in the given range. But there are no such time zones according to
+    our knowledge.
+    Note, DST changes and leap seconds do not interfere on a short time_t range:
+    - Leap seconds happen in winter and summer
+    - DST changes happen in spring and fall
+
+    The largest time zone change that ever happened in a single place
+    was 24 hours:
+    - from SST (UTC-11) - the American Samoa Time Zone
+    - to WST (UTC+13) - the Independent State of Samoa Time Zone
+    The Independent State of Samoa used SST until it moved across
+    the International Date Line at the end of 29 December 2011, to WST.
+    It is now 24 hours ahead of American Samoa
+    (and 25 hours in Southern hemisphere summer).
+    Let's use 24 hours as width (48 hours range total).
+  */
+
+  if (sec < width || sec > TIMESTAMP_MAX_VALUE - width)
+    return false;
+
+  MYSQL_TIME dtmin, dtmax;
+  gmt_sec_to_TIME(&dtmin, sec - width);
+  gmt_sec_to_TIME(&dtmax, sec + width);
+
+  ulonglong seconds;
+  ulong useconds;
+  if (calc_time_diff(&dtmax, &dtmin, 1, &seconds, &useconds))
+    return false; // dtmax is smaller than dtmin, should not happen.
+  if (seconds != (ulonglong) width * 2)
+    return false; // Anomalies found (DST changes or leap seconds)
+  return true;
+}
+
 #endif /* !defined(TESTTIME) && !defined(TZINFO2SQL) */
 
 
@@ -2764,6 +2788,8 @@ static const char *lock_tables=
   "  time_zone_transition WRITE,\n"
   "  time_zone_transition_type WRITE";
 static const char *trunc_tables_const=
+  "SET @old_alter_alg=@@SESSION.alter_algorithm;\n"
+  "SET session alter_algorithm='COPY';\n"
   "TRUNCATE TABLE time_zone;\n"
   "TRUNCATE TABLE time_zone_name;\n"
   "TRUNCATE TABLE time_zone_transition;\n"
@@ -2808,7 +2834,7 @@ main(int argc, char **argv)
 
   printf("set @wsrep_is_on=(%s);\n", wsrep_is_on);
   printf("SET STATEMENT SQL_MODE='' FOR "
-         "SELECT concat('%%', GROUP_CONCAT(OPTION), '%%') INTO @replicate_opt "
+         "SELECT concat('%%', GROUP_CONCAT(OPTION ORDER BY OPTION DESC), '%%') INTO @replicate_opt "
          " FROM"
          "   (SELECT DISTINCT concat('REPLICATE_', UPPER(ENGINE)) AS OPTION"
          "    FROM information_schema.TABLES"
@@ -2819,8 +2845,7 @@ main(int argc, char **argv)
          "                         'time_zone_transition_type',"
          "                         'time_zone_leap_second')"
          "      AND ENGINE in ('MyISAM',"
-         "                     'Aria')) AS o"
-         " ORDER BY OPTION DESC;\n");
+         "                     'Aria')) AS o;\n");
   printf("set @wsrep_cannot_replicate_tz=@wsrep_is_on AND (%s);\n", wsrep_cannot_replicate_tz);
   if (opt_skip_write_binlog)
     /* We turn off session wsrep if we cannot replicate using galera.
@@ -2912,6 +2937,9 @@ main(int argc, char **argv)
         "concat('ALTER TABLE time_zone_transition_type ENGINE=', "
 	  "@time_zone_transition_type_engine, ', ORDER BY Time_zone_id, Transition_type_id'), 'do 0');\n");
 
+  if (argc == 1 && !opt_leap)
+    printf("SET session alter_algorithm=@old_alter_alg;\n");
+  
   free_allocated_data();
   my_end(0);
   return 0;
