@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2018, MariaDB Corporation.
+   Copyright (c) 2018, 2020, MariaDB Corporation.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -17,6 +17,8 @@
 #ifndef VERS_STRING_INCLUDED
 #define VERS_STRING_INCLUDED
 
+#include "lex_string.h"
+
 /*
   LEX_CSTRING with comparison semantics.
 */
@@ -28,9 +30,8 @@ struct Compare_table_names
   {
     DBUG_ASSERT(a.str[a.length] == 0);
     DBUG_ASSERT(b.str[b.length] == 0);
-    return my_strnncoll(table_alias_charset,
-                        (uchar*)a.str, a.length,
-                        (uchar*)b.str, b.length);
+    return table_alias_charset->strnncoll(a.str, a.length,
+                                          b.str, b.length);
   }
 };
 
@@ -39,43 +40,23 @@ struct Compare_identifiers
 {
   int operator()(const LEX_CSTRING& a, const LEX_CSTRING& b) const
   {
+    DBUG_ASSERT(a.str != NULL);
+    DBUG_ASSERT(b.str != NULL);
     DBUG_ASSERT(a.str[a.length] == 0);
     DBUG_ASSERT(b.str[b.length] == 0);
     return my_strcasecmp(system_charset_info, a.str, b.str);
   }
 };
 
-class Lex_cstring : public LEX_CSTRING
-{
-  public:
-  Lex_cstring()
-  {
-    str= NULL;
-    length= 0;
-  }
-  Lex_cstring(const char *_str, size_t _len)
-  {
-    str= _str;
-    length= _len;
-  }
-  void set(const char *_str, size_t _len)
-  {
-    str= _str;
-    length= _len;
-  }
-};
 
 template <class Compare>
 struct Lex_cstring_with_compare : public Lex_cstring
 {
 public:
-  Lex_cstring_with_compare() {}
+  Lex_cstring_with_compare() = default;
   Lex_cstring_with_compare(const char *_str, size_t _len) :
     Lex_cstring(_str, _len)
   { }
-  Lex_cstring_with_compare(const LEX_STRING src) :
-    Lex_cstring(src.str, src.length)
-  { }
   Lex_cstring_with_compare(const LEX_CSTRING src) : Lex_cstring(src.str, src.length)
   { }
   Lex_cstring_with_compare(const char *_str) : Lex_cstring(_str, strlen(_str))
