@@ -359,7 +359,8 @@ bool wsrep_sst_donor_update (sys_var *self, THD* thd, enum_var_type type)
 
 bool wsrep_before_SE()
 {
-  return (wsrep_provider != NULL
+  return (wsrep_provider
+          && *wsrep_provider
           && strcmp (wsrep_provider,   WSREP_NONE)
           && strcmp (wsrep_sst_method, WSREP_SST_SKIP)
           && strcmp (wsrep_sst_method, WSREP_SST_MYSQLDUMP));
@@ -648,8 +649,8 @@ static void* sst_joiner_thread (void* a)
       if (!tmp || strlen(tmp) < (magic_len + 2) ||
           strncasecmp (tmp, magic, magic_len))
       {
-        WSREP_ERROR("Failed to read '%s <addr>' from: %s\n\tRead: '%s'",
-                    magic, arg->cmd, tmp);
+        WSREP_ERROR("Failed to read '%s <addr>' (got '%s') from: %s",
+                    magic, tmp, arg->cmd);
         proc.wait();
         if (proc.error()) err= proc.error();
       }
@@ -661,8 +662,8 @@ static void* sst_joiner_thread (void* a)
     else
     {
       err= proc.error();
-      WSREP_ERROR("Failed to execute: %s : %d (%s)",
-                  arg->cmd, err, strerror(err));
+      WSREP_ERROR("Failed to execute (%M): %s",
+                  err, arg->cmd);
     }
 
     /*
@@ -820,7 +821,6 @@ err:
       unireg_abort(1);
     }
 
-    thd->thread_stack= (char*) &thd;
     thd->security_ctx->skip_grants();
     thd->system_thread= SYSTEM_THREAD_GENERIC;
     thd->real_id= pthread_self();
@@ -1602,11 +1602,13 @@ static int sst_donate_mysqldump (const char*         addr,
                      WSREP_SST_OPT_PORT " '%u' "
                      WSREP_SST_OPT_LPORT " '%u' "
                      WSREP_SST_OPT_SOCKET " '%s' "
+                     WSREP_SST_OPT_DATA " '%s' "
                      "%s"
                      WSREP_SST_OPT_GTID " '%s:%lld,%d-%d-%llu' "
                      WSREP_SST_OPT_GTID_DOMAIN_ID " '%d'"
                      "%s",
                      addr, port, mysqld_port, mysqld_unix_port,
+                     mysql_real_data_home,
                      wsrep_defaults_file,
                      uuid_oss.str().c_str(), gtid.seqno().get(),
                      wsrep_gtid_server.domain_id, wsrep_gtid_server.server_id,
@@ -1943,7 +1945,8 @@ wait_signal:
       else
       {
         WSREP_WARN("Received unknown signal: '%s'", out);
-        err = -EINVAL;
+        /* since it is the end of the loop, we must set error code */
+        err=-EINVAL;
         proc.wait();
       }
     }
