@@ -1,4 +1,4 @@
-/* Copyright 2008-2025 Codership Oy <http://www.codership.com>
+/* Copyright 2008-2022 Codership Oy <http://www.codership.com>
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -27,8 +27,7 @@
 #include <cstdlib>
 #include "wsrep_trans_observer.h"
 #include "wsrep_server_state.h"
-#include "wsrep_mysqld.h"
-#include "wsrep_schema.h"
+#include "wsrep_plugin.h" /* wsrep_provider_plugin_is_enabled() */
 
 ulong   wsrep_reject_queries;
 
@@ -95,12 +94,17 @@ static bool refresh_provider_options()
   }
 }
 
+bool wsrep_refresh_provider_options()
+{
+  return refresh_provider_options();
+}
+
 void wsrep_set_wsrep_on(THD* thd)
 {
   if (thd)
     thd->wsrep_was_on= WSREP_ON_;
-  WSREP_PROVIDER_EXISTS_= wsrep_provider &&
-    strncasecmp(wsrep_provider, WSREP_NONE, FN_REFLEN);
+  WSREP_PROVIDER_EXISTS_= wsrep_provider && *wsrep_provider &&
+    strcasecmp(wsrep_provider, WSREP_NONE);
   WSREP_ON_= global_system_variables.wsrep_on && WSREP_PROVIDER_EXISTS_;
 }
 
@@ -110,10 +114,14 @@ bool wsrep_on_update (sys_var *self, THD* thd, enum_var_type var_type)
   {
     my_bool saved_wsrep_on= global_system_variables.wsrep_on;
 
-    thd->variables.wsrep_on= global_system_variables.wsrep_on;
+    thd->variables.wsrep_on= saved_wsrep_on;
 
     // If wsrep has not been inited we need to do it now
-    if (global_system_variables.wsrep_on && wsrep_provider && !wsrep_inited)
+    if (!wsrep_inited &&
+        saved_wsrep_on &&
+        wsrep_provider &&
+        *wsrep_provider &&
+        strcasecmp(wsrep_provider, WSREP_NONE))
     {
       // wsrep_init() rewrites provide if it fails
       char* tmp= strdup(wsrep_provider);
@@ -125,14 +133,6 @@ bool wsrep_on_update (sys_var *self, THD* thd, enum_var_type var_type)
         saved_wsrep_on= false;
       }
 
-      if (!wsrep_ready_get())
-      {
-        my_error(ER_GALERA_REPLICATION_NOT_SUPPORTED, MYF(0));
-        WSREP_INFO("Failed to start Galera replication. Please check your "
-                   "configuration.");
-        saved_wsrep_on= false;
-      }
-
       free(tmp);
       mysql_mutex_lock(&LOCK_global_system_variables);
     }
@@ -208,36 +208,6 @@ bool wsrep_on_check(sys_var *self, THD* thd, set_var* var)
   return false;
 }
 
-bool wsrep_causal_reads_update (sys_var *self, THD* thd, enum_var_type var_type)
-{
-  if (thd->variables.wsrep_causal_reads) {
-    thd->variables.wsrep_sync_wait |= WSREP_SYNC_WAIT_BEFORE_READ;
-  } else {
-    thd->variables.wsrep_sync_wait &= ~WSREP_SYNC_WAIT_BEFORE_READ;
-  }
-
-  // update global settings too.
-  if (global_system_variables.wsrep_causal_reads) {
-      global_system_variables.wsrep_sync_wait |= WSREP_SYNC_WAIT_BEFORE_READ;
-  } else {
-      global_system_variables.wsrep_sync_wait &= ~WSREP_SYNC_WAIT_BEFORE_READ;
-  }
-
-  return false;
-}
-
-bool wsrep_sync_wait_update (sys_var* self, THD* thd, enum_var_type var_type)
-{
-  thd->variables.wsrep_causal_reads= thd->variables.wsrep_sync_wait &
-          WSREP_SYNC_WAIT_BEFORE_READ;
-
-  // update global settings too
-  global_system_variables.wsrep_causal_reads= global_system_variables.wsrep_sync_wait &
-          WSREP_SYNC_WAIT_BEFORE_READ;
-
-  return false;
-}
-
 template<typename T>
 static T parse_value(char** startptr, char** endptr)
 {
@@ -363,12 +333,14 @@ bool wsrep_start_position_check (sys_var *self, THD* thd, set_var* var)
          var->save_result.string_value.length);
   start_pos_buf[var->save_result.string_value.length]= 0;
 
+
   WSREP_DEBUG("SST wsrep_start_position check for new position %s old %s",
-              start_pos_buf, wsrep_start_position);
+	     start_pos_buf, wsrep_start_position);
 
   // Verify the format.
   if (wsrep_start_position_verify(start_pos_buf)) return true;
 
+
   // Give error if position is updated when wsrep is not enabled or
   // provider is not loaded.
   if ((!WSREP_ON || !Wsrep_server_state::instance().is_provider_loaded())
@@ -455,6 +427,12 @@ static int wsrep_provider_verify (const char* provider_str)
 
 bool wsrep_provider_check (sys_var *self, THD* thd, set_var* var)
 {
+  if (wsrep_provider_plugin_enabled())
+  {
+    my_error(ER_INCORRECT_GLOBAL_LOCAL_VAR, MYF(0), var->var->name.str, "read only");
+    return true;
+  }
+
   char wsrep_provider_buf[FN_REFLEN];
 
   if ((! var->save_result.string_value.str) ||
@@ -546,6 +524,11 @@ bool wsrep_provider_options_check(sys_var *self, THD* thd, set_var* var)
     my_message(ER_WRONG_ARGUMENTS, "WSREP (galera) not started", MYF(0));
     return true;
   }
+  if (wsrep_provider_plugin_enabled())
+  {
+    my_error(ER_INCORRECT_GLOBAL_LOCAL_VAR, MYF(0), var->var->name.str, "read only");
+    return true;
+  }
   return false;
 }
 
@@ -675,7 +658,7 @@ bool wsrep_cluster_address_update (sys_var *self, THD* thd, enum_var_type type)
   {
     wsrep_create_rollbacker();
     WSREP_DEBUG("Cluster address update creating %ld applier threads running %lu",
-                wsrep_slave_threads, wsrep_running_applier_threads);
+	    wsrep_slave_threads, wsrep_running_applier_threads);
     wsrep_create_appliers(wsrep_slave_threads);
   }
   mysql_mutex_unlock(&LOCK_wsrep_cluster_config);
@@ -779,7 +762,7 @@ static void wsrep_slave_count_change_update ()
 {
   wsrep_slave_count_change = (wsrep_slave_threads - wsrep_running_applier_threads);
   WSREP_DEBUG("Change on slave threads: New %ld old %lu difference %d",
-              wsrep_slave_threads, wsrep_running_applier_threads, wsrep_slave_count_change);
+	  wsrep_slave_threads, wsrep_running_applier_threads, wsrep_slave_count_change);
 }
 
 bool wsrep_slave_threads_update (sys_var *self, THD* thd, enum_var_type type)
@@ -804,9 +787,9 @@ bool wsrep_slave_threads_update (sys_var *self, THD* thd, enum_var_type type)
     // Thread creation and execution is asyncronous, therefore we need
     // wait them to be started or error produced
     while (wsrep_running_applier_threads != (ulong)wsrep_slave_threads &&
-           !wsrep_thread_create_failed.load(std::memory_order_relaxed))
+	   !wsrep_thread_create_failed.load(std::memory_order_relaxed))
     {
-      my_sleep(1000);
+	    my_sleep(1000);
     }
 
     mysql_mutex_lock(&LOCK_global_system_variables);
@@ -887,47 +870,29 @@ bool wsrep_trx_fragment_size_check (sys_var *self, THD* thd, set_var* var)
 
   const ulong new_trx_fragment_size= var->value->val_uint();
 
-  if (new_trx_fragment_size)
-  {
-    // Give error if wsrep_trx_fragment_size is set and wsrep is disabled or
-    // provider is not loaded
-    if (!WSREP(thd) || !Wsrep_server_state::instance().is_provider_loaded())
-    {
-      push_warning (thd, Sql_condition::WARN_LEVEL_WARN,
-                    ER_WRONG_VALUE_FOR_VAR,
-                    "Cannot set 'wsrep_trx_fragment_size' to a value other than "
-                    "0 because wsrep is switched off.");
-      return true;
-    }
+  if (!WSREP(thd) && new_trx_fragment_size > 0) {
+    push_warning (thd, Sql_condition::WARN_LEVEL_WARN,
+                  ER_WRONG_VALUE_FOR_VAR,
+                  "Cannot set 'wsrep_trx_fragment_size' to a value other than "
+                  "0 because wsrep is switched off.");
+    return true;
+  }
 
-    if (!wsrep_provider_is_SR_capable())
-    {
-      push_warning (thd, Sql_condition::WARN_LEVEL_WARN,
-                    ER_WRONG_VALUE_FOR_VAR,
-                    "Cannot set 'wsrep_trx_fragment_size' to a value other than "
-                    "0 because the wsrep_provider does not support streaming "
-                    "replication.");
-      return true;
-    }
+  if (new_trx_fragment_size > 0 && !wsrep_provider_is_SR_capable()) {
+    push_warning (thd, Sql_condition::WARN_LEVEL_WARN,
+                  ER_WRONG_VALUE_FOR_VAR,
+                  "Cannot set 'wsrep_trx_fragment_size' to a value other than "
+                  "0 because the wsrep_provider does not support streaming "
+                  "replication.");
+    return true;
+  }
 
-    if (wsrep_protocol_version < 4)
-    {
-      push_warning (thd, Sql_condition::WARN_LEVEL_WARN,
-                    ER_WRONG_VALUE_FOR_VAR,
-                    "Cannot set 'wsrep_trx_fragment_size' to a value other than "
-                    "0 because cluster is not yet operating in Galera 4 mode.");
-      return true;
-    }
-
-    if (!ha_table_exists(thd, &WSREP_LEX_SCHEMA, &WSREP_LEX_STREAMING))
-    {
-      push_warning_printf(thd, Sql_condition::WARN_LEVEL_WARN,
-                          ER_WRONG_VALUE_FOR_VAR,
-                          "Cannot set 'wsrep_trx_fragment_size' to a value other than "
-                          "0 because streaming table %s.%s does not exists.",
-		          WSREP_LEX_SCHEMA.str, WSREP_LEX_STREAMING.str);
-      return true;
-    }
+  if (wsrep_protocol_version < 4  && new_trx_fragment_size > 0) {
+    push_warning (thd, Sql_condition::WARN_LEVEL_WARN,
+                  ER_WRONG_VALUE_FOR_VAR,
+                  "Cannot set 'wsrep_trx_fragment_size' to a value other than "
+                  "0 because cluster is not yet operating in Galera 4 mode.");
+    return true;
   }
 
   return false;
@@ -938,50 +903,28 @@ bool wsrep_trx_fragment_size_update(sys_var* self, THD *thd, enum_var_type)
   WSREP_DEBUG("wsrep_trx_fragment_size_update: %llu",
               thd->variables.wsrep_trx_fragment_size);
 
+  // Give error if wsrep_trx_fragment_size is set and wsrep is disabled or
+  // provider is not loaded
+  if (!WSREP_ON || !Wsrep_server_state::instance().is_provider_loaded())
+  {
+    push_warning (thd, Sql_condition::WARN_LEVEL_WARN,
+                  ER_WRONG_VALUE_FOR_VAR,
+                  "Cannot set 'wsrep_trx_fragment_size' because "
+                  "wsrep is switched off");
+    return true;
+  }
+
   if (thd->variables.wsrep_trx_fragment_size)
   {
     return thd->wsrep_cs().enable_streaming(
       wsrep_fragment_unit(thd->variables.wsrep_trx_fragment_unit),
       size_t(thd->variables.wsrep_trx_fragment_size));
   }
-  else if (wsrep_streaming_enabled(thd))
+  else
   {
     thd->wsrep_cs().disable_streaming();
+    return false;
   }
-  return false;
-}
-
-bool wsrep_trx_fragment_unit_check (sys_var *self, THD* thd, set_var* var)
-{
-  uint new_trx_fragment_unit= (uint)var->save_result.ulonglong_value;
-  // Do not allow setting fragment unit anything else than
-  // default if galera is not enabled and streaming replication
-  // table does not exists.
-  if (thd->variables.wsrep_trx_fragment_size ||
-      new_trx_fragment_unit != WSREP_FRAG_BYTES)
-  {
-    // Give error if wsrep_trx_fragment_unit is set and wsrep is disabled or
-    // provider is not loaded
-    if (!WSREP_ON || !Wsrep_server_state::instance().is_provider_loaded())
-    {
-      push_warning (thd, Sql_condition::WARN_LEVEL_WARN,
-                    ER_WRONG_VALUE_FOR_VAR,
-                   "Cannot set 'wsrep_trx_fragment_unit' because "
-                   "wsrep is switched off");
-      return true;
-    }
-
-    if (!ha_table_exists(thd, &WSREP_LEX_SCHEMA, &WSREP_LEX_STREAMING))
-    {
-      push_warning_printf(thd, Sql_condition::WARN_LEVEL_WARN,
-                          ER_WRONG_VALUE_FOR_VAR,
-                          "Cannot set 'wsrep_trx_fragment_unit' "
-                          "because streaming table %s.%s does not exists.",
-                          WSREP_LEX_SCHEMA.str, WSREP_LEX_STREAMING.str);
-      return true;
-    }
-  }
-  return false;
 }
 
 bool wsrep_trx_fragment_unit_update(sys_var* self, THD *thd, enum_var_type)
@@ -989,6 +932,17 @@ bool wsrep_trx_fragment_unit_update(sys_var* self, THD *thd, enum_var_type)
   WSREP_DEBUG("wsrep_trx_fragment_unit_update: %lu",
               thd->variables.wsrep_trx_fragment_unit);
 
+  // Give error if wsrep_trx_fragment_unit is set and wsrep is disabled or
+  // provider is not loaded
+  if (!WSREP_ON || !Wsrep_server_state::instance().is_provider_loaded())
+  {
+    push_warning (thd, Sql_condition::WARN_LEVEL_WARN,
+                  ER_WRONG_VALUE_FOR_VAR,
+                  "Cannot set 'wsrep_trx_fragment_unit' because "
+                  "wsrep is switched off");
+    return true;
+  }
+
   if (thd->variables.wsrep_trx_fragment_size)
   {
     return thd->wsrep_cs().enable_streaming(
@@ -1024,22 +978,6 @@ bool wsrep_max_ws_size_update(sys_var *self, THD *thd, enum_var_type)
 
 bool wsrep_mode_check(sys_var *self, THD* thd, set_var* var)
 {
-  ulonglong new_wsrep_mode= var->save_result.ulonglong_value;
-  ulonglong old_wsrep_mode= wsrep_mode;
-  wsrep_mode= new_wsrep_mode;
-  if (wsrep_check_mode(WSREP_MODE_REPLICATE_MYISAM) ||
-      wsrep_check_mode(WSREP_MODE_REPLICATE_ARIA))
-  {
-    if (!(wsrep_forced_binlog_format == BINLOG_FORMAT_UNSPEC ||
-          wsrep_forced_binlog_format == BINLOG_FORMAT_ROW))
-    {
-      my_message(ER_WRONG_ARGUMENTS, "wsrep_mode=[REPLICATE_MYISAM|REPLICATE_ARIA] "
-                 "can't be enabled if wsrep_forced_binlog != [NONE|ROW]", MYF(0));
-      wsrep_mode= old_wsrep_mode;
-      return true;
-    }
-  }
-  wsrep_mode= old_wsrep_mode;
   return false;
 }
 
@@ -1183,51 +1121,3 @@ bool wsrep_gtid_domain_id_update(sys_var* self, THD *thd, enum_var_type)
   return false;
 }
 
-bool wsrep_forced_binlog_format_check(sys_var *self, THD* thd, set_var* var)
-{
-  ulonglong new_forced_binlog_format= var->save_result.ulonglong_value;
-
-  if (!(new_forced_binlog_format == BINLOG_FORMAT_UNSPEC ||
-        new_forced_binlog_format == BINLOG_FORMAT_ROW))
-  {
-    if (wsrep_check_mode(WSREP_MODE_BINLOG_ROW_FORMAT_ONLY))
-    {
-      my_message(ER_WRONG_ARGUMENTS, "wsrep_forced_binlog_format=[MIXED|STATEMENT] can't be set "
-                 "if wsrep_mode=BINLOG_ROW_FORMAT_ONLY", MYF(0));
-      return true;
-    }
-
-    if (wsrep_check_mode(WSREP_MODE_REPLICATE_MYISAM) ||
-        wsrep_check_mode(WSREP_MODE_REPLICATE_ARIA))
-    {
-      my_message(ER_WRONG_ARGUMENTS, "wsrep_forced_binlog_format=[MIXED|STATEMENT] can't be set "
-                 "if wsrep_mode=[REPLICATE_MYISAM|REPLICATE_ARIA]", MYF(0));
-      return true;
-    }
-  }
-
-  return false;
-}
-
-bool wsrep_slave_threads_check (sys_var *self, THD* thd, set_var* var)
-{
-  ulonglong new_slave_threads= var->save_result.ulonglong_value;
-
-  if (!WSREP_ON)
-  {
-    my_message(ER_WRONG_ARGUMENTS, "WSREP (galera) not started", MYF(0));
-    return true;
-  }
-
-  if (new_slave_threads &&
-      Wsrep_server_state::instance().state() == wsrep::server_state::s_disconnected)
-  {
-    push_warning(thd, Sql_condition::WARN_LEVEL_WARN,
-                 ER_WRONG_VALUE_FOR_VAR,
-                 "Cannot set 'wsrep_slave_threads' because "
-                 "wsrep is disconnected");
-    return true;
-  }
-
-  return false;
-}
