@@ -57,7 +57,8 @@ using namespace messageqcpp;
 using namespace oam;
 namespace dmlpackageprocessor
 {
-DMLPackageProcessor::DMLResult DeletePackageProcessor::processPackage(dmlpackage::CalpontDMLPackage& cpackage)
+DMLPackageProcessor::DMLResult DeletePackageProcessor::processPackageInternal(
+    dmlpackage::CalpontDMLPackage& cpackage)
 {
   SUMMARY_INFO("DeletePackageProcessor::processPackage");
 
@@ -93,7 +94,7 @@ DMLPackageProcessor::DMLResult DeletePackageProcessor::processPackage(dmlpackage
   {
     logging::Message::Args args;
     logging::Message message(9);
-    args.add("Unknown error occured while getting unique number.");
+    args.add("Unknown error occurred while getting unique number.");
     message.format(args);
     result.result = DELETE_ERROR;
     result.message = message;
@@ -166,9 +167,6 @@ DMLPackageProcessor::DMLResult DeletePackageProcessor::processPackage(dmlpackage
 
           for (; i < numTries; i++)
           {
-#ifdef _MSC_VER
-            Sleep(rm_ts.tv_sec * 1000);
-#else
             struct timespec abs_ts;
 
             do
@@ -177,8 +175,6 @@ DMLPackageProcessor::DMLResult DeletePackageProcessor::processPackage(dmlpackage
               abs_ts.tv_nsec = rm_ts.tv_nsec;
             } while (nanosleep(&abs_ts, &rm_ts) < 0);
 
-#endif
-
             try
             {
               processID = ::getpid();
@@ -268,15 +264,22 @@ DMLPackageProcessor::DMLResult DeletePackageProcessor::processPackage(dmlpackage
   }
   catch (exception& ex)
   {
-    cerr << "DeletePackageProcessor::processPackage: " << ex.what() << endl;
-
-    //@Bug 4994 Cancelled job is not error
-    if (result.result == 0)
+    if (checkPPLostConnection(ex))
     {
-      result.result = DELETE_ERROR;
+      result.result = PP_LOST_CONNECTION;
     }
+    else
+    {
+      cerr << "DeletePackageProcessor::processPackage: " << ex.what() << endl;
 
-    result.message = Message(ex.what());
+      //@Bug 4994 Cancelled job is not error
+      if (result.result == 0)
+      {
+        result.result = DELETE_ERROR;
+      }
+
+      result.message = Message(ex.what());
+    }
   }
   catch (...)
   {
