@@ -22,10 +22,13 @@
 
 #pragma once
 
+#include <ostream>
 #include <string>
 #include <iostream>
 #include <cmath>
 #include <boost/shared_ptr.hpp>
+#include <vector>
+#include <unordered_set>
 
 #include <stdlib.h>
 #include <unistd.h>
@@ -37,6 +40,7 @@
 #include "mcs_decimal.h"
 #include "mcs_int64.h"
 #include "numericliteral.h"
+#include "nullstring.h"
 
 namespace messageqcpp
 {
@@ -56,6 +60,7 @@ namespace execplan
 typedef execplan::CalpontSystemCatalog::ColType Type;
 typedef datatypes::Decimal IDB_Decimal;
 
+using IncludeSet = std::unordered_set<std::string>;
 /**
  * @brief IDB_Regex struct
  *
@@ -141,7 +146,7 @@ struct Result
    , longDoubleVal(0)
    , floatVal(0)
    , boolVal(false)
-   , strVal("")
+   , strVal()
    , decimalVal(IDB_Decimal())
    , valueConverted(false)
   {
@@ -156,7 +161,7 @@ struct Result
   long double longDoubleVal;
   float floatVal;
   bool boolVal;
-  std::string strVal;
+  utils::NullString strVal;
   IDB_Decimal decimalVal;
   bool valueConverted;
 };
@@ -209,6 +214,7 @@ class TreeNode
    */
   virtual bool operator!=(const TreeNode* t) const = 0;
 
+  virtual std::string toCppCode(IncludeSet& includes) const = 0;
   // derivedTable mutator and accessor
   virtual const std::string& derivedTable() const
   {
@@ -261,8 +267,9 @@ class TreeNode
   /***********************************************************************
    *                     F&E framework                                   *
    ***********************************************************************/
-  virtual const std::string& getStrVal(rowgroup::Row& row, bool& isNull)
+  virtual const utils::NullString& getStrVal(rowgroup::Row& row, bool& isNull)
   {
+    isNull = isNull || fResult.strVal.isNull(); // XXX: NullString returns isNull, we should remove that parameter altogether.
     return fResult.strVal;
   }
   virtual int64_t getIntVal(rowgroup::Row& row, bool& isNull)
@@ -326,7 +333,7 @@ class TreeNode
   }
 
   inline bool getBoolVal();
-  inline const std::string& getStrVal(const long timeZone);
+  inline const utils::NullString& getStrVal(const long timeZone);
   inline int64_t getIntVal();
   inline uint64_t getUintVal();
   inline float getFloatVal();
@@ -404,13 +411,15 @@ inline bool TreeNode::getBoolVal()
       if (fResultType.colWidth <= 8)
         return (atoi((char*)(&fResult.origIntVal)) != 0);
 
-      return (atoi(fResult.strVal.c_str()) != 0);
+      idbassert(fResult.strVal.str());
+      return (atoi(fResult.strVal.str()) != 0);
 
     case CalpontSystemCatalog::VARCHAR:
       if (fResultType.colWidth <= 7)
         return (atoi((char*)(&fResult.origIntVal)) != 0);
 
-      return (atoi(fResult.strVal.c_str()) != 0);
+      idbassert(fResult.strVal.str());
+      return (atoi(fResult.strVal.str()) != 0);
 
     // FIXME: Huh???
     case CalpontSystemCatalog::VARBINARY:
@@ -419,7 +428,8 @@ inline bool TreeNode::getBoolVal()
       if (fResultType.colWidth <= 7)
         return (atoi((char*)(&fResult.origIntVal)) != 0);
 
-      return (atoi(fResult.strVal.c_str()) != 0);
+      idbassert(fResult.strVal.str());
+      return (atoi(fResult.strVal.str()) != 0);
 
     case CalpontSystemCatalog::BIGINT:
     case CalpontSystemCatalog::SMALLINT:
@@ -458,28 +468,28 @@ inline bool TreeNode::getBoolVal()
   return fResult.boolVal;
 }
 
-inline const std::string& TreeNode::getStrVal(const long timeZone)
+inline const utils::NullString& TreeNode::getStrVal(const long timeZone)
 {
   switch (fResultType.colDataType)
   {
-    case CalpontSystemCatalog::CHAR:
-      if (fResultType.colWidth <= 8)
-        fResult.strVal = (char*)(&fResult.origIntVal);
-
-      break;
-
     case CalpontSystemCatalog::VARCHAR:
       if (fResultType.colWidth <= 7)
-        fResult.strVal = (char*)(&fResult.origIntVal);
+      {
+        const char *intAsChar = (const char*) (&fResult.origIntVal);
+        fResult.strVal.assign((const uint8_t*)intAsChar, strlen(intAsChar));
+      }
 
       break;
 
-    // FIXME: ???
-    case CalpontSystemCatalog::VARBINARY:
+    case CalpontSystemCatalog::CHAR:
+    case CalpontSystemCatalog::VARBINARY: // XXX: TODO: we don't have varbinary support now, but it may be handled just like varchar.
     case CalpontSystemCatalog::BLOB:
     case CalpontSystemCatalog::TEXT:
-      if (fResultType.colWidth <= 7)
-        fResult.strVal = (char*)(&fResult.origIntVal);
+      if (fResultType.colWidth <= 8)
+      {
+        const char *intAsChar = (const char*) (&fResult.origIntVal);
+        fResult.strVal.assign((const uint8_t*)intAsChar, strlen(intAsChar));
+      }
 
       break;
 
@@ -494,7 +504,7 @@ inline const std::string& TreeNode::getStrVal(const long timeZone)
 #else
       snprintf(tmp, 20, "%ld", fResult.intVal);
 #endif
-      fResult.strVal = std::string(tmp);
+      fResult.strVal.assign(std::string(tmp));
       break;
     }
 
@@ -509,7 +519,7 @@ inline const std::string& TreeNode::getStrVal(const long timeZone)
 #else
       snprintf(tmp, 20, "%lu", fResult.uintVal);
 #endif
-      fResult.strVal = std::string(tmp);
+      fResult.strVal.assign(std::string(tmp));
       break;
     }
 
@@ -519,7 +529,7 @@ inline const std::string& TreeNode::getStrVal(const long timeZone)
       if ((fabs(fResult.floatVal) > (1.0 / IDB_pow[4])) && (fabs(fResult.floatVal) < (float)IDB_pow[6]))
       {
         snprintf(tmp, 312, "%f", fResult.floatVal);
-        fResult.strVal = removeTrailing0(tmp, 312);
+        fResult.strVal.assign(removeTrailing0(tmp, 312));
       }
       else
       {
@@ -530,14 +540,15 @@ inline const std::string& TreeNode::getStrVal(const long timeZone)
         if (std::isnan(exponent) || std::isnan(base))
         {
           snprintf(tmp, 312, "%f", fResult.floatVal);
-          fResult.strVal = removeTrailing0(tmp, 312);
+          fResult.strVal.assign(removeTrailing0(tmp, 312));
         }
         else
         {
           snprintf(tmp, 312, "%.5f", base);
-          fResult.strVal = removeTrailing0(tmp, 312);
+          std::string tmpCat(removeTrailing0(tmp, 312));
           snprintf(tmp, 312, "e%02d", exponent);
-          fResult.strVal += tmp;
+          tmpCat += tmp;
+          fResult.strVal.assign(tmpCat);
         }
 
         //				snprintf(tmp, 312, "%e.5", fResult.floatVal);
@@ -553,7 +564,7 @@ inline const std::string& TreeNode::getStrVal(const long timeZone)
       if ((fabs(fResult.doubleVal) > (1.0 / IDB_pow[13])) && (fabs(fResult.doubleVal) < (float)IDB_pow[15]))
       {
         snprintf(tmp, 312, "%f", fResult.doubleVal);
-        fResult.strVal = removeTrailing0(tmp, 312);
+        fResult.strVal.assign(removeTrailing0(tmp, 312));
       }
       else
       {
@@ -564,14 +575,15 @@ inline const std::string& TreeNode::getStrVal(const long timeZone)
         if (std::isnan(exponent) || std::isnan(base))
         {
           snprintf(tmp, 312, "%f", fResult.doubleVal);
-          fResult.strVal = removeTrailing0(tmp, 312);
+          fResult.strVal.assign(removeTrailing0(tmp, 312));
         }
         else
         {
           snprintf(tmp, 312, "%.9f", base);
-          fResult.strVal = removeTrailing0(tmp, 312);
+          std::string tmpCat(removeTrailing0(tmp, 312));
           snprintf(tmp, 312, "e%02d", exponent);
-          fResult.strVal += tmp;
+          tmpCat += tmp;
+          fResult.strVal.assign(tmpCat);
         }
 
         //				snprintf(tmp, 312, "%e", fResult.doubleVal);
@@ -587,7 +599,7 @@ inline const std::string& TreeNode::getStrVal(const long timeZone)
           (fabsl(fResult.longDoubleVal) < (float)IDB_pow[15]))
       {
         snprintf(tmp, 312, "%Lf", fResult.longDoubleVal);
-        fResult.strVal = removeTrailing0(tmp, 312);
+        fResult.strVal.assign(removeTrailing0(tmp, 312));
       }
       else
       {
@@ -598,14 +610,15 @@ inline const std::string& TreeNode::getStrVal(const long timeZone)
         if (std::isnan(exponent) || std::isnan(base))
         {
           snprintf(tmp, 312, "%Lf", fResult.longDoubleVal);
-          fResult.strVal = removeTrailing0(tmp, 312);
+          fResult.strVal.assign(removeTrailing0(tmp, 312));
         }
         else
         {
           snprintf(tmp, 312, "%.14Lf", base);
-          fResult.strVal = removeTrailing0(tmp, 312);
+          std::string tmpCat = removeTrailing0(tmp, 312);
           snprintf(tmp, 312, "e%02d", exponent);
-          fResult.strVal += tmp;
+          tmpCat += tmp;
+          fResult.strVal.assign(tmpCat);
         }
 
         //				snprintf(tmp, 312, "%e", fResult.doubleVal);
@@ -619,38 +632,42 @@ inline const std::string& TreeNode::getStrVal(const long timeZone)
     case CalpontSystemCatalog::UDECIMAL:
     {
       if (fResultType.colWidth == datatypes::MAXDECIMALWIDTH)
+      {
         // Explicit path for TSInt128 decimals with low precision
-        fResult.strVal = fResult.decimalVal.toString(true);
+        fResult.strVal = fResult.decimalVal.toNullString(true);
+      }
       else
-        fResult.strVal = fResult.decimalVal.toString();
+      {
+        fResult.strVal = fResult.decimalVal.toNullString(false);
+      }
       break;
     }
 
     case CalpontSystemCatalog::DATE:
     {
       dataconvert::DataConvert::dateToString(fResult.intVal, tmp, 255);
-      fResult.strVal = std::string(tmp);
+      fResult.strVal.assign(std::string(tmp));
       break;
     }
 
     case CalpontSystemCatalog::DATETIME:
     {
       dataconvert::DataConvert::datetimeToString(fResult.intVal, tmp, 255, fResultType.precision);
-      fResult.strVal = std::string(tmp);
+      fResult.strVal.assign(std::string(tmp));
       break;
     }
 
     case CalpontSystemCatalog::TIMESTAMP:
     {
       dataconvert::DataConvert::timestampToString(fResult.intVal, tmp, 255, timeZone, fResultType.precision);
-      fResult.strVal = std::string(tmp);
+      fResult.strVal.assign(std::string(tmp));
       break;
     }
 
     case CalpontSystemCatalog::TIME:
     {
       dataconvert::DataConvert::timeToString(fResult.intVal, tmp, 255, fResultType.precision);
-      fResult.strVal = std::string(tmp);
+      fResult.strVal.assign(std::string(tmp));
       break;
     }
 
@@ -671,7 +688,7 @@ inline int64_t TreeNode::getIntVal()
         return fResult.intVal;
       }
       datatypes::DataCondition cnverr;
-      literal::Converter<literal::SignedInteger> cnv(fResult.strVal, cnverr);
+      literal::Converter<literal::SignedInteger> cnv(fResult.strVal.safeString(""), cnverr);
       return cnv.toSInt<int64_t>(cnverr);
     }
     case CalpontSystemCatalog::VARCHAR:
@@ -683,7 +700,7 @@ inline int64_t TreeNode::getIntVal()
         return fResult.intVal;
 
       datatypes::DataCondition cnverr;
-      literal::Converter<literal::SignedInteger> cnv(fResult.strVal, cnverr);
+      literal::Converter<literal::SignedInteger> cnv(fResult.strVal.safeString(""), cnverr);
       return cnv.toSInt<int64_t>(cnverr);
     }
 
@@ -731,10 +748,10 @@ inline uint64_t TreeNode::getUintVal()
     case CalpontSystemCatalog::TEXT:
     {
       datatypes::DataCondition cnverr;
-      literal::Converter<literal::UnsignedInteger> cnv(fResult.strVal, cnverr);
+      literal::Converter<literal::UnsignedInteger> cnv(fResult.strVal.safeString(""), cnverr);
       if (datatypes::DataCondition::Code(cnverr) != 0)
       {
-        cerr << "error in unsigned int conversion from '" << fResult.strVal << "'";
+        cerr << "error in unsigned int conversion from '" << fResult.strVal.safeString() << "'";
       }
       return cnv.toXIntPositive<uint64_t>(cnverr);
     }
@@ -779,13 +796,15 @@ inline float TreeNode::getFloatVal()
       if (fResultType.colWidth <= 8)
         return atof((char*)(&fResult.origIntVal));
 
-      return atof(fResult.strVal.c_str());
+      idbassert(fResult.strVal.str());
+      return atof(fResult.strVal.str());
 
     case CalpontSystemCatalog::VARCHAR:
       if (fResultType.colWidth <= 7)
         return atof((char*)(&fResult.origIntVal));
 
-      return atof(fResult.strVal.c_str());
+      idbassert(fResult.strVal.str());
+      return atof(fResult.strVal.str());
 
     // FIXME: ???
     case CalpontSystemCatalog::VARBINARY:
@@ -794,7 +813,8 @@ inline float TreeNode::getFloatVal()
       if (fResultType.colWidth <= 7)
         return atof((char*)(&fResult.origIntVal));
 
-      return atof(fResult.strVal.c_str());
+      idbassert(fResult.strVal.str());
+      return atof(fResult.strVal.str());
 
     case CalpontSystemCatalog::BIGINT:
     case CalpontSystemCatalog::TINYINT:
@@ -847,13 +867,15 @@ inline double TreeNode::getDoubleVal()
       if (fResultType.colWidth <= 8)
         return strtod((char*)(&fResult.origIntVal), NULL);
 
-      return strtod(fResult.strVal.c_str(), NULL);
+      idbassert(fResult.strVal.str());
+      return strtod(fResult.strVal.str(), NULL);
 
     case CalpontSystemCatalog::VARCHAR:
       if (fResultType.colWidth <= 7)
         return strtod((char*)(&fResult.origIntVal), NULL);
 
-      return strtod(fResult.strVal.c_str(), NULL);
+      idbassert(fResult.strVal.str());
+      return strtod(fResult.strVal.str(), NULL);
 
     // FIXME: ???
     case CalpontSystemCatalog::VARBINARY:
@@ -862,7 +884,8 @@ inline double TreeNode::getDoubleVal()
       if (fResultType.colWidth <= 7)
         return strtod((char*)(&fResult.origIntVal), NULL);
 
-      return strtod(fResult.strVal.c_str(), NULL);
+      //idbassert(fResult.strVal.str());
+      return strtod(fResult.strVal.safeString("").c_str(), NULL);
 
     case CalpontSystemCatalog::BIGINT:
     case CalpontSystemCatalog::TINYINT:
@@ -915,13 +938,15 @@ inline long double TreeNode::getLongDoubleVal()
       if (fResultType.colWidth <= 8)
         return strtold((char*)(&fResult.origIntVal), NULL);
 
-      return strtold(fResult.strVal.c_str(), NULL);
+      idbassert(fResult.strVal.str());
+      return strtold(fResult.strVal.str(), NULL);
 
     case CalpontSystemCatalog::VARCHAR:
       if (fResultType.colWidth <= 7)
         return strtold((char*)(&fResult.origIntVal), NULL);
 
-      return strtold(fResult.strVal.c_str(), NULL);
+      idbassert(fResult.strVal.str());
+      return strtold(fResult.strVal.str(), NULL);
 
     // FIXME: ???
     case CalpontSystemCatalog::VARBINARY:
@@ -930,7 +955,8 @@ inline long double TreeNode::getLongDoubleVal()
       if (fResultType.colWidth <= 7)
         return strtold((char*)(&fResult.origIntVal), NULL);
 
-      return strtold(fResult.strVal.c_str(), NULL);
+      idbassert(fResult.strVal.str());
+      return strtold(fResult.strVal.str(), NULL);
 
     case CalpontSystemCatalog::BIGINT:
     case CalpontSystemCatalog::TINYINT:
