@@ -19,8 +19,7 @@
 
 /** @file */
 
-#ifndef JOBLIST_ERROR_INFO_H_
-#define JOBLIST_ERROR_INFO_H_
+#pragma once
 
 #include <string>
 #include <boost/shared_ptr.hpp>
@@ -53,4 +52,3 @@ typedef boost::shared_ptr<ErrorInfo> SErrorInfo;
 
 }  // namespace joblist
 
-#endif  // JOBLIST_ERROR_INFO_H_
\ No newline at end of file
