@@ -15,27 +15,22 @@
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301, USA. */
 
-
-
-#ifndef WRITETASK_H_
-#define WRITETASK_H_
+#pragma once
 
 #include "PosixTask.h"
 
 namespace storagemanager
 {
-
 class WriteTask : public PosixTask
 {
-    public:
-        WriteTask(int sock, uint length);
-        virtual ~WriteTask();
-        
-        bool run();
-    
-    private:
-        WriteTask();
+ public:
+  WriteTask(int sock, uint length);
+  virtual ~WriteTask();
+
+  bool run();
+
+ private:
+  WriteTask();
 };
 
-}
-#endif
+}  // namespace storagemanager
