@@ -15,11 +15,7 @@ inline bool isNumeric(int type, const char* attr)
   {
     return true;
   }
-#if _MSC_VER
-  if (_strnicmp("NULL", attr, 4) == 0))
-#else
   if (strncasecmp("NULL", attr, 4) == 0)
-#endif
     {
       return true;
     }
@@ -81,9 +77,6 @@ extern "C"
     int64_t cnt;
   };
 
-#ifdef _MSC_VER
-  __declspec(dllexport)
-#endif
       my_bool regr_avgx_init(UDF_INIT* initid, UDF_ARGS* args, char* message)
   {
     struct regr_avgx_data* data;
@@ -114,17 +107,11 @@ extern "C"
     return 0;
   }
 
-#ifdef _MSC_VER
-  __declspec(dllexport)
-#endif
       void regr_avgx_deinit(UDF_INIT* initid)
   {
     free(initid->ptr);
   }
 
-#ifdef _MSC_VER
-  __declspec(dllexport)
-#endif
       void regr_avgx_clear(UDF_INIT* initid, char* is_null __attribute__((unused)),
                            char* message __attribute__((unused)))
   {
@@ -133,9 +120,6 @@ extern "C"
     data->cnt = 0;
   }
 
-#ifdef _MSC_VER
-  __declspec(dllexport)
-#endif
       void regr_avgx_add(UDF_INIT* initid, UDF_ARGS* args, char* is_null,
                          char* message __attribute__((unused)))
   {
@@ -150,9 +134,6 @@ extern "C"
     data->sumx += xval;
   }
 
-#ifdef _MSC_VER
-  __declspec(dllexport)
-#endif
       double regr_avgx(UDF_INIT* initid, UDF_ARGS* args __attribute__((unused)), char* is_null,
                        char* error __attribute__((unused)))
   {
@@ -181,9 +162,6 @@ extern "C"
     int64_t cnt;
   };
 
-#ifdef _MSC_VER
-  __declspec(dllexport)
-#endif
       my_bool regr_avgy_init(UDF_INIT* initid, UDF_ARGS* args, char* message)
   {
     struct regr_avgy_data* data;
@@ -215,17 +193,11 @@ extern "C"
     return 0;
   }
 
-#ifdef _MSC_VER
-  __declspec(dllexport)
-#endif
       void regr_avgy_deinit(UDF_INIT* initid)
   {
     free(initid->ptr);
   }
 
-#ifdef _MSC_VER
-  __declspec(dllexport)
-#endif
       void regr_avgy_clear(UDF_INIT* initid, char* is_null __attribute__((unused)),
                            char* message __attribute__((unused)))
   {
@@ -234,9 +206,6 @@ extern "C"
     data->cnt = 0;
   }
 
-#ifdef _MSC_VER
-  __declspec(dllexport)
-#endif
       void regr_avgy_add(UDF_INIT* initid, UDF_ARGS* args, char* is_null,
                          char* message __attribute__((unused)))
   {
@@ -251,9 +220,6 @@ extern "C"
     data->sumy += yval;
   }
 
-#ifdef _MSC_VER
-  __declspec(dllexport)
-#endif
       double regr_avgy(UDF_INIT* initid, UDF_ARGS* args __attribute__((unused)), char* is_null,
                        char* error __attribute__((unused)))
   {
@@ -280,9 +246,6 @@ extern "C"
     int64_t cnt;
   };
 
-#ifdef _MSC_VER
-  __declspec(dllexport)
-#endif
       my_bool regr_count_init(UDF_INIT* initid, UDF_ARGS* args, char* message)
   {
     struct regr_count_data* data;
@@ -303,17 +266,11 @@ extern "C"
     return 0;
   }
 
-#ifdef _MSC_VER
-  __declspec(dllexport)
-#endif
       void regr_count_deinit(UDF_INIT* initid)
   {
     free(initid->ptr);
   }
 
-#ifdef _MSC_VER
-  __declspec(dllexport)
-#endif
       void regr_count_clear(UDF_INIT* initid, char* is_null __attribute__((unused)),
                             char* message __attribute__((unused)))
   {
@@ -321,9 +278,6 @@ extern "C"
     data->cnt = 0;
   }
 
-#ifdef _MSC_VER
-  __declspec(dllexport)
-#endif
       void regr_count_add(UDF_INIT* initid, UDF_ARGS* args, char* is_null,
                           char* message __attribute__((unused)))
   {
@@ -336,9 +290,6 @@ extern "C"
     ++data->cnt;
   }
 
-#ifdef _MSC_VER
-  __declspec(dllexport)
-#endif
       long long regr_count(UDF_INIT* initid, UDF_ARGS* args __attribute__((unused)), char* is_null,
                            char* error __attribute__((unused)))
   {
@@ -360,9 +311,6 @@ extern "C"
     long double sumxy;  // sum of (x*y)
   };
 
-#ifdef _MSC_VER
-  __declspec(dllexport)
-#endif
       my_bool regr_slope_init(UDF_INIT* initid, UDF_ARGS* args, char* message)
   {
     struct regr_slope_data* data;
@@ -395,17 +343,11 @@ extern "C"
     return 0;
   }
 
-#ifdef _MSC_VER
-  __declspec(dllexport)
-#endif
       void regr_slope_deinit(UDF_INIT* initid)
   {
     free(initid->ptr);
   }
 
-#ifdef _MSC_VER
-  __declspec(dllexport)
-#endif
       void regr_slope_clear(UDF_INIT* initid, char* is_null __attribute__((unused)),
                             char* message __attribute__((unused)))
   {
@@ -417,9 +359,6 @@ extern "C"
     data->sumxy = 0.0;
   }
 
-#ifdef _MSC_VER
-  __declspec(dllexport)
-#endif
       void regr_slope_add(UDF_INIT* initid, UDF_ARGS* args, char* is_null,
                           char* message __attribute__((unused)))
   {
@@ -438,9 +377,6 @@ extern "C"
     ++data->cnt;
   }
 
-#ifdef _MSC_VER
-  __declspec(dllexport)
-#endif
       double regr_slope(UDF_INIT* initid, UDF_ARGS* args __attribute__((unused)), char* is_null,
                         char* error __attribute__((unused)))
   {
@@ -479,9 +415,6 @@ extern "C"
     long double sumxy;  // sum of (x*y)
   };
 
-#ifdef _MSC_VER
-  __declspec(dllexport)
-#endif
       my_bool regr_intercept_init(UDF_INIT* initid, UDF_ARGS* args, char* message)
   {
     struct regr_intercept_data* data;
@@ -513,17 +446,11 @@ extern "C"
     return 0;
   }
 
-#ifdef _MSC_VER
-  __declspec(dllexport)
-#endif
       void regr_intercept_deinit(UDF_INIT* initid)
   {
     free(initid->ptr);
   }
 
-#ifdef _MSC_VER
-  __declspec(dllexport)
-#endif
       void regr_intercept_clear(UDF_INIT* initid, char* is_null __attribute__((unused)),
                                 char* message __attribute__((unused)))
   {
@@ -535,9 +462,6 @@ extern "C"
     data->sumxy = 0.0;
   }
 
-#ifdef _MSC_VER
-  __declspec(dllexport)
-#endif
       void regr_intercept_add(UDF_INIT* initid, UDF_ARGS* args, char* is_null,
                               char* message __attribute__((unused)))
   {
@@ -556,9 +480,6 @@ extern "C"
     ++data->cnt;
   }
 
-#ifdef _MSC_VER
-  __declspec(dllexport)
-#endif
       double regr_intercept(UDF_INIT* initid, UDF_ARGS* args __attribute__((unused)), char* is_null,
                             char* error __attribute__((unused)))
   {
@@ -598,9 +519,6 @@ extern "C"
     long double sumxy;  // sum of (x*y)
   };
 
-#ifdef _MSC_VER
-  __declspec(dllexport)
-#endif
       my_bool regr_r2_init(UDF_INIT* initid, UDF_ARGS* args, char* message)
   {
     struct regr_r2_data* data;
@@ -634,17 +552,11 @@ extern "C"
     return 0;
   }
 
-#ifdef _MSC_VER
-  __declspec(dllexport)
-#endif
       void regr_r2_deinit(UDF_INIT* initid)
   {
     free(initid->ptr);
   }
 
-#ifdef _MSC_VER
-  __declspec(dllexport)
-#endif
       void regr_r2_clear(UDF_INIT* initid, char* is_null __attribute__((unused)),
                          char* message __attribute__((unused)))
   {
@@ -657,9 +569,6 @@ extern "C"
     data->sumxy = 0.0;
   }
 
-#ifdef _MSC_VER
-  __declspec(dllexport)
-#endif
       void regr_r2_add(UDF_INIT* initid, UDF_ARGS* args, char* is_null, char* message __attribute__((unused)))
   {
     // Test for NULL in x and y
@@ -678,9 +587,6 @@ extern "C"
     ++data->cnt;
   }
 
-#ifdef _MSC_VER
-  __declspec(dllexport)
-#endif
       double regr_r2(UDF_INIT* initid, UDF_ARGS* args __attribute__((unused)), char* is_null,
                      char* error __attribute__((unused)))
   {
@@ -735,9 +641,6 @@ extern "C"
     long double sumxy;  // sum of (x*y)
   };
 
-#ifdef _MSC_VER
-  __declspec(dllexport)
-#endif
       my_bool corr_init(UDF_INIT* initid, UDF_ARGS* args, char* message)
   {
     struct corr_data* data;
@@ -771,17 +674,11 @@ extern "C"
     return 0;
   }
 
-#ifdef _MSC_VER
-  __declspec(dllexport)
-#endif
       void corr_deinit(UDF_INIT* initid)
   {
     free(initid->ptr);
   }
 
-#ifdef _MSC_VER
-  __declspec(dllexport)
-#endif
       void corr_clear(UDF_INIT* initid, char* is_null __attribute__((unused)),
                       char* message __attribute__((unused)))
   {
@@ -794,9 +691,6 @@ extern "C"
     data->sumxy = 0.0;
   }
 
-#ifdef _MSC_VER
-  __declspec(dllexport)
-#endif
       void corr_add(UDF_INIT* initid, UDF_ARGS* args, char* is_null, char* message __attribute__((unused)))
   {
     // Test for NULL in x and y
@@ -815,9 +709,6 @@ extern "C"
     ++data->cnt;
   }
 
-#ifdef _MSC_VER
-  __declspec(dllexport)
-#endif
       double corr(UDF_INIT* initid, UDF_ARGS* args __attribute__((unused)), char* is_null,
                   char* error __attribute__((unused)))
   {
@@ -869,9 +760,6 @@ extern "C"
     long double sumx2;  // sum of (x squared)
   };
 
-#ifdef _MSC_VER
-  __declspec(dllexport)
-#endif
       my_bool regr_sxx_init(UDF_INIT* initid, UDF_ARGS* args, char* message)
   {
     struct regr_sxx_data* data;
@@ -901,17 +789,11 @@ extern "C"
     return 0;
   }
 
-#ifdef _MSC_VER
-  __declspec(dllexport)
-#endif
       void regr_sxx_deinit(UDF_INIT* initid)
   {
     free(initid->ptr);
   }
 
-#ifdef _MSC_VER
-  __declspec(dllexport)
-#endif
       void regr_sxx_clear(UDF_INIT* initid, char* is_null __attribute__((unused)),
                           char* message __attribute__((unused)))
   {
@@ -921,9 +803,6 @@ extern "C"
     data->sumx2 = 0.0;
   }
 
-#ifdef _MSC_VER
-  __declspec(dllexport)
-#endif
       void regr_sxx_add(UDF_INIT* initid, UDF_ARGS* args, char* is_null,
                         char* message __attribute__((unused)))
   {
@@ -939,9 +818,6 @@ extern "C"
     ++data->cnt;
   }
 
-#ifdef _MSC_VER
-  __declspec(dllexport)
-#endif
       double regr_sxx(UDF_INIT* initid, UDF_ARGS* args __attribute__((unused)), char* is_null,
                       char* error __attribute__((unused)))
   {
@@ -975,9 +851,6 @@ extern "C"
     long double sumy2;  // sum of (y squared)
   };
 
-#ifdef _MSC_VER
-  __declspec(dllexport)
-#endif
       my_bool regr_syy_init(UDF_INIT* initid, UDF_ARGS* args, char* message)
   {
     struct regr_syy_data* data;
@@ -1007,17 +880,11 @@ extern "C"
     return 0;
   }
 
-#ifdef _MSC_VER
-  __declspec(dllexport)
-#endif
       void regr_syy_deinit(UDF_INIT* initid)
   {
     free(initid->ptr);
   }
 
-#ifdef _MSC_VER
-  __declspec(dllexport)
-#endif
       void regr_syy_clear(UDF_INIT* initid, char* is_null __attribute__((unused)),
                           char* message __attribute__((unused)))
   {
@@ -1027,9 +894,6 @@ extern "C"
     data->sumy2 = 0.0;
   }
 
-#ifdef _MSC_VER
-  __declspec(dllexport)
-#endif
       void regr_syy_add(UDF_INIT* initid, UDF_ARGS* args, char* is_null,
                         char* message __attribute__((unused)))
   {
@@ -1045,9 +909,6 @@ extern "C"
     ++data->cnt;
   }
 
-#ifdef _MSC_VER
-  __declspec(dllexport)
-#endif
       double regr_syy(UDF_INIT* initid, UDF_ARGS* args __attribute__((unused)), char* is_null,
                       char* error __attribute__((unused)))
   {
@@ -1083,9 +944,6 @@ extern "C"
     long double sumxy;  // sum of (x*y)
   };
 
-#ifdef _MSC_VER
-  __declspec(dllexport)
-#endif
       my_bool regr_sxy_init(UDF_INIT* initid, UDF_ARGS* args, char* message)
   {
     struct regr_sxy_data* data;
@@ -1117,17 +975,11 @@ extern "C"
     return 0;
   }
 
-#ifdef _MSC_VER
-  __declspec(dllexport)
-#endif
       void regr_sxy_deinit(UDF_INIT* initid)
   {
     free(initid->ptr);
   }
 
-#ifdef _MSC_VER
-  __declspec(dllexport)
-#endif
       void regr_sxy_clear(UDF_INIT* initid, char* is_null __attribute__((unused)),
                           char* message __attribute__((unused)))
   {
@@ -1138,9 +990,6 @@ extern "C"
     data->sumxy = 0.0;
   }
 
-#ifdef _MSC_VER
-  __declspec(dllexport)
-#endif
       void regr_sxy_add(UDF_INIT* initid, UDF_ARGS* args, char* is_null,
                         char* message __attribute__((unused)))
   {
@@ -1158,9 +1007,6 @@ extern "C"
     ++data->cnt;
   }
 
-#ifdef _MSC_VER
-  __declspec(dllexport)
-#endif
       double regr_sxy(UDF_INIT* initid, UDF_ARGS* args __attribute__((unused)), char* is_null,
                       char* error __attribute__((unused)))
   {
@@ -1195,9 +1041,6 @@ extern "C"
     long double sumxy;  // sum of (x*y)
   };
 
-#ifdef _MSC_VER
-  __declspec(dllexport)
-#endif
       my_bool covar_pop_init(UDF_INIT* initid, UDF_ARGS* args, char* message)
   {
     struct covar_pop_data* data;
@@ -1229,17 +1072,11 @@ extern "C"
     return 0;
   }
 
-#ifdef _MSC_VER
-  __declspec(dllexport)
-#endif
       void covar_pop_deinit(UDF_INIT* initid)
   {
     free(initid->ptr);
   }
 
-#ifdef _MSC_VER
-  __declspec(dllexport)
-#endif
       void covar_pop_clear(UDF_INIT* initid, char* is_null __attribute__((unused)),
                            char* message __attribute__((unused)))
   {
@@ -1250,9 +1087,6 @@ extern "C"
     data->sumxy = 0.0;
   }
 
-#ifdef _MSC_VER
-  __declspec(dllexport)
-#endif
       void covar_pop_add(UDF_INIT* initid, UDF_ARGS* args, char* is_null,
                          char* message __attribute__((unused)))
   {
@@ -1270,9 +1104,6 @@ extern "C"
     ++data->cnt;
   }
 
-#ifdef _MSC_VER
-  __declspec(dllexport)
-#endif
       double covar_pop(UDF_INIT* initid, UDF_ARGS* args __attribute__((unused)), char* is_null,
                        char* error __attribute__((unused)))
   {
@@ -1306,9 +1137,6 @@ extern "C"
     long double sumxy;  // sum of (x*y)
   };
 
-#ifdef _MSC_VER
-  __declspec(dllexport)
-#endif
       my_bool covar_samp_init(UDF_INIT* initid, UDF_ARGS* args, char* message)
   {
     struct covar_samp_data* data;
@@ -1340,17 +1168,11 @@ extern "C"
     return 0;
   }
 
-#ifdef _MSC_VER
-  __declspec(dllexport)
-#endif
       void covar_samp_deinit(UDF_INIT* initid)
   {
     free(initid->ptr);
   }
 
-#ifdef _MSC_VER
-  __declspec(dllexport)
-#endif
       void covar_samp_clear(UDF_INIT* initid, char* is_null __attribute__((unused)),
                             char* message __attribute__((unused)))
   {
@@ -1361,9 +1183,6 @@ extern "C"
     data->sumxy = 0.0;
   }
 
-#ifdef _MSC_VER
-  __declspec(dllexport)
-#endif
       void covar_samp_add(UDF_INIT* initid, UDF_ARGS* args, char* is_null,
                           char* message __attribute__((unused)))
   {
@@ -1381,9 +1200,6 @@ extern "C"
     ++data->cnt;
   }
 
-#ifdef _MSC_VER
-  __declspec(dllexport)
-#endif
       double covar_samp(UDF_INIT* initid, UDF_ARGS* args __attribute__((unused)), char* is_null,
                         char* error __attribute__((unused)))
   {
