@@ -79,6 +79,12 @@ class IDBDataFile;
 
 namespace BRM
 {
+#define EM_MAGIC_V1 0x76f78b1c
+#define EM_MAGIC_V2 0x76f78b1d
+#define EM_MAGIC_V3 0x76f78b1e
+#define EM_MAGIC_V4 0x76f78b1f
+#define EM_MAGIC_V5 0x76f78b20
+
 using PartitionNumberT = uint32_t;
 using DBRootT = uint16_t;
 using SegmentT = uint16_t;
@@ -158,6 +164,8 @@ struct EMCasualPartition_struct
   EXPORT EMCasualPartition_struct();
   EXPORT EMCasualPartition_struct(const int64_t lo, const int64_t hi, const int32_t seqNum);
   EXPORT EMCasualPartition_struct(const int128_t bigLo, const int128_t bigHi, const int32_t seqNum);
+  EXPORT EMCasualPartition_struct(const int64_t lo, const int64_t hi, const int32_t seqNum,
+                                  const char status);
   EXPORT EMCasualPartition_struct(const EMCasualPartition_struct& em);
   EXPORT EMCasualPartition_struct& operator=(const EMCasualPartition_struct& em);
 };
@@ -182,6 +190,8 @@ struct EMEntry
   int16_t status;  // extent avail for query or not, or out of service
   EMPartition_t partition;
   EXPORT EMEntry();
+  EMEntry(const InlineLBIDRange& range, int fileID, uint32_t bOffset, HWM_t hwm, PartitionNumberT pNum,
+          uint16_t sNum, DBRootT dRoot, uint16_t cWid, int16_t s, EMPartition_t partition);
   EXPORT EMEntry(const EMEntry&);
   EXPORT EMEntry& operator=(const EMEntry&);
   EXPORT bool operator<(const EMEntry&) const;
@@ -253,8 +263,7 @@ class ExtentMapRBTreeImpl
  public:
   ~ExtentMapRBTreeImpl() = default;
 
-  static ExtentMapRBTreeImpl* makeExtentMapRBTreeImpl(unsigned key, off_t size, bool readOnly, bool& emLocked,
-                                                      const MasterSegmentTable* emSegTable = nullptr);
+  static ExtentMapRBTreeImpl* makeExtentMapRBTreeImpl(unsigned key, off_t size, bool readOnly = false);
 
   static void refreshShm()
   {
@@ -1036,9 +1045,9 @@ class ExtentMap : public Undoable
   EXPORT std::vector<InlineLBIDRange> getFreeListEntries();
 
   EXPORT void dumpTo(std::ostream& os);
-  EXPORT const bool* getEMLockStatus();
-  EXPORT const bool* getEMFLLockStatus();
-  EXPORT const bool* getEMIndexLockStatus();
+  EXPORT const std::atomic<bool>* getEMLockStatus();
+  EXPORT const std::atomic<bool>* getEMFLLockStatus();
+  EXPORT const std::atomic<bool>* getEMIndexLockStatus();
   size_t EMIndexShmemSize();
   size_t EMIndexShmemFree();
 
@@ -1062,6 +1071,7 @@ class ExtentMap : public Undoable
   static const constexpr size_t EM_INCREMENT = EM_INCREMENT_ROWS * sizeof(EMEntry);
   static const constexpr size_t EM_FREELIST_INITIAL_SIZE = 50 * sizeof(InlineLBIDRange);
   static const constexpr size_t EM_FREELIST_INCREMENT = 50 * sizeof(InlineLBIDRange);
+  static const constexpr size_t EM_SAVE_NUM_PER_BATCH = 1000000;
   // RBTree constants.
   static const size_t EM_RB_TREE_NODE_SIZE = sizeof(EMEntry) + 8 * sizeof(uint64_t);
   static const size_t EM_RB_TREE_EMPTY_SIZE = 1024;
@@ -1088,7 +1098,9 @@ class ExtentMap : public Undoable
   time_t fCacheTime;  // timestamp associated with config cache
 
   int numUndoRecords;
-  bool flLocked, emLocked, emIndexLocked;
+  std::atomic<bool> flLocked{false};
+  std::atomic<bool> emLocked{false};
+  std::atomic<bool> emIndexLocked{false};
 
   static boost::mutex mutex;  // @bug5355 - made mutex static
   static boost::mutex emIndexMutex;
@@ -1174,6 +1186,11 @@ class ExtentMap : public Undoable
 
   int _markInvalid(const LBID_t lbid, const execplan::CalpontSystemCatalog::ColDataType colDataType);
 
+  MSTEntry* _getTableLock(const OPS op, std::atomic<bool>& lockedState, const int table);
+  void _getTableLockUpgradeIfNeeded(const OPS op, std::atomic<bool>& lockedState, const int table);
+  void _getTableLockDowngradeIfNeeded(const OPS op, std::atomic<bool>& lockedState, const int table);
+  void _releaseTable(const OPS op, std::atomic<bool>& lockedState, const int table);
+
   template <typename T>
   void load(T* in);
 
