@@ -21,6 +21,7 @@
  *******************************************************************************/
 /** @file */
 
+#include "mcs_basic_types.h"
 #define WRITEENGINEXMLJOB_DLLEXPORT
 #include "we_xmljob.h"
 #undef WRITEENGINEXMLJOB_DLLEXPORT
@@ -36,7 +37,8 @@
 #include "we_convertor.h"
 #include "dataconvert.h"
 #include <boost/date_time/posix_time/posix_time.hpp>
-#include <boost/filesystem.hpp>
+#include <boost/filesystem/path.hpp>
+#include <boost/filesystem/convenience.hpp>
 
 #include <sys/time.h>
 
@@ -45,28 +47,6 @@ using namespace execplan;
 
 namespace WriteEngine
 {
-// Maximum saturation value for DECIMAL types based on precision
-// TODO MCOL-641 add support here. see dataconvert.cpp
-const long long columnstore_precision[19] = {0,
-                                             9,
-                                             99,
-                                             999,
-                                             9999,
-                                             99999,
-                                             999999,
-                                             9999999,
-                                             99999999,
-                                             999999999,
-                                             9999999999LL,
-                                             99999999999LL,
-                                             999999999999LL,
-                                             9999999999999LL,
-                                             99999999999999LL,
-                                             999999999999999LL,
-                                             9999999999999999LL,
-                                             99999999999999999LL,
-                                             999999999999999999LL};
-
 //------------------------------------------------------------------------------
 // Constructor
 //------------------------------------------------------------------------------
@@ -694,13 +674,14 @@ void XMLJob::initSatLimits(JobColumn& curColumn) const
   }
   else if (curColumn.typeName == ColDataTypeStr[CalpontSystemCatalog::DECIMAL])
   {
-    curColumn.fMinIntSat = -columnstore_precision[curColumn.precision];
-    curColumn.fMaxIntSat = columnstore_precision[curColumn.precision];
+    curColumn.fMaxIntSat = dataconvert::decimalRangeUp<int128_t>(curColumn.precision);
+    curColumn.fMinIntSat = -curColumn.fMaxIntSat;
+
   }
   else if (curColumn.typeName == ColDataTypeStr[CalpontSystemCatalog::UDECIMAL])
   {
     curColumn.fMinIntSat = 0;
-    curColumn.fMaxIntSat = columnstore_precision[curColumn.precision];
+    curColumn.fMaxIntSat = dataconvert::decimalRangeUp<int128_t>(curColumn.precision);
   }
   else if (curColumn.typeName == ColDataTypeStr[CalpontSystemCatalog::FLOAT])
   {
@@ -890,6 +871,15 @@ void XMLJob::fillInXMLDataAsLoaded(execplan::CalpontSystemCatalog::RIDList& colR
       col.compressionType = colType.compressionType;
       col.dctnry.fCompressionType = colType.compressionType;
 
+      if (colType.charsetNumber != 0)
+      {
+        col.cs = &datatypes::Charset(colType.charsetNumber).getCharset();
+      }
+      else
+      {
+        col.cs = &my_charset_latin1;
+      }
+
       if (colType.autoincrement)
         col.autoIncFlag = true;
       else
@@ -929,13 +919,13 @@ void XMLJob::fillInXMLDataAsLoaded(execplan::CalpontSystemCatalog::RIDList& colR
 void XMLJob::fillInXMLDataNotNullDefault(const std::string& fullTblName,
                                          execplan::CalpontSystemCatalog::ColType& colType, JobColumn& col)
 {
-  const std::string col_defaultValue(colType.defaultValue);
+  const NullString col_defaultValue(colType.defaultValue);
 
   if (colType.constraintType == execplan::CalpontSystemCatalog::NOTNULL_CONSTRAINT)
   {
     col.fNotNull = true;
 
-    if (!col_defaultValue.empty())
+    if (!col_defaultValue.isNull())
       col.fWithDefault = true;
   }
   else if (colType.constraintType == execplan::CalpontSystemCatalog::DEFAULT_CONSTRAINT)
@@ -962,7 +952,7 @@ void XMLJob::fillInXMLDataNotNullDefault(const std::string& fullTblName,
       case execplan::CalpontSystemCatalog::BIGINT:
       {
         errno = 0;
-        col.fDefaultInt = strtoll(col_defaultValue.c_str(), 0, 10);
+        col.fDefaultInt = strtoll(col_defaultValue.str(), 0, 10);
 
         if (errno == ERANGE)
           bDefaultConvertError = true;
@@ -977,7 +967,7 @@ void XMLJob::fillInXMLDataNotNullDefault(const std::string& fullTblName,
       case execplan::CalpontSystemCatalog::UBIGINT:
       {
         errno = 0;
-        col.fDefaultUInt = strtoull(col_defaultValue.c_str(), 0, 10);
+        col.fDefaultUInt = strtoull(col_defaultValue.str(), 0, 10);
 
         if (errno == ERANGE)
           bDefaultConvertError = true;
@@ -990,11 +980,11 @@ void XMLJob::fillInXMLDataNotNullDefault(const std::string& fullTblName,
       {
         if (LIKELY(colType.colWidth == datatypes::MAXDECIMALWIDTH))
         {
-          col.fDefaultWideDecimal = colType.decimal128FromString(col_defaultValue, &bDefaultConvertError);
+          col.fDefaultWideDecimal = colType.decimal128FromString(col_defaultValue.safeString(), &bDefaultConvertError);
         }
         else
         {
-          col.fDefaultInt = Convertor::convertDecimalString(col_defaultValue.c_str(),
+          col.fDefaultInt = Convertor::convertDecimalString(col_defaultValue.str(),
                                                             col_defaultValue.length(), colType.scale);
 
           if (errno == ERANGE)
@@ -1007,7 +997,7 @@ void XMLJob::fillInXMLDataNotNullDefault(const std::string& fullTblName,
       case execplan::CalpontSystemCatalog::DATE:
       {
         int convertStatus;
-        int32_t dt = dataconvert::DataConvert::convertColumnDate(col_defaultValue.c_str(),
+        int32_t dt = dataconvert::DataConvert::convertColumnDate(col_defaultValue.str(),
                                                                  dataconvert::CALPONTDATE_ENUM, convertStatus,
                                                                  col_defaultValue.length());
 
@@ -1022,7 +1012,7 @@ void XMLJob::fillInXMLDataNotNullDefault(const std::string& fullTblName,
       {
         int convertStatus;
         int64_t dt = dataconvert::DataConvert::convertColumnDatetime(
-            col_defaultValue.c_str(), dataconvert::CALPONTDATETIME_ENUM, convertStatus,
+            col_defaultValue.str(), dataconvert::CALPONTDATETIME_ENUM, convertStatus,
             col_defaultValue.length());
 
         if (convertStatus != 0)
@@ -1036,7 +1026,7 @@ void XMLJob::fillInXMLDataNotNullDefault(const std::string& fullTblName,
       {
         int convertStatus;
         int64_t dt = dataconvert::DataConvert::convertColumnTimestamp(
-            col_defaultValue.c_str(), dataconvert::CALPONTDATETIME_ENUM, convertStatus,
+            col_defaultValue.str(), dataconvert::CALPONTDATETIME_ENUM, convertStatus,
             col_defaultValue.length(), fTimeZone);
 
         if (convertStatus != 0)
@@ -1049,7 +1039,7 @@ void XMLJob::fillInXMLDataNotNullDefault(const std::string& fullTblName,
       case execplan::CalpontSystemCatalog::TIME:
       {
         int convertStatus;
-        int64_t dt = dataconvert::DataConvert::convertColumnTime(col_defaultValue.c_str(),
+        int64_t dt = dataconvert::DataConvert::convertColumnTime(col_defaultValue.str(),
                                                                  dataconvert::CALPONTTIME_ENUM, convertStatus,
                                                                  col_defaultValue.length());
 
@@ -1066,7 +1056,7 @@ void XMLJob::fillInXMLDataNotNullDefault(const std::string& fullTblName,
       case execplan::CalpontSystemCatalog::UDOUBLE:
       {
         errno = 0;
-        col.fDefaultDbl = strtod(col_defaultValue.c_str(), 0);
+        col.fDefaultDbl = strtod(col_defaultValue.str(), 0);
 
         if (errno == ERANGE)
           bDefaultConvertError = true;
@@ -1210,9 +1200,6 @@ int XMLJob::genJobXMLFileName(const string& sXMLJobDir, const string& jobDir, co
     // attempt to make it absolute so that we can log the full pathname.
     if (!xmlFilePath.has_root_path())
     {
-#ifdef _MSC_VER
-      // nothing else to do
-#else
       char cwdPath[4096];
       char* err;
       err = getcwd(cwdPath, sizeof(cwdPath));
@@ -1224,7 +1211,6 @@ int XMLJob::genJobXMLFileName(const string& sXMLJobDir, const string& jobDir, co
       string trailingPath(xmlFilePath.string());
       xmlFilePath = cwdPath;
       xmlFilePath /= trailingPath;
-#endif
     }
   }
 
