@@ -20,13 +20,13 @@
 #include "sql_class.h"
 //#include "sql_time.h"
 
-#if defined(__WIN__)
+#if defined(_WIN32)
 //#include <windows.h>
-#else   // !__WIN__
+#else   // !_WIN32
 #include <string.h>
 #include <sys/types.h>
 #include <sys/stat.h>
-#endif  // !__WIN__
+#endif  // !_WIN32
 
 /***********************************************************************/
 /*  Include application header files:                                  */
@@ -65,7 +65,7 @@ void BLOCKFILTER::Printf(PGLOBAL, FILE *f, uint n)
 
   fprintf(f, "%sBLOCKFILTER: at %p opc=%d opm=%d result=%d\n",
           m, this, Opc, Opm, Result);
-  } // end of Print
+  } // end of Printf
 
 /***********************************************************************/
 /*  Make string output of BLOCKFILTER contents.                        */
@@ -73,7 +73,7 @@ void BLOCKFILTER::Printf(PGLOBAL, FILE *f, uint n)
 void BLOCKFILTER::Prints(PGLOBAL, char *ps, uint z)
   {
   strncat(ps, "BlockFilter(s)", z);
-  } // end of Print
+  } // end of Prints
 
 
 /* ---------------------- Class BLKFILLOG ---------------------------- */
@@ -241,7 +241,7 @@ int BLKFILARI::BlockEval(PGLOBAL)
       break;
     } // endswitch Opc
 
-  if (trace)
+  if (trace(1))
     htrc("BlockEval: op=%d n=%d rc=%d\n", Opc, n, Result);
 
   return Result;
@@ -338,7 +338,7 @@ int BLKFILAR2::BlockEval(PGLOBAL)
       break;
     } // endswitch Opc
 
-  if (trace)
+  if (trace(1))
     htrc("BlockEval2: op=%d n=%d rc=%d\n", Opc, n, Result);
 
   return Result;
@@ -412,7 +412,7 @@ void BLKFILMR2::MakeValueBitmap(void)
   Void = !Bmp[N];          // There are no good values in the file
 
   for (i = 0; i < N; i++) {
-    Bxp[i] = ~0U;
+    Bxp[i] = ~0;
 
     if (noteq)
       Bmp[i] = Bxp[i];
@@ -474,7 +474,7 @@ int BLKFILMR2::BlockEval(PGLOBAL)
       break;
     } // endswitch Opc
 
-  if (trace)
+  if (trace(1))
     htrc("BlockEval2: op=%d n=%d rc=%d\n", Opc, n, Result);
 
   return Result;
@@ -567,7 +567,7 @@ int BLKSPCARI::BlockEval(PGLOBAL)
       break;
     } // endswitch Opc
 
-  if (trace)
+  if (trace(1))
     htrc("BlockEval: op=%d n=%d rc=%d\n", Opc, n, Result);
 
   return Result;
@@ -594,7 +594,7 @@ BLKFILIN::BLKFILIN(PGLOBAL g, PTDBDOS tdbp, int op, int opm, PXOB *xp)
   Type = Arap->GetResultType();
 
   if (Colp->GetResultType() != Type) {
-    sprintf(g->Message, "BLKFILIN: %s", MSG(VALTYPE_NOMATCH));
+    snprintf(g->Message, sizeof(g->Message), "BLKFILIN: %s", MSG(VALTYPE_NOMATCH));
 		throw g->Message;
 	} else if (Colp->GetValue()->IsCi())
     Arap->SetPrecision(g, 1);        // Case insensitive
@@ -708,7 +708,7 @@ void BLKFILIN2::MakeValueBitmap(void)
   Void = !Bmp[N];          // There are no good values in the file
 
   for (i = 0; i < N; i++) {
-    Bxp[i] = ~0U;
+    Bxp[i] = ~0;
 
     if (noteq) {
       Bmp[i] = Bxp[i];
@@ -828,7 +828,7 @@ BLKFILIN2::BLKFILIN2(PGLOBAL g, PTDBDOS tdbp, int op, int opm, PXOB *xp)
         Bxp[i] |= btp;
 
       for (N = i--; i >= 0; i--)
-        Bxp[i] = ~0U;
+        Bxp[i] = ~0;
 
       break;
       } // endif Bmp
@@ -1006,9 +1006,9 @@ void BLOCKINDEX::Printf(PGLOBAL g, FILE *f, UINT n)
     m, this, Next, (Colp) ? Colp->GetName() : "Rowid", Kxp, Result);
 
   if (Next)
-    Next->Print(g, f, n);
+    Next->Printf(g, f, n);
 
-  } // end of Print
+  } // end of Printf
 
 /***********************************************************************/
 /*  Make string output of BLOCKINDEX contents.                         */
@@ -1016,7 +1016,7 @@ void BLOCKINDEX::Printf(PGLOBAL g, FILE *f, UINT n)
 void BLOCKINDEX::Prints(PGLOBAL g, char *ps, UINT z)
   {
   strncat(ps, "BlockIndex(es)", z);
-  } // end of Print
+  } // end of Prints
 
 /* ------------------------------------------------------------------- */
 
