@@ -63,23 +63,23 @@ class LIBXMLDOC : public XMLDOCUMENT {
   LIBXMLDOC(char *nsl, char *nsdf, char *enc, PFBLOCK fp);
 
   // Properties
-  virtual short  GetDocType(void) {return TYPE_FB_XML2;}
-  virtual void  *GetDocPtr(void) {return Docp;}
-  virtual void   SetNofree(bool b) {Nofreelist = b;}
+  short  GetDocType(void) override {return TYPE_FB_XML2;}
+  void  *GetDocPtr(void) override {return Docp;}
+  void   SetNofree(bool b) override {Nofreelist = b;}
 
   // Methods
-	virtual bool    Initialize(PGLOBAL g, PCSZ entry, bool zipped);
-  virtual bool    ParseFile(PGLOBAL g, char *fn);
-  virtual bool    NewDoc(PGLOBAL g, PCSZ ver);
-  virtual void    AddComment(PGLOBAL g, char *com);
-  virtual PXNODE  GetRoot(PGLOBAL g);
-  virtual PXNODE  NewRoot(PGLOBAL g, char *name);
-  virtual PXNODE  NewPnode(PGLOBAL g, char *name);
-  virtual PXATTR  NewPattr(PGLOBAL g);
-  virtual PXLIST  NewPlist(PGLOBAL g);
-  virtual int     DumpDoc(PGLOBAL g, char *ofn);
-  virtual void    CloseDoc(PGLOBAL g, PFBLOCK xp);
-  virtual PFBLOCK LinkXblock(PGLOBAL g, MODE m, int rc, char *fn);
+	bool    Initialize(PGLOBAL g, PCSZ entry, bool zipped) override;
+  bool    ParseFile(PGLOBAL g, char *fn) override;
+  bool    NewDoc(PGLOBAL g, PCSZ ver) override;
+  void    AddComment(PGLOBAL g, char *com) override;
+  PXNODE  GetRoot(PGLOBAL g) override;
+  PXNODE  NewRoot(PGLOBAL g, char *name) override;
+  PXNODE  NewPnode(PGLOBAL g, char *name) override;
+  PXATTR  NewPattr(PGLOBAL g) override;
+  PXLIST  NewPlist(PGLOBAL g) override;
+  int     DumpDoc(PGLOBAL g, char *ofn) override;
+  void    CloseDoc(PGLOBAL g, PFBLOCK xp) override;
+  PFBLOCK LinkXblock(PGLOBAL g, MODE m, int rc, char *fn) override;
 
  protected:
 //        bool     CheckDocument(FILE *of, xmlNodePtr np);
@@ -105,23 +105,23 @@ class XML2NODE : public XMLNODE {
   friend class XML2NODELIST;
  public:
   // Properties
-  virtual char  *GetName(PGLOBAL g) {return (char*)Nodep->name;}
-  virtual int    GetType(void);
-  virtual PXNODE GetNext(PGLOBAL g);
-  virtual PXNODE GetChild(PGLOBAL g);
+  char  *GetName(PGLOBAL g) override {return (char*)Nodep->name;}
+  int    GetType(void) override;
+  PXNODE GetNext(PGLOBAL g) override;
+  PXNODE GetChild(PGLOBAL g) override;
 
   // Methods
-  virtual RCODE  GetContent(PGLOBAL g, char *buf, int len);
-  virtual bool   SetContent(PGLOBAL g, char *txtp, int len);
-  virtual PXNODE Clone(PGLOBAL g, PXNODE np);
-  virtual PXLIST GetChildElements(PGLOBAL g, char *xp, PXLIST lp);
-  virtual PXLIST SelectNodes(PGLOBAL g, char *xp, PXLIST lp);
-  virtual PXNODE SelectSingleNode(PGLOBAL g, char *xp, PXNODE np);
-  virtual PXATTR GetAttribute(PGLOBAL g, char *name, PXATTR ap);
-  virtual PXNODE AddChildNode(PGLOBAL g, PCSZ name, PXNODE np);
-  virtual PXATTR AddProperty(PGLOBAL g, char *name, PXATTR ap);
-  virtual void   AddText(PGLOBAL g, PCSZ txtp);
-  virtual void   DeleteChild(PGLOBAL g, PXNODE dnp);
+  RCODE  GetContent(PGLOBAL g, char *buf, int len) override;
+  bool   SetContent(PGLOBAL g, char *txtp, int len) override;
+  PXNODE Clone(PGLOBAL g, PXNODE np) override;
+  PXLIST GetChildElements(PGLOBAL g, char *xp, PXLIST lp) override;
+  PXLIST SelectNodes(PGLOBAL g, char *xp, PXLIST lp) override;
+  PXNODE SelectSingleNode(PGLOBAL g, char *xp, PXNODE np) override;
+  PXATTR GetAttribute(PGLOBAL g, char *name, PXATTR ap) override;
+  PXNODE AddChildNode(PGLOBAL g, PCSZ name, PXNODE np) override;
+  PXATTR AddProperty(PGLOBAL g, char *name, PXATTR ap) override;
+  void   AddText(PGLOBAL g, PCSZ txtp) override;
+  void   DeleteChild(PGLOBAL g, PXNODE dnp) override;
 
  protected:
   // Constructor
@@ -141,9 +141,9 @@ class XML2NODELIST : public XMLNODELIST {
   friend class XML2NODE;
  public:
   // Methods
-  virtual int    GetLength(void);
-  virtual PXNODE GetItem(PGLOBAL g, int n, PXNODE np);
-  virtual bool   DropItem(PGLOBAL g, int n);
+  int    GetLength(void) override;
+  PXNODE GetItem(PGLOBAL g, int n, PXNODE np) override;
+  bool   DropItem(PGLOBAL g, int n) override;
 
  protected:
   // Constructor
@@ -161,12 +161,12 @@ class XML2ATTR : public XMLATTRIBUTE {
   friend class XML2NODE;
  public:
   // Properties
-  virtual char  *GetName(PGLOBAL g) {return (char*)Atrp->name;}
-  virtual PXATTR GetNext(PGLOBAL g);
+  char  *GetName(PGLOBAL g) override {return (char*)Atrp->name;}
+  PXATTR GetNext(PGLOBAL g) override;
 
   // Methods
-  virtual RCODE  GetText(PGLOBAL g, char *bufp, int len);
-  virtual bool   SetText(PGLOBAL g, char *txtp, int len);
+  RCODE  GetText(PGLOBAL g, char *bufp, int len) override;
+  bool   SetText(PGLOBAL g, char *txtp, int len) override;
 
  protected:
   // Constructor
