@@ -39,12 +39,12 @@
 /***********************************************************************/
 #include "my_global.h"
 #include "my_pthread.h"
-#if defined(__WIN__)
+#if defined(_WIN32)
 #include <io.h>
 #include <fcntl.h>
 #include <errno.h>
 #define BIGMEM         1048576            // 1 Megabyte
-#else     // !__WIN__
+#else     // !_WIN32
 #include <unistd.h>
 #include <fcntl.h>
 //#if defined(THREAD)
@@ -52,7 +52,7 @@
 //#endif   // THREAD
 #include <stdarg.h>
 #define BIGMEM      2147483647            // Max int value
-#endif    // !__WIN__
+#endif    // !_WIN32
 #include <locale.h>
 
 /***********************************************************************/
@@ -89,11 +89,11 @@ extern "C" {
 extern char version[];
 } // extern "C"
 
-//#if defined(__WIN__)
+//#if defined(_WIN32)
 //extern CRITICAL_SECTION parsec;      // Used calling the Flex parser
-//#else   // !__WIN__
+//#else   // !_WIN32
 extern pthread_mutex_t parmut;
-//#endif  // !__WIN__
+//#endif  // !_WIN32
 
 // The debug trace used by the main thread
 FILE *pfile = NULL;
@@ -313,7 +313,7 @@ PDBUSER PlgMakeUser(PGLOBAL g)
   PDBUSER dbuserp;
 
   if (!(dbuserp = (PDBUSER)malloc(sizeof(DBUSERBLK)))) {
-    sprintf(g->Message, MSG(MALLOC_ERROR), "PlgMakeUser");
+    snprintf(g->Message, sizeof(g->Message), MSG(MALLOC_ERROR), "PlgMakeUser");
     return NULL;
     } // endif dbuserp
 
@@ -381,19 +381,19 @@ char *SetPath(PGLOBAL g, const char *path)
 			return NULL;
 
 		if (PlugIsAbsolutePath(path)) {
-			strcpy(buf, path);
+			snprintf(buf, len, "%s", path);
 			return buf;
 		} // endif path
 
 		if (*path != '.') {
-#if defined(__WIN__)
+#if defined(_WIN32)
 			const char *s = "\\";
-#else   // !__WIN__
+#else   // !_WIN32
 			const char *s = "/";
-#endif  // !__WIN__
-			strcat(strcat(strcat(strcpy(buf, "."), s), path), s);
+#endif  // !_WIN32
+			snprintf(buf, len, ".%s%s%s", s, path, s);
 		} else
-			strcpy(buf, path);
+			snprintf(buf, len, "%s", path);
 
 	} // endif path
 
@@ -409,14 +409,14 @@ char *ExtractFromPath(PGLOBAL g, char *pBuff, char *FileName, OPVAL op)
   char *drive = NULL, *direc = NULL, *fname = NULL, *ftype = NULL;
 
   switch (op) {           // Determine which part to extract
-#if defined(__WIN__)
+#if defined(_WIN32)
     case OP_FDISK: drive = pBuff; break;
 #endif   // !UNIX
     case OP_FPATH: direc = pBuff; break;
     case OP_FNAME: fname = pBuff; break;
     case OP_FTYPE: ftype = pBuff; break;
     default:
-      sprintf(g->Message, MSG(INVALID_OPER), op, "ExtractFromPath");
+      snprintf(g->Message, sizeof(g->Message), MSG(INVALID_OPER), op, "ExtractFromPath");
       return NULL;
     } // endswitch op
 
@@ -425,12 +425,13 @@ char *ExtractFromPath(PGLOBAL g, char *pBuff, char *FileName, OPVAL op)
   return pBuff;
   } // end of PlgExtractFromPath
 
+
+#ifdef NOT_USED
 /***********************************************************************/
-/*  Check the occurence and matching of a pattern against a string.    */
+/*  Check the occurrence and matching of a pattern against a string.    */
 /*  Because this function is only used for catalog name checking,      */
 /*  it must be case insensitive.                                       */
 /***********************************************************************/
-#ifdef NOT_USED
 static bool PlugCheckPattern(PGLOBAL g, LPCSTR string, LPCSTR pat)
   {
   if (pat && strlen(pat)) {
@@ -444,7 +445,7 @@ static bool PlugCheckPattern(PGLOBAL g, LPCSTR string, LPCSTR pat)
     return true;
 
   } // end of PlugCheckPattern
-#endif
+#endif /* NOT_USED */
 
 /***********************************************************************/
 /*  PlugEvalLike: evaluates a LIKE clause.                             */
@@ -571,7 +572,7 @@ bool EvalLikePattern(LPCSTR sp, LPCSTR tp)
     b = (t || !*sp);                  /*   true if %  or void strg.    */
   else if (!t) {
     /*******************************************************************/
-    /*  No character to skip, check occurence of <subtring-specifier>  */
+    /*  No character to skip, check occurrence of <subtring-specifier>  */
     /*  at the very beginning of remaining string.                     */
     /*******************************************************************/
     if (p) {
@@ -585,7 +586,7 @@ bool EvalLikePattern(LPCSTR sp, LPCSTR tp)
     if (p)
       /*****************************************************************/
       /*  Here is the case explaining why we need a recursive routine. */
-      /*  The test must be done not only against the first occurence   */
+      /*  The test must be done not only against the first occurrence   */
       /*  of the <substring-specifier> in the remaining string,        */
       /*  but also with all eventual succeeding ones.                  */
       /*****************************************************************/
@@ -1251,7 +1252,7 @@ void *PlgDBalloc(PGLOBAL g, void *area, MBLOCK& mp)
     // For allocations greater than one fourth of remaining storage
     // in the area, do allocate from virtual storage.
 		const char*v = "malloc";
-#if defined(__WIN__)
+#if defined(_WIN32)
 		if (mp.Size >= BIGMEM) {
 			v = "VirtualAlloc";
 			mp.Memp = VirtualAlloc(NULL, mp.Size, MEM_COMMIT | MEM_RESERVE, PAGE_READWRITE);
@@ -1354,7 +1355,7 @@ void PlgDBfree(MBLOCK& mp)
   {
 	if (!mp.Sub && mp.Memp) {
 		const char*v = "free";
-#if defined(__WIN__)
+#if defined(_WIN32)
 		if (mp.Size >= BIGMEM) {
 			v = "VirtualFree";
 			VirtualFree(mp.Memp, 0, MEM_RELEASE);
@@ -1398,7 +1399,7 @@ void *PlgDBSubAlloc(PGLOBAL g, void *memp, size_t size)
          memp, size, pph->To_Free, pph->FreeBlk);
 
   if (size > pph->FreeBlk) {   /* Not enough memory left in pool */
-    sprintf(g->Message,
+    snprintf(g->Message, sizeof(g->Message),
     "Not enough memory in Work area for request of %zd (used=%zd free=%zd)",
             size, pph->To_Free, pph->FreeBlk);
 
@@ -1529,7 +1530,7 @@ DllExport void NewPointer(PTABS t, void *oldv, void *newv)
     if (!(tp = new TABPTR)) {
       PGLOBAL g = t->G;
 
-      sprintf(g->Message, "NewPointer: %s", MSG(MEM_ALLOC_ERROR));
+      snprintf(g->Message, sizeof(g->Message), "NewPointer: %s", MSG(MEM_ALLOC_ERROR));
 			throw 3;
 		} else {
       tp->Next = t->P1;
@@ -1556,15 +1557,15 @@ int FileComp(PGLOBAL g, char *file1, char *file2)
   bp[0] = buff1; bp[1] = buff2;
 
   for (i = 0; i < 2; i++) {
-#if defined(__WIN__)
+#if defined(_WIN32)
     h[i]= global_open(g, MSGID_NONE, fn[i], _O_RDONLY | _O_BINARY);
-#else   // !__WIN__
+#else   // !_WIN32
     h[i]= global_open(g, MSGOD_NONE, fn[i], O_RDONLY);
-#endif  // !__WIN__
+#endif  // !_WIN32
 
     if (h[i] == -1) {
 //      if (errno != ENOENT) {
-        sprintf(g->Message, MSG(OPEN_MODE_ERROR),
+        snprintf(g->Message, sizeof(g->Message), MSG(OPEN_MODE_ERROR),
                 "rb", (int)errno, fn[i]);
         strcat(strcat(g->Message, ": "), strerror(errno));
 				throw 666;
@@ -1573,7 +1574,7 @@ int FileComp(PGLOBAL g, char *file1, char *file2)
 
     } else {
       if ((len[i] = _filelength(h[i])) < 0) {
-        sprintf(g->Message, MSG(FILELEN_ERROR), "_filelength", fn[i]);
+        snprintf(g->Message, sizeof(g->Message), MSG(FILELEN_ERROR), "_filelength", fn[i]);
 				throw 666;
 			} // endif len
 
@@ -1587,7 +1588,7 @@ int FileComp(PGLOBAL g, char *file1, char *file2)
   while (rc == -1) {
     for (i = 0; i < 2; i++)
       if ((n[i] = read(h[i], bp[i], 4096)) < 0) {
-        sprintf(g->Message, MSG(READ_ERROR), fn[i], strerror(errno));
+        snprintf(g->Message, sizeof(g->Message), MSG(READ_ERROR), fn[i], strerror(errno));
         goto fin;
         } // endif n
 
