@@ -1,7 +1,7 @@
 /*****************************************************************************
 
 Copyright (c) 1995, 2017, Oracle and/or its affiliates. All Rights Reserved.
-Copyright (c) 2017, 2020, MariaDB Corporation.
+Copyright (c) 2017, 2022, MariaDB Corporation.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free Software
@@ -53,10 +53,10 @@ class buf_dblwr_t
     element* buf_block_arr;
   };
 
-  /** the page number of the first doublewrite block (block_size() pages) */
-  page_id_t block1= page_id_t(0, 0);
-  /** the page number of the second doublewrite block (block_size() pages) */
-  page_id_t block2= page_id_t(0, 0);
+  /** the page number of the first doublewrite block (block_size pages) */
+  page_id_t block1{0, 0};
+  /** the page number of the second doublewrite block (block_size pages) */
+  page_id_t block2{0, 0};
 
   /** mutex protecting the data members below */
   mysql_mutex_t mutex;
@@ -66,103 +66,124 @@ class buf_dblwr_t
   bool batch_running;
   /** number of expected flush_buffered_writes_completed() calls */
   unsigned flushing_buffered_writes;
-  /** pages submitted to flush_buffered_writes() */
-  ulint pages_submitted;
   /** number of flush_buffered_writes_completed() calls */
   ulint writes_completed;
   /** number of pages written by flush_buffered_writes_completed() */
   ulint pages_written;
 
   slot slots[2];
-  slot *active_slot= &slots[0];
+  slot *active_slot;
 
-  /** Initialize the doublewrite buffer data structure.
-  @param header   doublewrite page header in the TRX_SYS page */
-  inline void init(const byte *header);
-
-  /** Flush possible buffered writes to persistent storage. */
-  bool flush_buffered_writes(const ulint size);
+  /** Size of the doublewrite block in pages */
+  uint32_t block_size;
 
 public:
+  /** Values of use */
+  enum usage {
+    /** Assume that writes are atomic */
+    USE_NO= 0,
+    /** Use the doublewrite buffer with full durability */
+    USE_YES,
+    /** Durable writes to the doublewrite buffer, not to data files */
+    USE_FAST
+  };
+  /** The value of innodb_doublewrite */
+  ulong use;
+private:
+  /** Initialise the persistent storage of the doublewrite buffer.
+  @param header   doublewrite page header in the TRX_SYS page */
+  inline void init(const byte *header) noexcept;
+
+  /** Flush possible buffered writes to persistent storage. */
+  bool flush_buffered_writes(const ulint size) noexcept;
+
+public:
+  /** Initialise the doublewrite buffer data structures. */
+  void init() noexcept;
   /** Create or restore the doublewrite buffer in the TRX_SYS page.
   @return whether the operation succeeded */
-  bool create();
+  bool create() noexcept;
   /** Free the doublewrite buffer. */
-  void close();
+  void close() noexcept;
 
   /** Acquire the mutex */
-  void lock() { mysql_mutex_lock(&mutex); }
-  /** @return the number of submitted page writes */
-  ulint submitted() const
-  { mysql_mutex_assert_owner(&mutex); return pages_submitted; }
+  void lock() noexcept { mysql_mutex_lock(&mutex); }
   /** @return the number of completed batches */
-  ulint batches() const
+  ulint batches() const noexcept
   { mysql_mutex_assert_owner(&mutex); return writes_completed; }
   /** @return the number of final pages written */
-  ulint written() const
+  ulint written() const noexcept
   { mysql_mutex_assert_owner(&mutex); return pages_written; }
   /** Release the mutex */
-  void unlock() { mysql_mutex_unlock(&mutex); }
+  void unlock() noexcept { mysql_mutex_unlock(&mutex); }
 
   /** Initialize the doublewrite buffer memory structure on recovery.
   If we are upgrading from a version before MySQL 4.1, then this
   function performs the necessary update operations to support
   innodb_file_per_table. If we are in a crash recovery, this function
-  loads the pages from double write buffer into memory.
+  loads the pages from double write buffer which are not older than
+  the checkpoint into memory.
   @param file File handle
   @param path Path name of file
   @return DB_SUCCESS or error code */
-  dberr_t init_or_load_pages(pfs_os_file_t file, const char *path);
+  dberr_t init_or_load_pages(pfs_os_file_t file, const char *path) noexcept;
 
   /** Process and remove the double write buffer pages for all tablespaces. */
-  void recover();
+  void recover() noexcept;
 
   /** Update the doublewrite buffer on data page write completion. */
-  void write_completed();
+  void write_completed() noexcept;
   /** Flush possible buffered writes to persistent storage.
   It is very important to call this function after a batch of writes has been
   posted, and also when we may have to wait for a page latch!
   Otherwise a deadlock of threads can occur. */
-  void flush_buffered_writes();
+  void flush_buffered_writes() noexcept;
   /** Update the doublewrite buffer on write batch completion
   @param request  the completed batch write request */
-  void flush_buffered_writes_completed(const IORequest &request);
-
-  /** Size of the doublewrite block in pages */
-  uint32_t block_size() const { return FSP_EXTENT_SIZE; }
+  void flush_buffered_writes_completed(const IORequest &request) noexcept;
 
   /** Schedule a page write. If the doublewrite memory buffer is full,
   flush_buffered_writes() will be invoked to make space.
   @param request    asynchronous write request
   @param size       payload size in bytes */
-  void add_to_batch(const IORequest &request, size_t size);
+  void add_to_batch(const IORequest &request, size_t size) noexcept;
 
-  /** Determine whether the doublewrite buffer is initialized */
-  bool is_initialised() const
+  /** Determine whether the doublewrite buffer has been created */
+  bool is_created() const noexcept
   { return UNIV_LIKELY(block1 != page_id_t(0, 0)); }
 
-  /** @return whether a page identifier is part of the doublewrite buffer */
-  bool is_inside(const page_id_t id) const
+  /** @return whether the doublewrite buffer is in use */
+  bool in_use() const { return is_created() && use; }
+  /** @return whether fsync() is needed on non-doublewrite pages */
+  bool need_fsync() const { return use < USE_FAST; }
+
+  void set_use(ulong use)
   {
-    if (!is_initialised())
+    ut_ad(use <= USE_FAST);
+    mysql_mutex_lock(&mutex);
+    this->use= use;
+    mysql_mutex_unlock(&mutex);
+  }
+
+  /** @return whether a page identifier is part of the doublewrite buffer */
+  bool is_inside(const page_id_t id) const noexcept
+  {
+    if (!is_created())
       return false;
     ut_ad(block1 < block2);
     if (id < block1)
       return false;
-    const uint32_t size= block_size();
-    return id < block1 + size || (id >= block2 && id < block2 + size);
+    return id < block1 + block_size ||
+      (id >= block2 && id < block2 + block_size);
   }
 
   /** Wait for flush_buffered_writes() to be fully completed */
-  void wait_flush_buffered_writes()
+  void wait_flush_buffered_writes() noexcept
   {
-    if (is_initialised())
-    {
-      mysql_mutex_lock(&mutex);
-      while (batch_running)
-        my_cond_wait(&cond, &mutex.m_mutex);
-      mysql_mutex_unlock(&mutex);
-    }
+    mysql_mutex_lock(&mutex);
+    while (batch_running)
+      my_cond_wait(&cond, &mutex.m_mutex);
+    mysql_mutex_unlock(&mutex);
   }
 };
 
