@@ -2,7 +2,7 @@
 
 Copyright (c) 1996, 2018, Oracle and/or its affiliates. All Rights Reserved.
 Copyright (c) 2012, Facebook Inc.
-Copyright (c) 2013, 2022, MariaDB Corporation.
+Copyright (c) 2013, 2023, MariaDB Corporation.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free Software
@@ -146,21 +146,21 @@ dict_table_open_on_id(table_id_t table_id, bool dict_locked,
                       MDL_ticket **mdl= nullptr)
   MY_ATTRIBUTE((warn_unused_result));
 
-/** Release a metadata lock.
-@param thd    connection that holds mdl
-@param mdl    metadata lock, or nullptr */
-void mdl_release(THD *thd, MDL_ticket *mdl) noexcept;
+/** Decrement the count of open handles */
+void dict_table_close(dict_table_t *table);
 
-/** Release a table reference and a metadata lock.
-@param table  referenced table
-@param thd    connection that holds mdl
-@param mdl    metadata lock, or nullptr */
-inline void dict_table_close(dict_table_t* table, THD *thd, MDL_ticket *mdl)
-  noexcept
-{
-  table->release();
-  mdl_release(thd, mdl);
-}
+/** Decrements the count of open handles of a table.
+@param[in,out]	table		table
+@param[in]	dict_locked	whether dict_sys.latch is being held
+@param[in]	thd		thread to release MDL
+@param[in]	mdl		metadata lock or NULL if the thread is a
+				foreground one. */
+void
+dict_table_close(
+	dict_table_t*	table,
+	bool		dict_locked,
+	THD*		thd = NULL,
+	MDL_ticket*	mdl = NULL);
 
 /*********************************************************************//**
 Gets the minimum number of bytes per character.
@@ -618,8 +618,6 @@ dict_table_get_next_index(
 #define dict_index_is_auto_gen_clust(index) (index)->is_gen_clust()
 #define dict_index_is_unique(index) (index)->is_unique()
 #define dict_index_is_spatial(index) (index)->is_spatial()
-#define dict_index_is_ibuf(index) (index)->is_ibuf()
-#define dict_index_is_sec_or_ibuf(index) !(index)->is_primary()
 #define dict_index_has_virtual(index) (index)->has_virtual()
 
 /** Get all the FTS indexes on a table.
@@ -634,7 +632,7 @@ dict_table_get_all_fts_indexes(
 /********************************************************************//**
 Gets the number of user-defined non-virtual columns in a table in the
 dictionary cache.
-@return number of user-defined (e.g., not ROW_ID) non-virtual
+@return number of user-defined (e.g., not DB_ROW_ID) non-virtual
 columns of a table */
 UNIV_INLINE
 unsigned
@@ -674,7 +672,7 @@ TPOOL_SUPPRESS_TSAN
 @return estimated number of rows */
 inline uint64_t dict_table_get_n_rows(const dict_table_t *table)
 {
-  ut_ad(table->stat_initialized());
+  ut_ad(table->stat_initialized);
   return table->stat_n_rows;
 }
 
@@ -1347,27 +1345,10 @@ private:
   std::atomic<table_id_t> temp_table_id{DICT_HDR_FIRST_ID};
   /** hash table of temporary table IDs */
   hash_table_t temp_id_hash;
-  /** the next value of DB_ROW_ID, backed by DICT_HDR_ROW_ID
-  (FIXME: remove this, and move to dict_table_t) */
-  Atomic_relaxed<row_id_t> row_id;
-  /** The synchronization interval of row_id */
-  static constexpr size_t ROW_ID_WRITE_MARGIN= 256;
 public:
   /** Diagnostic message for exceeding the lock_wait() timeout */
   static const char fatal_msg[];
 
-  /** @return A new value for GEN_CLUST_INDEX(DB_ROW_ID) */
-  inline row_id_t get_new_row_id() noexcept;
-
-  /** Ensure that row_id is not smaller than id, on IMPORT TABLESPACE */
-  inline void update_row_id(row_id_t id) noexcept;
-
-  /** Recover the global DB_ROW_ID sequence on database startup */
-  void recover_row_id(row_id_t id) noexcept
-  {
-    row_id= ut_uint64_align_up(id, ROW_ID_WRITE_MARGIN) + ROW_ID_WRITE_MARGIN;
-  }
-
   /** @return a new temporary table ID */
   table_id_t acquire_temporary_table_id() noexcept
   {
@@ -1657,27 +1638,6 @@ bool
 dict_table_have_virtual_index(
 	dict_table_t*	table);
 
-/** Helper for opening the InnoDB persistent statistics tables */
-class dict_stats final
-{
-  MDL_context *mdl_context= nullptr;
-  MDL_ticket *mdl_table= nullptr, *mdl_index= nullptr;
-  dict_table_t *table_stats= nullptr, *index_stats= nullptr;
-
-public:
-  dict_stats()= default;
-
-  /** Open the statistics tables.
-  @return whether the operation failed */
-  bool open(THD *thd) noexcept;
-
-  /** Close the statistics tables after !open_tables(thd). */
-  void close() noexcept;
-
-  dict_table_t *table() const noexcept { return table_stats; }
-  dict_table_t *index() const noexcept { return index_stats; }
-};
-
 #include "dict0dict.inl"
 
 #endif
