@@ -355,9 +355,9 @@ and how many pages are currently used.
 @param[out]     used    number of pages that are used (not more than reserved)
 @param[in,out]  mtr     mini-transaction
 @return number of reserved pages */
-uint32_t fseg_n_reserved_pages(const buf_block_t &block,
-                               const fseg_header_t *header, uint32_t *used,
-                               mtr_t *mtr) noexcept
+ulint fseg_n_reserved_pages(const buf_block_t &block,
+                            const fseg_header_t *header, ulint *used,
+                            mtr_t *mtr)
   MY_ATTRIBUTE((nonnull));
 /**********************************************************************//**
 Allocates a single free page from a segment. This function implements
@@ -549,6 +549,13 @@ inline void fsp_init_file_page(
 	mtr->init(block);
 }
 
+/** Truncate the system tablespace
+@param shutdown Called during shutdown */
+void fsp_system_tablespace_truncate(bool shutdown);
+
+/** Truncate the temporary tablespace */
+void fsp_shrink_temp_space();
+
 #ifndef UNIV_DEBUG
 # define fsp_init_file_page(space, block, mtr) fsp_init_file_page(block, mtr)
 #endif
