@@ -309,15 +309,6 @@ public:
   @retval 0 if the transaction only modified temporary tablespaces */
   lsn_t commit_lsn() const { ut_ad(has_committed()); return m_commit_lsn; }
 
-  /** Note that we are inside the change buffer code. */
-  void enter_ibuf() { m_inside_ibuf= true; }
-
-  /** Note that we have exited from the change buffer code. */
-  void exit_ibuf() { m_inside_ibuf= false; }
-
-  /** @return true if we are inside the change buffer code */
-  bool is_inside_ibuf() const { return m_inside_ibuf; }
-
   /** Note that some pages have been freed */
   void set_trim_pages() { m_trim_pages= true; }
 
@@ -435,8 +426,10 @@ public:
     m_memo.emplace_back(mtr_memo_slot_t{object, type});
   }
 
+  /** @return the size of the log is empty */
+  size_t get_log_size() const { return m_log.size(); }
   /** @return whether the log and memo are empty */
-  bool is_empty() const { return !get_savepoint() && m_log.empty(); }
+  bool is_empty() const { return !get_savepoint() && !get_log_size(); }
 
   /** Write an OPT_PAGE_CHECKSUM record. */
   inline void page_checksum(const buf_page_t &bpage);
@@ -616,11 +609,10 @@ public:
   @param type           file operation
   @param space_id       tablespace identifier
   @param path           file path
-  @param new_path       new file path for type=FILE_RENAME
-  @return number of bytes written */
-  inline size_t log_file_op(mfile_type_t type, uint32_t space_id,
-                            const char *path,
-                            const char *new_path= nullptr) noexcept;
+  @param new_path       new file path for type=FILE_RENAME */
+  inline void log_file_op(mfile_type_t type, uint32_t space_id,
+                          const char *path,
+                          const char *new_path= nullptr);
 
   /** Add freed page numbers to freed_pages */
   void add_freed_offset(fil_space_t *space, uint32_t page)
@@ -645,6 +637,17 @@ public:
   /** Note that log_sys.latch is no longer being held exclusively. */
   void flag_wr_unlock() noexcept { ut_ad(m_latch_ex); m_latch_ex= false; }
 
+  /** type of page flushing is needed during commit() */
+  enum page_flush_ahead
+  {
+    /** no need to trigger page cleaner */
+    PAGE_FLUSH_NO= 0,
+    /** asynchronous flushing is needed */
+    PAGE_FLUSH_ASYNC,
+    /** furious flushing is needed */
+    PAGE_FLUSH_SYNC
+  };
+
 private:
   /** Handle any pages that were freed during the mini-transaction. */
   void process_freed_pages();
@@ -680,52 +683,42 @@ private:
   tablespace was modified for the first time since fil_names_clear(). */
   ATTRIBUTE_NOINLINE ATTRIBUTE_COLD void name_write() noexcept;
 
-  /** Encrypt the log
-  @return the total size in bytes, excluding the 8-byte nonce */
-  ATTRIBUTE_NOINLINE size_t encrypt() noexcept;
-
-  /** Calculate m_crc of m_log.
-  @return the total size in bytes, including the 5-byte trailer and CRC-32C */
-  ATTRIBUTE_NOINLINE size_t crc32c() noexcept;
+  /** Encrypt the log */
+  ATTRIBUTE_NOINLINE void encrypt();
 
   /** Commit the mini-transaction log.
   @tparam pmem log_sys.is_mmap()
   @param mtr   mini-transaction
-  @param lsns  {start_lsn,flush_ahead_lsn} */
+  @param lsns  {start_lsn,flush_ahead} */
   template<bool pmem>
-  static void commit_log(mtr_t *mtr, std::pair<lsn_t,lsn_t> lsns) noexcept;
-
-  /** Release log_sys.latch. */
-  void commit_log_release() noexcept;
+  static void commit_log(mtr_t *mtr, std::pair<lsn_t,page_flush_ahead> lsns);
 
   /** Append the redo log records to the redo log buffer.
-  @return {start_lsn,flush_ahead_lsn} */
-  std::pair<lsn_t,lsn_t> do_write() noexcept;
+  @return {start_lsn,flush_ahead} */
+  std::pair<lsn_t,page_flush_ahead> do_write();
 
   /** Append the redo log records to the redo log buffer.
+  @tparam spin whether to use the spin-only log_sys.lock_lsn()
   @tparam mmap log_sys.is_mmap()
   @param mtr   mini-transaction
   @param len   number of bytes to write
-  @return {start_lsn,flush_ahead_lsn} */
-  template<bool mmap> static
-  std::pair<lsn_t,lsn_t> finish_writer(mtr_t *mtr, size_t len);
+  @return {start_lsn,flush_ahead} */
+  template<bool spin,bool mmap> static
+  std::pair<lsn_t,page_flush_ahead> finish_writer(mtr_t *mtr, size_t len);
 
   /** The applicable variant of commit_log() */
-  static void (*commit_logger)(mtr_t *, std::pair<lsn_t,lsn_t>);
+  static void (*commit_logger)(mtr_t *, std::pair<lsn_t,page_flush_ahead>);
   /** The applicable variant of finish_writer() */
-  static std::pair<lsn_t,lsn_t> (*finisher)(mtr_t *, size_t);
+  static std::pair<lsn_t,page_flush_ahead> (*finisher)(mtr_t *, size_t);
 
-  std::pair<lsn_t,lsn_t> finish_write(size_t len)
+  std::pair<lsn_t,page_flush_ahead> finish_write(size_t len)
   { return finisher(this, len); }
 public:
+  /** Poll interval in log_sys.lock_lsn(); 0 to use log_sys.lsn_lock.
+  Protected by LOCK_global_system_variables and log_sys.latch. */
+  static unsigned spin_wait_delay;
   /** Update finisher when spin_wait_delay is changing to or from 0. */
   static void finisher_update();
-
-  /** Decode the length of a record.
-  @param l     log record
-  @param size  total size of the record
-  @return the log record payload after the encoded length */
-  static const byte *parse_length(const byte *l, uint32_t *size) noexcept;
 private:
 
   /** Release all latches. */
@@ -770,10 +763,6 @@ private:
   /** whether log_sys.latch is locked exclusively */
   uint16_t m_latch_ex:1;
 
-  /** whether change buffer is latched; only needed in non-debug builds
-  to suppress some read-ahead operations, @see ibuf_inside() */
-  uint16_t m_inside_ibuf:1;
-
   /** whether the pages has been trimmed */
   uint16_t m_trim_pages:1;
 
