@@ -1,7 +1,7 @@
 /*****************************************************************************
 
 Copyright (c) 1996, 2016, Oracle and/or its affiliates. All Rights Reserved.
-Copyright (c) 2015, 2022, MariaDB Corporation.
+Copyright (c) 2015, 2023, MariaDB Corporation.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free Software
@@ -638,8 +638,7 @@ public:
   {
     /** The largest encountered transaction identifier for which no
     transaction was observed to be active. This is a cache to speed up
-    trx_sys_t::find_same_or_older() as well as to elide some calls to
-    trx_sys_t::find().
+    trx_sys_t::find_same_or_older().
 
     This will be zero-initialized in Pool::Pool() and not initialized
     when a transaction object in the pool is freed and reused. The
@@ -710,7 +709,7 @@ public:
 
   Regular transactions:
   * NOT_STARTED -> ACTIVE -> COMMITTED -> NOT_STARTED
-  * NOT_STARTED -> ABORTED (when THD::mark_transaction_to_rollback() is called)
+  * NOT_STARTED -> ABORTED (when thd_mark_transaction_to_rollback() is called)
   * ABORTED -> NOT_STARTED (acknowledging the rollback of a transaction)
 
   Auto-commit non-locking read-only:
@@ -810,13 +809,9 @@ public:
   /** normally set; "SET unique_checks=0, foreign_key_checks=0"
   enables bulk insert into an empty table */
   unsigned check_unique_secondary:1;
-  /** whether an insert into an empty table is active
-  Possible states are
-  TRX_NO_BULK
-  TRX_DML_BULK
-  TRX_DDL_BULK
-  @see trx_bulk_insert in trx0types.h */
-  unsigned bulk_insert:2;
+
+  /** whether an insert into an empty table is active */
+  unsigned bulk_insert:1;
 	/*------------------------------*/
 	/* MySQL has a transaction coordinator to coordinate two phase
 	commit between multiple storage engines and the binary log. When
@@ -1097,10 +1092,8 @@ public:
   bool has_stats_table_lock() const;
 
   /** Free the memory to trx_pools */
-  void free() noexcept;
+  void free();
 
-  /** Clear commit_lsn and free the memory */
-  void clear_and_free() noexcept { ut_d(commit_lsn= 0;) free(); }
 
   void assert_freed() const
   {
@@ -1125,7 +1118,6 @@ public:
     ut_ad(!is_not_inheriting_locks());
     ut_ad(check_foreigns);
     ut_ad(check_unique_secondary);
-    ut_ad(bulk_insert == TRX_NO_BULK);
   }
 
   /** This has to be invoked on SAVEPOINT or at the end of a statement.
@@ -1151,8 +1143,6 @@ public:
   rollback to the start of a statement will work. */
   void end_bulk_insert()
   {
-    if (bulk_insert == TRX_DDL_BULK)
-      return;
     for (auto& t : mod_tables)
       t.second.end_bulk_insert();
   }
@@ -1160,15 +1150,7 @@ public:
   /** @return whether a bulk insert into empty table is in progress */
   bool is_bulk_insert() const
   {
-    switch (bulk_insert) {
-    case TRX_NO_BULK:
-      return false;
-    case TRX_DDL_BULK:
-      return true;
-    default:
-      ut_ad(bulk_insert == TRX_DML_BULK);
-    }
-    if (check_unique_secondary || check_foreigns)
+    if (!bulk_insert || check_unique_secondary || check_foreigns)
       return false;
     for (const auto& t : mod_tables)
       if (t.second.is_bulk_insert())
@@ -1198,11 +1180,9 @@ public:
   /** Do the bulk insert for the buffered insert operation
   for the transaction.
   @return DB_SUCCESS or error code */
-  template<trx_bulk_insert type= TRX_DML_BULK>
   dberr_t bulk_insert_apply()
   {
-    static_assert(type != TRX_NO_BULK, "");
-    return bulk_insert == type ? bulk_insert_apply_low(): DB_SUCCESS;
+    return UNIV_UNLIKELY(bulk_insert) ? bulk_insert_apply_low(): DB_SUCCESS;
   }
 
 private:
