@@ -211,6 +211,17 @@ close_and_exit:
 	always refer to an existing undo log record. */
 	ut_ad(row_get_rec_trx_id(rec, index, offsets));
 
+#ifdef ENABLED_DEBUG_SYNC
+  DBUG_EXECUTE_IF("enable_row_purge_remove_clust_if_poss_low_sync_point",
+                  debug_sync_set_action
+                  (current_thd,
+                   STRING_WITH_LEN(
+                     "now SIGNAL "
+                       "row_purge_remove_clust_if_poss_low_before_delete "
+                     "WAIT_FOR "
+                       "row_purge_remove_clust_if_poss_low_cont"));
+                  );
+#endif
 	if (mode == BTR_MODIFY_LEAF) {
 		success = DB_FAIL != btr_cur_optimistic_delete(
 			btr_pcur_get_btr_cur(&node->pcur), 0, &mtr);
@@ -267,6 +278,448 @@ row_purge_remove_clust_if_poss(
 	return(false);
 }
 
+/** Check a virtual column value index secondary virtual index matches
+that of current cluster index record, which is recreated from information
+stored in undo log
+@param[in]	rec		record in the clustered index
+@param[in]	icentry		the index entry built from a cluster row
+@param[in]	clust_index	cluster index
+@param[in]	clust_offsets	offsets on the cluster record
+@param[in]	index		the secondary index
+@param[in]	ientry		the secondary index entry
+@param[in]	roll_ptr	the rollback pointer for the purging record
+@param[in]	trx_id		trx id for the purging record
+@param[in,out]	mtr		mini-transaction
+@param[in,out]	v_row		dtuple holding the virtual rows (if needed)
+@return true if matches, false otherwise */
+static
+bool
+row_purge_vc_matches_cluster(
+	const rec_t*	rec,
+	const dtuple_t* icentry,
+	dict_index_t*	clust_index,
+	rec_offs*	clust_offsets,
+	dict_index_t*	index,
+	const dtuple_t* ientry,
+	roll_ptr_t	roll_ptr,
+	trx_id_t	trx_id,
+	mtr_t*		mtr,
+	dtuple_t**	vrow)
+{
+	const rec_t*	version;
+	rec_t*          prev_version;
+	mem_heap_t*	heap2;
+	mem_heap_t*	heap = NULL;
+	mem_heap_t*	tuple_heap;
+	ulint		num_v = dict_table_get_n_v_cols(index->table);
+	bool		compare[REC_MAX_N_FIELDS];
+	ulint		n_fields = dtuple_get_n_fields(ientry);
+	ulint		n_non_v_col = 0;
+	ulint		n_cmp_v_col = 0;
+	const dfield_t* field1;
+	dfield_t*	field2;
+	ulint		i;
+
+	/* First compare non-virtual columns (primary keys) */
+	ut_ad(index->n_fields == n_fields);
+	ut_ad(n_fields == dtuple_get_n_fields(icentry));
+	ut_ad(mtr->memo_contains_page_flagged(rec,
+					      MTR_MEMO_PAGE_S_FIX
+					      | MTR_MEMO_PAGE_X_FIX));
+
+	{
+		const dfield_t* a = ientry->fields;
+		const dfield_t* b = icentry->fields;
+
+		for (const dict_field_t *ifield = index->fields,
+			     *const end = &index->fields[index->n_fields];
+		     ifield != end; ifield++, a++, b++) {
+			if (!ifield->col->is_virtual()) {
+				if (cmp_dfield_dfield(a, b)) {
+					return false;
+				}
+				n_non_v_col++;
+			}
+		}
+	}
+
+	tuple_heap = mem_heap_create(1024);
+
+	ut_ad(n_fields > n_non_v_col);
+
+	*vrow = dtuple_create_with_vcol(tuple_heap, 0, num_v);
+	dtuple_init_v_fld(*vrow);
+
+	for (i = 0; i < num_v; i++) {
+		dfield_get_type(dtuple_get_nth_v_field(*vrow, i))->mtype
+			 = DATA_MISSING;
+		compare[i] = false;
+	}
+
+	version = rec;
+
+	while (n_cmp_v_col < n_fields - n_non_v_col) {
+		heap2 = heap;
+		heap = mem_heap_create(1024);
+		roll_ptr_t	cur_roll_ptr = row_get_rec_roll_ptr(
+			version, clust_index, clust_offsets);
+
+		ut_ad(cur_roll_ptr != 0);
+		ut_ad(roll_ptr != 0);
+
+		trx_undo_prev_version_build(
+			version, clust_index, clust_offsets,
+			heap, &prev_version, mtr,
+			TRX_UNDO_PREV_IN_PURGE | TRX_UNDO_GET_OLD_V_VALUE,
+			nullptr, vrow);
+
+		if (heap2) {
+			mem_heap_free(heap2);
+		}
+
+		if (!prev_version) {
+			/* Versions end here */
+			goto func_exit;
+		}
+
+		clust_offsets = rec_get_offsets(prev_version, clust_index,
+						NULL,
+						clust_index->n_core_fields,
+						ULINT_UNDEFINED, &heap);
+
+		ulint	entry_len = dict_index_get_n_fields(index);
+
+		for (i = 0; i < entry_len; i++) {
+			const dict_field_t*	ind_field
+				 = dict_index_get_nth_field(index, i);
+			const dict_col_t*	col = ind_field->col;
+			field1 = dtuple_get_nth_field(ientry, i);
+
+			if (!col->is_virtual()) {
+				continue;
+			}
+
+			const dict_v_col_t*     v_col
+                                = reinterpret_cast<const dict_v_col_t*>(col);
+			field2
+				= dtuple_get_nth_v_field(*vrow, v_col->v_pos);
+
+			if ((dfield_get_type(field2)->mtype != DATA_MISSING)
+			    && (!compare[v_col->v_pos])) {
+
+				if (ind_field->prefix_len != 0
+				    && !dfield_is_null(field2)) {
+					field2->len = unsigned(
+						dtype_get_at_most_n_mbchars(
+							field2->type.prtype,
+							field2->type.mbminlen,
+							field2->type.mbmaxlen,
+							ind_field->prefix_len,
+							field2->len,
+							static_cast<char*>
+							(field2->data)));
+				}
+
+				/* The index field mismatch */
+				if (cmp_dfield_dfield(field2, field1)) {
+					mem_heap_free(tuple_heap);
+					mem_heap_free(heap);
+					return(false);
+				}
+
+				compare[v_col->v_pos] = true;
+				n_cmp_v_col++;
+			}
+		}
+
+		trx_id_t	rec_trx_id = row_get_rec_trx_id(
+			prev_version, clust_index, clust_offsets);
+
+		if (rec_trx_id < trx_id || roll_ptr == cur_roll_ptr) {
+			break;
+		}
+
+		version = prev_version;
+	}
+
+func_exit:
+	if (n_cmp_v_col == 0) {
+		*vrow = NULL;
+	}
+
+	mem_heap_free(tuple_heap);
+	mem_heap_free(heap);
+
+	/* FIXME: In the case of n_cmp_v_col is not the same as
+	n_fields - n_non_v_col, callback is needed to compare the rest
+	columns. At the timebeing, we will need to return true */
+	return (true);
+}
+
+/** @return whether two data tuples are equal */
+bool dtuple_coll_eq(const dtuple_t &tuple1, const dtuple_t &tuple2)
+{
+  ut_ad(tuple1.magic_n == DATA_TUPLE_MAGIC_N);
+  ut_ad(tuple2.magic_n == DATA_TUPLE_MAGIC_N);
+  ut_ad(dtuple_check_typed(&tuple1));
+  ut_ad(dtuple_check_typed(&tuple2));
+  ut_ad(tuple1.n_fields == tuple2.n_fields);
+
+  for (ulint i= 0; i < tuple1.n_fields; i++)
+    if (cmp_dfield_dfield(&tuple1.fields[i], &tuple2.fields[i]))
+      return false;
+  return true;
+}
+
+/** Finds out if a version of the record, where the version >= the current
+purge_sys.view, should have ientry as its secondary index entry. We check
+if there is any not delete marked version of the record where the trx
+id >= purge view, and the secondary index entry == ientry; exactly in
+this case we return TRUE.
+@param node    purge node
+@param index   secondary index
+@param ientry  secondary index entry
+@param mtr     mini-transaction
+@return whether ientry cannot be purged */
+static bool row_purge_is_unsafe(const purge_node_t &node,
+                                dict_index_t *index,
+                                const dtuple_t *ientry, mtr_t *mtr)
+{
+	const rec_t*	rec = btr_pcur_get_rec(&node.pcur);
+	roll_ptr_t	roll_ptr = node.roll_ptr;
+	trx_id_t	trx_id = node.trx_id;
+	const rec_t*	version;
+	rec_t*		prev_version;
+	dict_index_t*	clust_index = node.pcur.index();
+	rec_offs*	clust_offsets;
+	mem_heap_t*	heap;
+	dtuple_t*	row;
+	const dtuple_t*	entry;
+	dtuple_t*	vrow = NULL;
+	mem_heap_t*	v_heap = NULL;
+	dtuple_t*	cur_vrow = NULL;
+
+	ut_ad(index->table == clust_index->table);
+	heap = mem_heap_create(1024);
+	clust_offsets = rec_get_offsets(rec, clust_index, NULL,
+					clust_index->n_core_fields,
+					ULINT_UNDEFINED, &heap);
+
+	if (dict_index_has_virtual(index)) {
+		v_heap = mem_heap_create(100);
+	}
+
+	if (!rec_get_deleted_flag(rec, rec_offs_comp(clust_offsets))) {
+		row_ext_t*	ext;
+
+		/* The top of the stack of versions is locked by the
+		mtr holding a latch on the page containing the
+		clustered index record. The bottom of the stack is
+		locked by the fact that the purge_sys.view must
+		'overtake' any read view of an active transaction.
+		Thus, it is safe to fetch the prefixes for
+		externally stored columns. */
+		row = row_build(ROW_COPY_POINTERS, clust_index,
+				rec, clust_offsets,
+				NULL, NULL, NULL, &ext, heap);
+
+		if (dict_index_has_virtual(index)) {
+
+
+#ifdef DBUG_OFF
+# define dbug_v_purge false
+#else /* DBUG_OFF */
+                        bool    dbug_v_purge = false;
+#endif /* DBUG_OFF */
+
+			DBUG_EXECUTE_IF(
+				"ib_purge_virtual_index_callback",
+				dbug_v_purge = true;);
+
+			roll_ptr_t t_roll_ptr = row_get_rec_roll_ptr(
+				rec, clust_index, clust_offsets);
+
+			/* if the row is newly inserted, then the virtual
+			columns need to be computed */
+			if (trx_undo_roll_ptr_is_insert(t_roll_ptr)
+			    || dbug_v_purge) {
+
+				if (!row_vers_build_clust_v_col(
+					    row, clust_index, index, heap)) {
+					goto unsafe_to_purge;
+				}
+
+				entry = row_build_index_entry(
+					row, ext, index, heap);
+				if (entry && dtuple_coll_eq(*ientry, *entry)) {
+					goto unsafe_to_purge;
+				}
+			} else {
+				/* Build index entry out of row */
+				entry = row_build_index_entry(row, ext, index, heap);
+				/* entry could only be NULL if
+				the clustered index record is an uncommitted
+				inserted record whose BLOBs have not been
+				written yet. The secondary index record
+				can be safely removed, because it cannot
+				possibly refer to this incomplete
+				clustered index record. (Insert would
+				always first be completed for the
+				clustered index record, then proceed to
+				secondary indexes.) */
+
+				if (entry && row_purge_vc_matches_cluster(
+					    rec, entry,
+					    clust_index, clust_offsets,
+					    index, ientry, roll_ptr,
+					    trx_id, mtr, &vrow)) {
+					goto unsafe_to_purge;
+				}
+			}
+			clust_offsets = rec_get_offsets(rec, clust_index, NULL,
+							clust_index
+							->n_core_fields,
+							ULINT_UNDEFINED, &heap);
+		} else {
+
+			entry = row_build_index_entry(
+				row, ext, index, heap);
+
+			/* If entry == NULL, the record contains unset BLOB
+			pointers.  This must be a freshly inserted record.  If
+			this is called from
+			row_purge_remove_sec_if_poss_low(), the thread will
+			hold latches on the clustered index and the secondary
+			index.  Because the insert works in three steps:
+
+				(1) insert the record to clustered index
+				(2) store the BLOBs and update BLOB pointers
+				(3) insert records to secondary indexes
+
+			the purge thread can safely ignore freshly inserted
+			records and delete the secondary index record.  The
+			thread that inserted the new record will be inserting
+			the secondary index records. */
+
+			/* NOTE that we cannot do the comparison as binary
+			fields because the row is maybe being modified so that
+			the clustered index record has already been updated to
+			a different binary value in a char field, but the
+			collation identifies the old and new value anyway! */
+			if (entry && dtuple_coll_eq(*ientry, *entry)) {
+unsafe_to_purge:
+				mem_heap_free(heap);
+
+				if (v_heap) {
+					mem_heap_free(v_heap);
+				}
+				return true;
+			}
+		}
+	} else if (dict_index_has_virtual(index)) {
+		/* The current cluster index record could be
+		deleted, but the previous version of it might not. We will
+		need to get the virtual column data from undo record
+		associated with current cluster index */
+
+		cur_vrow = row_vers_build_cur_vrow(
+			rec, clust_index, &clust_offsets,
+			index, trx_id, roll_ptr, heap, v_heap, mtr);
+	}
+
+	version = rec;
+
+	for (;;) {
+		mem_heap_t* heap2 = heap;
+		heap = mem_heap_create(1024);
+		vrow = NULL;
+
+		trx_undo_prev_version_build(version,
+					    clust_index, clust_offsets,
+					    heap, &prev_version, mtr,
+					    TRX_UNDO_CHECK_PURGE_PAGES,
+					    nullptr,
+					    dict_index_has_virtual(index)
+					    ? &vrow : nullptr);
+		mem_heap_free(heap2); /* free version and clust_offsets */
+
+		if (!prev_version) {
+			/* Versions end here */
+			mem_heap_free(heap);
+
+			if (v_heap) {
+				mem_heap_free(v_heap);
+			}
+
+			return false;
+		}
+
+		clust_offsets = rec_get_offsets(prev_version, clust_index,
+						NULL,
+						clust_index->n_core_fields,
+						ULINT_UNDEFINED, &heap);
+
+		if (dict_index_has_virtual(index)) {
+			if (vrow) {
+				if (dtuple_vcol_data_missing(*vrow, *index)) {
+					goto nochange_index;
+				}
+				/* Keep the virtual row info for the next
+				version, unless it is changed */
+				mem_heap_empty(v_heap);
+				cur_vrow = dtuple_copy(vrow, v_heap);
+				dtuple_dup_v_fld(cur_vrow, v_heap);
+			}
+
+			if (!cur_vrow) {
+				/* Nothing for this index has changed,
+				continue */
+nochange_index:
+				version = prev_version;
+				continue;
+			}
+		}
+
+		if (!rec_get_deleted_flag(prev_version,
+					  rec_offs_comp(clust_offsets))) {
+			row_ext_t*	ext;
+
+			/* The stack of versions is locked by mtr.
+			Thus, it is safe to fetch the prefixes for
+			externally stored columns. */
+			row = row_build(ROW_COPY_POINTERS, clust_index,
+					prev_version, clust_offsets,
+					NULL, NULL, NULL, &ext, heap);
+
+			if (dict_index_has_virtual(index)) {
+				ut_ad(cur_vrow);
+				ut_ad(row->n_v_fields == cur_vrow->n_v_fields);
+				dtuple_copy_v_fields(row, cur_vrow);
+			}
+
+			entry = row_build_index_entry(row, ext, index, heap);
+
+			/* If entry == NULL, the record contains unset
+			BLOB pointers.  This must be a freshly
+			inserted record that we can safely ignore.
+			For the justification, see the comments after
+			the previous row_build_index_entry() call. */
+
+			/* NOTE that we cannot do the comparison as binary
+			fields because maybe the secondary index record has
+			already been updated to a different binary value in
+			a char field, but the collation identifies the old
+			and new value anyway! */
+
+			if (entry && dtuple_coll_eq(*ientry, *entry)) {
+				goto unsafe_to_purge;
+			}
+		}
+
+		version = prev_version;
+	}
+}
+
 /** Determines if it is possible to remove a secondary index entry.
 Removal is possible if the secondary index entry does not refer to any
 not delete marked version of a clustered index record where DB_TRX_ID
@@ -280,71 +733,75 @@ would refer to.
 However, in that case, the user transaction would also re-insert the
 secondary index entry after purge has removed it and released the leaf
 page latch.
-@param[in,out]	node		row purge node
-@param[in]	index		secondary index
-@param[in]	entry		secondary index entry
-@param[in,out]	sec_pcur	secondary index cursor or NULL
-				if it is called for purge buffering
-				operation.
-@param[in,out]	sec_mtr		mini-transaction which holds
-				secondary index entry or NULL if it is
-				called for purge buffering operation.
-@param[in]	is_tree		true=pessimistic purge,
-				false=optimistic (leaf-page only)
-@return true if the secondary index record can be purged */
-static
-bool
-row_purge_poss_sec(
-	purge_node_t*	node,
-	dict_index_t*	index,
-	const dtuple_t*	entry,
-	btr_pcur_t*	sec_pcur,
-	mtr_t*		sec_mtr,
-	bool		is_tree)
+@param node   row purge node
+@param index  secondary index
+@param entry  secondary index entry
+@param mtr    mini-transaction for looking up clustered index
+@return whether the secondary index record can be purged */
+static bool row_purge_poss_sec(purge_node_t *node, dict_index_t *index,
+			       const dtuple_t *entry, mtr_t *mtr)
 {
-	bool	can_delete;
-	mtr_t	mtr;
+  ut_ad(!index->is_clust());
+  const auto savepoint= mtr->get_savepoint();
+  bool can_delete= !row_purge_reposition_pcur(BTR_SEARCH_LEAF, node, mtr);
 
-	ut_ad(!dict_index_is_clust(index));
+  if (!can_delete)
+  {
+    ut_ad(node->pcur.pos_state == BTR_PCUR_IS_POSITIONED);
+    can_delete= !row_purge_is_unsafe(*node, index, entry, mtr);
+    node->pcur.pos_state = BTR_PCUR_WAS_POSITIONED;
+    node->pcur.latch_mode= BTR_NO_LATCHES;
+  }
 
-	mtr_start(&mtr);
-
-	can_delete = !row_purge_reposition_pcur(BTR_SEARCH_LEAF, node, &mtr)
-		|| !row_vers_old_has_index_entry(true,
-						 btr_pcur_get_rec(&node->pcur),
-						 &mtr, index, entry,
-						 node->roll_ptr, node->trx_id);
-
-	/* Persistent cursor is closed if reposition fails. */
-	if (node->found_clust) {
-		btr_pcur_commit_specify_mtr(&node->pcur, &mtr);
-	} else {
-		mtr.commit();
-	}
-
-	ut_ad(mtr.has_committed());
-
-	return can_delete;
+  mtr->rollback_to_savepoint(savepoint);
+  return can_delete;
 }
 
-/***************************************************************
-Removes a secondary index entry if possible, by modifying the
-index tree.  Does not try to buffer the delete.
-@return TRUE if success or if not found */
-static MY_ATTRIBUTE((nonnull, warn_unused_result))
-ibool
-row_purge_remove_sec_if_poss_tree(
-/*==============================*/
-	purge_node_t*	node,	/*!< in: row purge node */
-	dict_index_t*	index,	/*!< in: index */
-	const dtuple_t*	entry)	/*!< in: index entry */
+/** Report an error about not delete-marked secondary index record
+that was about to be purged.
+@param cur   cursor on the secondary index record
+@param entry search key */
+ATTRIBUTE_COLD ATTRIBUTE_NOINLINE
+static void row_purge_del_mark_error(const btr_cur_t &cursor,
+                                     const dtuple_t &entry)
+{
+  const dict_index_t *index= cursor.index();
+  ib::error() << "tried to purge non-delete-marked record in index "
+              << index->name << " of table " << index->table->name
+              << ": tuple: " << entry
+              << ", record: " << rec_index_print(cursor.page_cur.rec, index);
+  ut_ad(0);
+}
+
+__attribute__((nonnull, warn_unused_result))
+/** Remove a secondary index entry if possible, by modifying the index tree.
+@param node             purge node
+@param index            secondary index
+@param entry            index entry
+@param page_max_trx_id  the PAGE_MAX_TRX_ID
+                        when row_purge_remove_sec_if_poss_leaf() was invoked
+@return whether the operation succeeded */
+static bool row_purge_remove_sec_if_poss_tree(purge_node_t *node,
+					      dict_index_t *index,
+					      const dtuple_t *entry,
+					      trx_id_t page_max_trx_id)
 {
 	btr_pcur_t		pcur;
-	ibool			success	= TRUE;
+	bool			success	= true;
 	dberr_t			err;
 	mtr_t			mtr;
 
 	log_free_check();
+#ifdef ENABLED_DEBUG_SYNC
+	DBUG_EXECUTE_IF("enable_row_purge_sec_tree_sync",
+		debug_sync_set_action(current_thd, STRING_WITH_LEN(
+			"now SIGNAL "
+			"purge_sec_tree_begin"));
+		debug_sync_set_action(current_thd, STRING_WITH_LEN(
+			"now WAIT_FOR "
+			"purge_sec_tree_execute"));
+	);
+#endif
 	mtr.start();
 	index->set_modified(mtr);
 	pcur.btr_cur.page_cur.index = index;
@@ -371,30 +828,19 @@ row_purge_remove_sec_if_poss_tree(
 	which cannot be purged yet, requires its existence. If some requires,
 	we should do nothing. */
 
-	if (row_purge_poss_sec(node, index, entry, &pcur, &mtr, true)) {
+	if (page_max_trx_id
+	    == page_get_max_trx_id(btr_cur_get_page(&pcur.btr_cur))
+	    || row_purge_poss_sec(node, index, entry, &mtr)) {
 
 		/* Remove the index record, which should have been
 		marked for deletion. */
-		if (!rec_get_deleted_flag(btr_cur_get_rec(
-						btr_pcur_get_btr_cur(&pcur)),
-					  dict_table_is_comp(index->table))) {
-			ib::error()
-				<< "tried to purge non-delete-marked record"
-				" in index " << index->name
-				<< " of table " << index->table->name
-				<< ": tuple: " << *entry
-				<< ", record: " << rec_index_print(
-					btr_cur_get_rec(
-						btr_pcur_get_btr_cur(&pcur)),
-					index);
-
-			ut_ad(0);
-
+		if (!rec_get_deleted_flag(btr_pcur_get_rec(&pcur),
+					  index->table->not_redundant())) {
+			row_purge_del_mark_error(pcur.btr_cur, *entry);
 			goto func_exit;
 		}
 
-		btr_cur_pessimistic_delete(&err, FALSE,
-					   btr_pcur_get_btr_cur(&pcur),
+		btr_cur_pessimistic_delete(&err, FALSE, &pcur.btr_cur,
 					   0, false, &mtr);
 		switch (UNIV_EXPECT(err, DB_SUCCESS)) {
 		case DB_SUCCESS:
@@ -410,26 +856,45 @@ row_purge_remove_sec_if_poss_tree(
 func_exit:
 	btr_pcur_close(&pcur); // FIXME: need this?
 	mtr.commit();
-
-	return(success);
+	return success;
 }
 
-/***************************************************************
-Removes a secondary index entry without modifying the index tree,
-if possible.
-@retval true if success or if not found
-@retval false if row_purge_remove_sec_if_poss_tree() should be invoked */
-static MY_ATTRIBUTE((nonnull, warn_unused_result))
-bool
-row_purge_remove_sec_if_poss_leaf(
-/*==============================*/
-	purge_node_t*	node,	/*!< in: row purge node */
-	dict_index_t*	index,	/*!< in: index */
-	const dtuple_t*	entry)	/*!< in: index entry */
+/** Compute a nonzero return value of row_purge_remove_sec_if_poss_leaf().
+@param page  latched secondary index page
+@return PAGE_MAX_TRX_ID for row_purge_remove_sec_if_poss_tree()
+@retval 1 if a further row_purge_poss_sec() check is necessary */
+ATTRIBUTE_NOINLINE ATTRIBUTE_COLD
+static trx_id_t row_purge_check(const page_t *page) noexcept
+{
+  trx_id_t id= page_get_max_trx_id(page);
+  ut_ad(id);
+  if (trx_sys.find_same_or_older_in_purge(purge_sys.query->trx, id))
+    /* Because an active transaction may modify the secondary index
+    but not PAGE_MAX_TRX_ID, row_purge_poss_sec() must be invoked
+    again after re-latching the page. Let us return a bogus ID. Yes,
+    an actual transaction with ID 1 would create the InnoDB dictionary
+    tables in dict_sys_t::create_or_check_sys_tables(), but it would
+    exclusively write TRX_UNDO_INSERT_REC records. Purging those
+    records never involves row_purge_remove_sec_if_poss_tree(). */
+    id= 1;
+  return id;
+}
+
+__attribute__((nonnull, warn_unused_result))
+/** Remove a secondary index entry if possible, without modifying the tree.
+@param node             purge node
+@param index            secondary index
+@param entry            index entry
+@return PAGE_MAX_TRX_ID for row_purge_remove_sec_if_poss_tree()
+@retval 1 if a further row_purge_poss_sec() check is necessary
+@retval 0 if success or if not found */
+static trx_id_t row_purge_remove_sec_if_poss_leaf(purge_node_t *node,
+                                                  dict_index_t *index,
+                                                  const dtuple_t *entry)
 {
 	mtr_t			mtr;
 	btr_pcur_t		pcur;
-	bool			success	= true;
+	trx_id_t		page_max_trx_id = 0;
 
 	log_free_check();
 	ut_ad(index->table == node->table);
@@ -453,31 +918,20 @@ row_purge_remove_sec_if_poss_leaf(
 found:
 		/* Before attempting to purge a record, check
 		if it is safe to do so. */
-		if (row_purge_poss_sec(node, index, entry, &pcur, &mtr, false)) {
-			btr_cur_t* btr_cur = btr_pcur_get_btr_cur(&pcur);
-
+		if (row_purge_poss_sec(node, index, entry, &mtr)) {
 			/* Only delete-marked records should be purged. */
-			if (!rec_get_deleted_flag(
-				btr_cur_get_rec(btr_cur),
-				dict_table_is_comp(index->table))) {
-
-				ib::error()
-					<< "tried to purge non-delete-marked"
-					" record" " in index " << index->name
-					<< " of table " << index->table->name
-					<< ": tuple: " << *entry
-					<< ", record: "
-					<< rec_index_print(
-						btr_cur_get_rec(btr_cur),
-						index);
+			if (!rec_get_deleted_flag(btr_pcur_get_rec(&pcur),
+						  index->table
+						  ->not_redundant())) {
+				row_purge_del_mark_error(pcur.btr_cur, *entry);
 				mtr.commit();
 				dict_set_corrupted(index, "purge");
 				goto cleanup;
 			}
 
 			if (index->is_spatial()) {
-				const buf_block_t* block = btr_cur_get_block(
-					btr_cur);
+				const buf_block_t* block = btr_pcur_get_block(
+					&pcur);
                                 const page_id_t id{block->page.id()};
 
 				if (id.page_no() != index->page
@@ -494,8 +948,11 @@ found:
 				}
 			}
 
-			success = btr_cur_optimistic_delete(btr_cur, 0, &mtr)
-				!= DB_FAIL;
+			if (btr_cur_optimistic_delete(&pcur.btr_cur, 0, &mtr)
+			    == DB_FAIL) {
+				page_max_trx_id = row_purge_check(
+					btr_pcur_get_page(&pcur));
+			}
 		}
 	}
 
@@ -503,7 +960,7 @@ func_exit:
 	mtr.commit();
 cleanup:
 	btr_pcur_close(&pcur);
-	return success;
+	return page_max_trx_id;
 }
 
 /***********************************************************//**
@@ -516,38 +973,21 @@ row_purge_remove_sec_if_poss(
 	dict_index_t*	index,	/*!< in: index */
 	const dtuple_t*	entry)	/*!< in: index entry */
 {
-	ibool	success;
-	ulint	n_tries		= 0;
+  if (UNIV_UNLIKELY(!entry))
+    /* The node->row must have lacked some fields of this index. This
+    is possible when the undo log record was written before this index
+    was created. */
+    return;
 
-	/*	fputs("Purge: Removing secondary record\n", stderr); */
-
-	if (!entry) {
-		/* The node->row must have lacked some fields of this
-		index. This is possible when the undo log record was
-		written before this index was created. */
-		return;
-	}
-
-	if (row_purge_remove_sec_if_poss_leaf(node, index, entry)) {
-
-		return;
-	}
-retry:
-	success = row_purge_remove_sec_if_poss_tree(node, index, entry);
-	/* The delete operation may fail if we have little
-	file space left: TODO: easiest to crash the database
-	and restart with more file space */
-
-	if (!success && n_tries < BTR_CUR_RETRY_DELETE_N_TIMES) {
-
-		n_tries++;
-
-		std::this_thread::sleep_for(BTR_CUR_RETRY_SLEEP_TIME);
-
-		goto retry;
-	}
-
-	ut_a(success);
+  if (trx_id_t page_max_trx_id=
+      row_purge_remove_sec_if_poss_leaf(node, index, entry))
+    for (auto n_tries= BTR_CUR_RETRY_DELETE_N_TIMES;
+         !row_purge_remove_sec_if_poss_tree(node, index, entry,
+                                            page_max_trx_id);
+         std::this_thread::sleep_for(BTR_CUR_RETRY_SLEEP_TIME))
+      /* The delete operation may fail if we have little
+      file space left (if innodb_file_per_table=0?) */
+      ut_a(--n_tries);
 }
 
 /***********************************************************//**
@@ -609,7 +1049,7 @@ static void row_purge_reset_trx_id(purge_node_t* node, mtr_t* mtr)
 		mem_heap_t*	heap = NULL;
 		/* Reserve enough offsets for the PRIMARY KEY and 2 columns
 		so that we can access DB_TRX_ID, DB_ROLL_PTR. */
-		rec_offs offsets_[REC_OFFS_HEADER_SIZE + MAX_REF_PARTS + 2];
+		rec_offs offsets_[REC_OFFS_HEADER_SIZE + MAX_REF_PARTS + 3];
 		rec_offs_init(offsets_);
 		rec_offs*	offsets = rec_get_offsets(
 			rec, index, offsets_, index->n_core_fields,
@@ -649,7 +1089,7 @@ static void row_purge_reset_trx_id(purge_node_t* node, mtr_t* mtr)
 				byte*	ptr = rec_get_nth_field(
 					rec, offsets, trx_id_pos, &len);
 				ut_ad(len == DATA_TRX_ID_LEN);
-				size_t offs = page_offset(ptr);
+				size_t offs = ptr - block->page.frame;
 				mtr->memset(block, offs, DATA_TRX_ID_LEN, 0);
 				offs += DATA_TRX_ID_LEN;
 				mtr->write<1,mtr_t::MAYBE_NOP>(
