@@ -47,6 +47,31 @@ trx_sys_t		trx_sys;
 #ifdef UNIV_DEBUG
 /* Flag to control TRX_RSEG_N_SLOTS behavior debugging. */
 uint	trx_rseg_n_slots_debug = 0;
+
+void rw_trx_hash_t::validate_element(trx_t *trx)
+{
+  ut_ad(!trx->read_only || !trx->rsegs.m_redo.rseg);
+  ut_ad(!trx->is_autocommit_non_locking());
+  ut_d(bool acquire_trx_mutex= !trx->mutex_is_owner());
+  ut_d(if (acquire_trx_mutex) trx->mutex_lock());
+  switch (trx->state) {
+  case TRX_STATE_NOT_STARTED:
+  case TRX_STATE_ABORTED:
+    ut_error;
+  case TRX_STATE_PREPARED:
+  case TRX_STATE_PREPARED_RECOVERED:
+  case TRX_STATE_COMMITTED_IN_MEMORY:
+    ut_ad(!trx->is_autocommit_non_locking());
+    break;
+  case TRX_STATE_ACTIVE:
+    if (!trx->is_autocommit_non_locking())
+      break;
+    ut_ad(!trx->is_recovered);
+    ut_ad(trx->read_only);
+    ut_ad(trx->mysql_thd);
+  }
+  ut_d(if (acquire_trx_mutex) trx->mutex_unlock());
+}
 #endif
 
 /** Display the MySQL binlog offset info if it is present in the trx
@@ -146,6 +171,10 @@ void trx_sys_t::create()
   m_initialised= true;
   trx_list.create();
   rw_trx_hash.init();
+  for (auto &rseg : temp_rsegs)
+    rseg.init(nullptr, FIL_NULL);
+  for (auto &rseg : rseg_array)
+    rseg.init(nullptr, FIL_NULL);
 }
 
 size_t trx_sys_t::history_size()
@@ -202,6 +231,19 @@ TPOOL_SUPPRESS_TSAN size_t trx_sys_t::history_size_approx() const
   return size;
 }
 
+my_bool trx_sys_t::find_same_or_older_callback(void *el, void *i) noexcept
+{
+  auto element= static_cast<rw_trx_hash_element_t *>(el);
+  auto id= static_cast<trx_id_t*>(i);
+  return element->id <= *id;
+}
+
+
+bool trx_sys_t::find_same_or_older_low(trx_t *trx, trx_id_t id) noexcept
+{
+  return rw_trx_hash.iterate(trx, find_same_or_older_callback, &id);
+}
+
 /** Create a persistent rollback segment.
 @param space_id   system or undo tablespace id
 @return pointer to new rollback segment
@@ -215,7 +257,8 @@ static trx_rseg_t *trx_rseg_create(uint32_t space_id)
 
   if (fil_space_t *space= mtr.x_lock_space(space_id))
   {
-    ut_ad(space->purpose == FIL_TYPE_TABLESPACE);
+    ut_ad(!space->is_temporary());
+    ut_ad(!space->is_being_imported());
     if (buf_block_t *sys_header= trx_sysf_get(&mtr))
     {
       ulint rseg_id= trx_sys_rseg_find_free(sys_header);
@@ -224,6 +267,7 @@ static trx_rseg_t *trx_rseg_create(uint32_t space_id)
           ? nullptr : trx_rseg_header_create(space, rseg_id, 0, &mtr, &err))
       {
         rseg= &trx_sys.rseg_array[rseg_id];
+        rseg->destroy();
         rseg->init(space, rblock->page.id().page_no());
         ut_ad(rseg->is_persistent());
         mtr.write<4,mtr_t::MAYBE_NOP>
@@ -322,13 +366,8 @@ trx_sys_t::close()
 	rw_trx_hash.destroy();
 
 	/* There can't be any active transactions. */
-
-	for (ulint i = 0; i < array_elements(temp_rsegs); ++i) {
-		temp_rsegs[i].destroy();
-	}
-	for (ulint i = 0; i < array_elements(rseg_array); ++i) {
-		rseg_array[i].destroy();
-	}
+	for (auto& rseg : temp_rsegs) rseg.destroy();
+	for (auto& rseg : rseg_array) rseg.destroy();
 
 	ut_a(trx_list.empty());
 	trx_list.close();
@@ -343,6 +382,7 @@ size_t trx_sys_t::any_active_transactions(size_t *prepared)
   trx_sys.trx_list.for_each([&](const trx_t &trx) {
     switch (trx.state) {
     case TRX_STATE_NOT_STARTED:
+    case TRX_STATE_ABORTED:
       break;
     case TRX_STATE_ACTIVE:
       if (!trx.id)
