@@ -1,7 +1,7 @@
 /*****************************************************************************
 
 Copyright (c) 1996, 2016, Oracle and/or its affiliates. All Rights Reserved.
-Copyright (c) 2015, 2022, MariaDB Corporation.
+Copyright (c) 2015, 2023, MariaDB Corporation.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free Software
@@ -24,9 +24,7 @@ The transaction
 Created 3/26/1996 Heikki Tuuri
 *******************************************************/
 
-#define MYSQL_SERVER
 #include "trx0trx.h"
-#include "sql_class.h" // THD
 
 #ifdef WITH_WSREP
 #include <mysql/service_wsrep.h>
@@ -136,6 +134,8 @@ trx_init(
 
 	trx->will_lock = false;
 
+	trx->bulk_insert = false;
+
 	trx->apply_online_log = false;
 
 	ut_d(trx->start_file = 0);
@@ -357,7 +357,7 @@ trx_t *trx_create()
 }
 
 /** Free the memory to trx_pools */
-void trx_t::free() noexcept
+void trx_t::free()
 {
   autoinc_locks.fake_defined();
 #ifdef HAVE_MEM_CHECK
@@ -378,10 +378,8 @@ void trx_t::free() noexcept
   ut_ad(magic_n == TRX_MAGIC_N);
   ut_ad(!read_only);
   ut_ad(!lock.wait_lock);
-  ut_ad(!commit_lsn);
 
   dict_operation= false;
-  commit_lsn= 0;
   trx_sys.deregister_trx(this);
   check_unique_secondary= true;
   check_foreigns= true;
@@ -454,7 +452,7 @@ void trx_t::free() noexcept
 /** Transition to committed state, to release implicit locks. */
 TRANSACTIONAL_INLINE inline void trx_t::commit_state()
 {
-  ut_d(auto trx_state= state);
+  ut_d(auto trx_state{state});
   ut_ad(trx_state == TRX_STATE_PREPARED ||
         trx_state == TRX_STATE_PREPARED_RECOVERED ||
         trx_state == TRX_STATE_ACTIVE);
@@ -519,7 +517,6 @@ TRANSACTIONAL_TARGET void trx_free_at_shutdown(trx_t *trx)
 	ut_a(trx->magic_n == TRX_MAGIC_N);
 
 	ut_d(trx->apply_online_log = false);
-	trx->bulk_insert = 0;
 	trx->commit_state();
 	trx->release_locks();
 	trx->mod_tables.clear();
@@ -810,7 +807,7 @@ static void trx_assign_rseg_low(trx_t *trx)
 	undo tablespaces that are scheduled for truncation. */
 	static Atomic_counter<unsigned>	rseg_slot;
 	unsigned slot = rseg_slot++ % TRX_SYS_N_RSEGS;
-	DBUG_EXECUTE_IF("assign_same_rseg", slot= 0;);
+	ut_d(if (trx_rseg_n_slots_debug) slot = 0);
 	ut_d(const auto start_scan_slot = slot);
 	ut_d(bool look_for_rollover = false);
 	trx_rseg_t*	rseg;
@@ -822,9 +819,8 @@ static void trx_assign_rseg_low(trx_t *trx)
 			rseg = &trx_sys.rseg_array[slot];
 			ut_ad(!look_for_rollover || start_scan_slot != slot);
 			ut_d(look_for_rollover = true);
+			ut_d(if (!trx_rseg_n_slots_debug))
 			slot = (slot + 1) % TRX_SYS_N_RSEGS;
-			DBUG_EXECUTE_IF("assign_same_rseg",
-				slot= (slot - 1) % TRX_SYS_N_RSEGS;);
 
 			if (!rseg->space) {
 				continue;
@@ -904,21 +900,16 @@ trx_start_low(
 	ut_ad(UT_LIST_GET_LEN(trx->lock.trx_locks) == 0);
 
 	/* Check whether it is an AUTOCOMMIT SELECT */
-        if (const THD* thd = trx->mysql_thd) {
-		trx->auto_commit = !(thd->variables.option_bits
-                                     & (OPTION_NOT_AUTOCOMMIT | OPTION_BEGIN))
-			&& thd->lex->sql_command == SQLCOM_SELECT;
-		trx->read_only = (!trx->dict_operation && thd->tx_read_only)
-			|| srv_read_only_mode;
-		if (!trx->auto_commit) {
-			trx->will_lock = true;
-		} else if (!trx->will_lock) {
-			trx->read_only = true;
-		}
-	} else {
-		trx->auto_commit = false;
-		trx->read_only = false;
+	trx->auto_commit = thd_trx_is_auto_commit(trx->mysql_thd);
+
+	trx->read_only = srv_read_only_mode
+		|| (!trx->dict_operation
+		    && thd_trx_is_read_only(trx->mysql_thd));
+
+	if (!trx->auto_commit) {
 		trx->will_lock = true;
+	} else if (!trx->will_lock) {
+		trx->read_only = true;
 	}
 
 #ifdef WITH_WSREP
@@ -1257,10 +1248,8 @@ static void trx_flush_log_if_needed(lsn_t lsn, trx_t *trx)
   if (log_sys.get_flushed_lsn(std::memory_order_relaxed) >= lsn)
     return;
 
-  ut_ad(!trx->mysql_thd || !trx->mysql_thd->tx_read_only);
-
   const bool flush=
-    (srv_file_flush_method != SRV_NOSYNC &&
+    (!my_disable_sync &&
      (srv_flush_log_at_trx_commit & 1));
   if (!log_sys.is_mmap())
   {
@@ -1524,7 +1513,6 @@ bool trx_t::commit_cleanup() noexcept
   *detailed_error= '\0';
   mod_tables.clear();
 
-  bulk_insert= TRX_NO_BULK;
   check_foreigns= true;
   check_unique_secondary= true;
   assert_freed();
@@ -1749,14 +1737,12 @@ void trx_commit_complete_for_mysql(trx_t *trx)
     return;
   switch (srv_flush_log_at_trx_commit) {
   case 0:
-    goto func_exit;
+    return;
   case 1:
     if (trx->active_commit_ordered)
       return;
   }
   trx_flush_log_if_needed(lsn, trx);
- func_exit:
-  trx->commit_lsn= 0;
 }
 
 /**********************************************************************//**
@@ -1979,8 +1965,7 @@ trx_prepare(
 
 		We must not be holding any mutexes or latches here. */
 		if (auto f = srv_flush_log_at_trx_commit) {
-			log_write_up_to(lsn, (f & 1) && srv_file_flush_method
-					!= SRV_NOSYNC);
+			log_write_up_to(lsn, (f & 1) && !my_disable_sync);
 		}
 
 		if (!UT_LIST_GET_LEN(trx->lock.trx_locks)
@@ -1988,7 +1973,7 @@ trx_prepare(
 			/* Do not release any locks at the
 			SERIALIZABLE isolation level. */
 		} else if (!trx->mysql_thd
-			   || trx->mysql_thd->lex->sql_command
+			   || thd_sql_command(trx->mysql_thd)
 			   != SQLCOM_XA_PREPARE) {
 			/* Do not release locks for XA COMMIT ONE PHASE
 			or for internal distributed transactions
