@@ -11,7 +11,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02111-1301 USA */
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1335 USA */
 
 /*
   WL#3234 Maria control file
@@ -21,6 +21,8 @@
 #ifndef _ma_control_file_h
 #define _ma_control_file_h
 
+C_MODE_START
+
 #define CONTROL_FILE_BASE_NAME "aria_log_control"
 /*
   Major version for control file. Should only be changed when doing
@@ -35,7 +37,7 @@
   LSN of the last checkoint
   (if last_checkpoint_lsn == LSN_IMPOSSIBLE then there was never a checkpoint)
 */
-extern LSN last_checkpoint_lsn;
+extern volatile LSN last_checkpoint_lsn;
 /*
   Last log number (if last_logno == FILENO_IMPOSSIBLE then there is no log
   file yet)
@@ -59,15 +61,20 @@ typedef enum enum_control_file_error {
   CONTROL_FILE_MISSING,
   CONTROL_FILE_INCONSISTENT_INFORMATION,
   CONTROL_FILE_WRONG_BLOCKSIZE,
+  CONTROL_FILE_LOCKED,
   CONTROL_FILE_UNKNOWN_ERROR /* any other error */
 } CONTROL_FILE_ERROR;
 
-C_MODE_START
+
 CONTROL_FILE_ERROR ma_control_file_open(my_bool create_if_missing,
-                                        my_bool print_error);
+                                        my_bool print_error,
+                                        my_bool wait_for_lock,
+                                        int open_flags);
 int ma_control_file_write_and_force(LSN last_checkpoint_lsn_arg,
                                     uint32 last_logno_arg, TrID max_trid_arg,
                                     uint8 recovery_failures_arg);
+/* For simple programs that creates Aria files*/
+CONTROL_FILE_ERROR ma_control_file_open_or_create();
 int ma_control_file_end(void);
 my_bool ma_control_file_inited(void);
 my_bool print_aria_log_control(void);
