@@ -54,7 +54,7 @@ int main(int argc __attribute__((unused)), char *argv[])
 
   bzero(long_tr_id, 6);
 #ifndef DBUG_OFF
-#if defined(__WIN__)
+#if defined(_WIN32)
   default_dbug_option= "d:t:i:O,\\ma_test_loghandler.trace";
 #else
   default_dbug_option= "d:t:i:o,/tmp/ma_test_loghandler.trace";
@@ -66,7 +66,7 @@ int main(int argc __attribute__((unused)), char *argv[])
   }
 #endif
 
-  if (ma_control_file_open(TRUE, TRUE, TRUE))
+  if (ma_control_file_open(TRUE, TRUE, TRUE, control_file_open_flags))
   {
     fprintf(stderr, "Can't init control file (%d)\n", errno);
     exit(1);
@@ -139,7 +139,7 @@ int main(int argc __attribute__((unused)), char *argv[])
     }
   }
 
-  if (ma_control_file_open(TRUE, TRUE, TRUE))
+  if (ma_control_file_open(TRUE, TRUE, TRUE, control_file_open_flags))
   {
     fprintf(stderr, "Can't init control file (%d)\n", errno);
     exit(1);
