@@ -1,6 +1,6 @@
 /* -*- c-basic-offset: 2 -*- */
 /*
-  Copyright(C) 2013  Kouhei Sutou <kou@clear-code.com>
+  Copyright(C) 2013-2015  Kouhei Sutou <kou@clear-code.com>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -14,7 +14,7 @@
 
   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
-  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1335  USA
 */
 
 #ifndef MRN_LOCK_HPP_
@@ -26,10 +26,11 @@
 namespace mrn {
   class Lock {
   public:
-    Lock(mysql_mutex_t *mutex);
+    Lock(mysql_mutex_t *mutex, bool execute=true);
     ~Lock();
   private:
     mysql_mutex_t *mutex_;
+    bool execute_;
   };
 }
 
