@@ -1984,7 +1984,7 @@ end:
 }
 
 int pfs_discover_table_names(handlerton *hton __attribute__((unused)),
-                             LEX_CSTRING *db,
+                             const LEX_CSTRING *db,
                              MY_DIR *dir __attribute__((unused)),
                              handlerton::discovered_list *result)
 {
