@@ -19,12 +19,19 @@
 #define SPD_INIT_ALLOC_ROOT(A, B, C, D)                                       \
   init_alloc_root(PSI_INSTRUMENT_ME, A, B, C, D)
 
+/** Maximum possible number of `SPIDER_DBTON`s available to use */
 #define SPIDER_DBTON_SIZE 15
 
 #ifndef SIZEOF_STORED_DOUBLE
 #define SIZEOF_STORED_DOUBLE 8
 #endif
 
+/**
+  Possible wrapper values, e.g. for `SPIDER_DBTON::wrapper` and
+  `SPIDER_SHARE::tgt_wrappers`.
+
+  fixme: change this to enum
+*/
 #define SPIDER_DB_WRAPPER_MYSQL "mysql"
 #define SPIDER_DB_WRAPPER_MARIADB "mariadb"
 
@@ -172,11 +179,6 @@ typedef st_spider_result SPIDER_RESULT;
 #define SPIDER_SQL_TYPE_TMP_SQL (1 << 5)
 #define SPIDER_SQL_TYPE_DROP_TMP_TABLE_SQL (1 << 6)
 #define SPIDER_SQL_TYPE_OTHER_SQL (1 << 7)
-#define SPIDER_SQL_TYPE_SELECT_HS (1 << 9)
-#define SPIDER_SQL_TYPE_INSERT_HS (1 << 10)
-#define SPIDER_SQL_TYPE_UPDATE_HS (1 << 11)
-#define SPIDER_SQL_TYPE_DELETE_HS (1 << 12)
-#define SPIDER_SQL_TYPE_OTHER_HS (1 << 13)
 
 enum spider_bulk_upd_start {
   SPD_BU_NOT_START,
@@ -501,7 +503,6 @@ typedef struct spider_table_holder SPIDER_TABLE_HOLDER;
 typedef struct spider_link_idx_holder
 {
   spider_table_link_idx_holder *table_link_idx_holder;
-  /* The index of active link */
   int link_idx;
   int link_status;
   spider_link_idx_holder *next_table;
@@ -608,7 +609,7 @@ public:
   );
   SPIDER_CONN_HOLDER *create_conn_holder();
   bool has_conn_holder();
-  void clear_conn_holder_checked();
+  void clear_conn_holder_from_conn();
   bool check_conn_same_conn(
     SPIDER_CONN *conn_arg
   );
@@ -647,6 +648,8 @@ struct st_spider_db_request_key
 class spider_db_util
 {
 public:
+  /** Same as the `SPIDER_DBTON::dbton_id` of the `SPIDER_DBTON`
+  containing this `spider_db_util` */
   uint dbton_id;
   spider_db_util() = default;
   virtual ~spider_db_util() = default;
@@ -841,6 +844,9 @@ public:
   virtual void free_result() = 0;
   virtual SPIDER_DB_ROW *current_row() = 0;
   virtual SPIDER_DB_ROW *fetch_row(MY_BITMAP *skips = NULL) = 0;
+  virtual SPIDER_DB_ROW *fetch_row_from_result_buffer(
+    spider_db_result_buffer *spider_res_buf
+  ) = 0;
   virtual SPIDER_DB_ROW *fetch_row_from_tmp_table(
     TABLE *tmp_table
   ) = 0;
@@ -884,6 +890,7 @@ public:
     SPIDER_SHARE *spider_share,
     CHARSET_INFO *access_charset
   ) = 0;
+  virtual uint limit_mode();
 };
 
 class spider_db_conn
@@ -1637,7 +1644,10 @@ static const LEX_CSTRING maturity_name[] =
 */
 typedef struct st_spider_dbton
 {
+  /** The index of this dbton in `spider_dbton` */
   uint dbton_id;
+  /** The wrapper of this dbton, same possible values as each element
+  of `SPIDER_SHARE::tgt_wrappers` */
   const char *wrapper;
   enum spider_db_access_type db_access_type;
   int (*init)();
@@ -1685,7 +1695,7 @@ typedef struct st_spider_result
     st_spider_result   *next;
   SPIDER_POSITION      *first_position; /* for quick mode */
   int                  pos_page_size; /* for quick mode */
-  longlong             record_num;    /* number of rows */
+  longlong             record_num;
   bool                 finish_flg;
   bool                 use_position;
   bool                 first_pos_use_position;
@@ -1728,7 +1738,7 @@ typedef struct st_spider_result_list
   bool                    sorted;
   bool                    desc_flg;
   longlong                current_row_num;
-  longlong                record_num; /* number of rows */
+  longlong                record_num;
   bool                    finish_flg;
   longlong                limit_num;
   longlong                internal_offset;
