@@ -593,7 +593,8 @@ static int
 my_strnncollsp_nchars_tis620(CHARSET_INFO * cs,
                              const uchar *a, size_t a_length,
                              const uchar *b, size_t b_length,
-                             size_t nchars)
+                             size_t nchars,
+                             uint flags)
 {
   set_if_smaller(a_length, nchars);
   set_if_smaller(b_length, nchars);
@@ -933,7 +934,9 @@ static MY_CHARSET_HANDLER my_charset_handler=
     my_well_formed_char_length_8bit,
     my_copy_8bit,
     my_wc_mb_bin, /* native_to_mb */
-    my_wc_to_printable_generic
+    my_wc_to_printable_generic,
+    my_casefold_multiply_1,
+    my_casefold_multiply_1
 };
 
 
@@ -953,12 +956,10 @@ struct charset_info_st my_charset_tis620_thai_ci=
     NULL,		/* uca          */
     NULL,		/* tab_to_uni   */
     NULL,		/* tab_from_uni */
-    &my_unicase_default,/* caseinfo     */
+    NULL,               /* casefold     */
     NULL,		/* state_map    */
     NULL,		/* ident_map    */
     4,			/* strxfrm_multiply */
-    1,                  /* caseup_multiply  */
-    1,                  /* casedn_multiply  */
     1,			/* mbminlen   */
     1,			/* mbmaxlen  */
     0,			/* min_sort_char */
@@ -985,12 +986,10 @@ struct charset_info_st my_charset_tis620_bin=
     NULL,		/* uca          */
     NULL,		/* tab_to_uni   */
     NULL,		/* tab_from_uni */
-    &my_unicase_default,/* caseinfo     */
+    NULL,               /* casefold     */
     NULL,		/* state_map    */
     NULL,		/* ident_map    */
     1,			/* strxfrm_multiply */
-    1,                  /* caseup_multiply  */
-    1,                  /* casedn_multiply  */
     1,			/* mbminlen   */
     1,			/* mbmaxlen  */
     0,			/* min_sort_char */
@@ -1018,12 +1017,10 @@ struct charset_info_st my_charset_tis620_thai_nopad_ci=
     NULL,                  /* uca              */
     NULL,                  /* tab_to_uni       */
     NULL,                  /* tab_from_uni     */
-    &my_unicase_default,   /* caseinfo         */
+    NULL,                  /* casefold         */
     NULL,                  /* state_map        */
     NULL,                  /* ident_map        */
     4,                     /* strxfrm_multiply */
-    1,                     /* caseup_multiply  */
-    1,                     /* casedn_multiply  */
     1,                     /* mbminlen         */
     1,                     /* mbmaxlen         */
     0,                     /* min_sort_char    */
@@ -1051,12 +1048,10 @@ struct charset_info_st my_charset_tis620_nopad_bin=
     NULL,                  /* uca              */
     NULL,                  /* tab_to_uni       */
     NULL,                  /* tab_from_uni     */
-    &my_unicase_default,   /* caseinfo         */
+    NULL,                  /* casefold         */
     NULL,                  /* state_map        */
     NULL,                  /* ident_map        */
     1,                     /* strxfrm_multiply */
-    1,                     /* caseup_multiply  */
-    1,                     /* casedn_multiply  */
     1,                     /* mbminlen         */
     1,                     /* mbmaxlen         */
     0,                     /* min_sort_char    */
