@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2018-2025 Codership Oy <info@codership.com>
+ * Copyright (C) 2018 Codership Oy <info@codership.com>
  *
  * This file is part of wsrep-lib.
  *
@@ -29,7 +29,6 @@
 
 #include <cstring>
 
-#include <memory>
 #include <string>
 #include <vector>
 #include <ostream>
@@ -48,8 +47,7 @@ namespace wsrep
     class tls_service;
     class allowlist_service;
     class event_service;
-    class client_service;
-    class connection_monitor_service;
+
     class stid
     {
     public:
@@ -285,6 +283,7 @@ namespace wsrep
             static const int streaming = (1 << 15);
             static const int snapshot = (1 << 16);
             static const int nbo = (1 << 17);
+
             /** decipher capability bitmask */
             static std::string str(int);
         };
@@ -376,7 +375,6 @@ namespace wsrep
          */
         virtual enum status bf_abort(wsrep::seqno bf_seqno,
                                      wsrep::transaction_id victim_trx,
-                                     wsrep::client_service& victim_ctx,
                                      wsrep::seqno& victim_seqno) = 0;
         virtual enum status rollback(wsrep::transaction_id) = 0;
         virtual enum status commit_order_enter(const wsrep::ws_handle&,
@@ -409,7 +407,6 @@ namespace wsrep
          * Leave total order isolation critical section
          */
         virtual enum status leave_toi(wsrep::client_id,
-                                      const wsrep::ws_meta& ws_meta,
                                       const wsrep::mutable_buffer& err) = 0;
 
         /**
@@ -480,7 +477,6 @@ namespace wsrep
             wsrep::tls_service* tls_service;
             wsrep::allowlist_service* allowlist_service;
             wsrep::event_service* event_service;
-            wsrep::connection_monitor_service* connection_monitor_service;
 
             // some GCC and clang versions don't support C++11 default
             // initializers fully, so we need to use explicit constructors
@@ -492,20 +488,17 @@ namespace wsrep
                 , tls_service()
                 , allowlist_service()
                 , event_service()
-                , connection_monitor_service()
             {
             }
 
             services(wsrep::thread_service* thr,
                      wsrep::tls_service*    tls,
                      wsrep::allowlist_service* all,
-                     wsrep::event_service*  event,
-                     wsrep::connection_monitor_service* con)
+                     wsrep::event_service*  event)
                 : thread_service(thr)
                 , tls_service(tls)
                 , allowlist_service(all)
                 , event_service(event)
-                , connection_monitor_service(con)
             {
             }
         };
@@ -516,12 +509,11 @@ namespace wsrep
          * @param provider_options Initial options to provider
          * @param thread_service Optional thread service implementation.
          */
-        static std::unique_ptr<provider> make_provider(
-            wsrep::server_state&,
-            const std::string& provider_spec,
-            const std::string& provider_options,
-            const wsrep::provider::services& services
-            = wsrep::provider::services());
+        static provider* make_provider(wsrep::server_state&,
+                                       const std::string& provider_spec,
+                                       const std::string& provider_options,
+                                       const wsrep::provider::services& services
+                                       = wsrep::provider::services());
 
     protected:
         wsrep::server_state& server_state_;
