@@ -68,8 +68,7 @@ void wsrep::client_state::close()
     keep_command_error_ = false;
     lock.unlock();
     if (transaction_.active() &&
-        (mode_ != m_local ||
-         transaction_.state() != wsrep::transaction::s_prepared))
+        (mode_ != m_local || !client_service_.is_prepared_xa()))
     {
         client_service_.bf_rollback();
         transaction_.after_statement();
@@ -366,12 +365,12 @@ int wsrep::client_state::next_fragment(const wsrep::ws_meta& meta)
     return transaction_.next_fragment(meta);
 }
 
-int wsrep::client_state::before_prepare()
+int wsrep::client_state::before_prepare(const wsrep::provider::seq_cb_t* seq_cb)
 {
     wsrep::unique_lock<wsrep::mutex> lock(mutex_);
     assert(owning_thread_id_ == wsrep::this_thread::get_id());
     assert(state_ == s_exec);
-    return transaction_.before_prepare(lock);
+    return transaction_.before_prepare(lock, seq_cb);
 }
 
 int wsrep::client_state::after_prepare()
@@ -382,11 +381,11 @@ int wsrep::client_state::after_prepare()
     return transaction_.after_prepare(lock);
 }
 
-int wsrep::client_state::before_commit()
+int wsrep::client_state::before_commit(const wsrep::provider::seq_cb_t* seq_cb)
 {
     assert(owning_thread_id_ == wsrep::this_thread::get_id());
     assert(state_ == s_exec || mode_ == m_local);
-    return transaction_.before_commit();
+    return transaction_.before_commit(seq_cb);
 }
 
 int wsrep::client_state::ordered_commit()
@@ -1039,8 +1038,9 @@ void wsrep::client_state::state(
         };
     if (!allowed[state_][state])
     {
-        wsrep::log_warning() << "client_state: Unallowed state transition: "
-                             << state_ << " -> " << state;
+        wsrep::log_warning()
+            << "client_state: Unallowed state transition: "
+            << wsrep::to_string(state_) << " -> " << wsrep::to_string(state);
         assert(0);
     }
     state_hist_.push_back(state_);
